@@ -163,7 +163,7 @@ let mcode fn (s,info,mc,pos) =
 		    | Ast.Indent s -> s
 		    | Ast.Space s -> s in
 		  print_string str line col; Some line
-	      |	Some lb when line =|= lb ->
+	      |	Some lb when line = lb ->
 		  print_string (get_string_info str) line col; Some line
 	      |	_ ->
 		  force_newline();
@@ -179,7 +179,7 @@ let mcode fn (s,info,mc,pos) =
       let line_before = print_comments None info.Ast.strbef in
       (match line_before with
 	None -> ()
-      |	Some lb when lb =|= info.Ast.line -> ()
+      |	Some lb when lb = info.Ast.line -> ()
       |	_ -> force_newline());
       fn s line lcol;
       let _ = print_comments (Some info.Ast.line) info.Ast.straft in
@@ -239,32 +239,22 @@ let handle_metavar name fn =
       pr_barrier line rcol
 in
 (* --------------------------------------------------------------------- *)
-let dots between fn d =
-  match Ast.unwrap d with
-    Ast.DOTS(l) -> param_print_between between fn l
-  | Ast.CIRCLES(l) -> param_print_between between fn l
-  | Ast.STARS(l) -> param_print_between between fn l
-in
+let dots between fn d = param_print_between between fn (Ast.unwrap d) in
 
 let dots_before_and_after before fn d =
-  match Ast.unwrap d with
-    Ast.DOTS(l) -> param_print_before_and_after before fn l
-  | Ast.CIRCLES(l) -> param_print_before_and_after before fn l
-  | Ast.STARS(l) -> param_print_before_and_after before fn l
-in
+  param_print_before_and_after before fn (Ast.unwrap d) in
 
 let nest_dots starter ender fn f d =
   mcode print_string starter;
   f(); start_block();
-  (match Ast.unwrap d with
-    Ast.DOTS(l) | Ast.CIRCLES(l) | Ast.STARS(l) ->
-      print_between force_newline fn l; end_block l);
+  let l = Ast.unwrap d in
+  print_between force_newline fn l; end_block l;
   mcode print_string ender
 in
 
-let print_disj_list fn l =
+let print_disj_list fn l sep =
   print_text "\n(\n";
-  print_between (function _ -> print_text "\n|\n") fn l;
+  print_between (function _ -> print_text ("\n"^sep^"\n")) fn l;
   print_text "\n)\n" in
 
 (* --------------------------------------------------------------------- *)
@@ -275,7 +265,7 @@ let rec ident i =
       Ast.Id(name) -> mcode print_string name
     | Ast.MetaId(name,_,_,_) ->
 	handle_metavar name (function
-			       | (Ast_c.MetaIdVal (id,_)) -> print_text id
+			       | (Ast_c.MetaIdVal id) -> print_text id
 			       | _ -> raise (Impossible 142)
 			    )
     | Ast.MetaFunc(name,_,_,_) ->
@@ -293,18 +283,16 @@ let rec ident i =
 
     | Ast.DisjId(id_list) ->
 	if generating
-	then print_disj_list ident id_list
+	then print_disj_list ident id_list "|"
 	else raise CantBeInPlus
-    | Ast.OptIdent(_) | Ast.UniqueIdent(_) ->
-	raise CantBeInPlus
-in
+    | Ast.OptIdent(_) -> raise CantBeInPlus in
 
 
 (* --------------------------------------------------------------------- *)
 (* Expression *)
 
 let rec expression e =
-  let top = 0 in 
+  let top = 0 in
   let assign = 1 in
   let cond = 2 in
   let log_or = 3 in
@@ -321,36 +309,84 @@ let rec expression e =
   let unary = 14 in
   let postfix = 15 in
   let primary = 16 in
+  let minprio = top in
+  let left_prec_of_c op =
+    let left_prec_of_arith = function
+	Ast_c.Plus | Ast_c.Minus -> addit
+      | Ast_c.Mul | Ast_c.Div | Ast_c.Mod -> mulit
+      | Ast_c.Min | Ast_c.Max -> relat
+      | Ast_c.DecLeft | Ast_c.DecRight -> shift
+      | Ast_c.And -> bit_and
+      | Ast_c.Or -> bit_or
+      | Ast_c.Xor -> bit_xor in
+    let left_prec_of_logical = function
+	Ast_c.Inf | Ast_c.Sup | Ast_c.InfEq | Ast_c.SupEq -> relat
+      | Ast_c.Eq | Ast_c.NotEq -> equal
+      | Ast_c.AndLog -> log_and
+      | Ast_c.OrLog -> log_or in
+    match Ast_c.unwrap op with
+      | Ast_c.Arith op' -> left_prec_of_arith op'
+      | Ast_c.Logical op' -> left_prec_of_logical op' in
+  let right_prec_of_c op =
+    let right_prec_of_arith = function
+	Ast_c.Plus | Ast_c.Minus -> mulit
+      | Ast_c.Mul | Ast_c.Div | Ast_c.Mod -> cast
+      | Ast_c.Min | Ast_c.Max -> shift
+      | Ast_c.DecLeft | Ast_c.DecRight -> addit
+      | Ast_c.And -> equal
+      | Ast_c.Or -> bit_xor
+      | Ast_c.Xor -> bit_and in
+    let right_prec_of_logical = function
+	Ast_c.Inf -> shift
+      | Ast_c.Sup -> shift
+      | Ast_c.InfEq -> shift
+      | Ast_c.SupEq -> shift
+      | Ast_c.Eq -> relat
+      | Ast_c.NotEq -> relat
+      | Ast_c.AndLog -> bit_or
+      | Ast_c.OrLog -> log_and in
+    match Ast_c.unwrap op with
+      Ast_c.Arith op' -> right_prec_of_arith op'
+    | Ast_c.Logical op' -> right_prec_of_logical op' in
   let left_prec_of op =
-    let left_prec_of_arith op = match Ast.unwrap_mcode op with
-      | Ast.Plus | Ast.Minus -> addit
+    let left_prec_of_arith op =
+      match Ast.unwrap_mcode op with
+	Ast.Plus | Ast.Minus -> addit
       | Ast.Mul | Ast.Div | Ast.Mod -> mulit
       | Ast.Min | Ast.Max -> relat
       | Ast.DecLeft | Ast.DecRight -> shift
       | Ast.And -> bit_and
       | Ast.Or -> bit_or
       | Ast.Xor -> bit_xor in
-    let left_prec_of_logical op = match Ast.unwrap_mcode op with	  
-      | Ast.Inf | Ast.Sup | Ast.InfEq | Ast.SupEq -> relat
+    let left_prec_of_logical op =
+      match Ast.unwrap_mcode op with
+	Ast.Inf | Ast.Sup | Ast.InfEq | Ast.SupEq -> relat
       | Ast.Eq | Ast.NotEq -> equal
       | Ast.AndLog -> log_and
       | Ast.OrLog -> log_or in
     match Ast.unwrap op with
-      | Ast.Arith op' -> left_prec_of_arith op'
-      | Ast.Logical op' -> left_prec_of_logical op'    
-      | Ast.MetaBinary (mv,_,_,_)->
-        failwith "Here, see to which operator the metavariable has been bound and return thepriority of that operator." in
-  let right_prec_of op =
-    let right_prec_of_arith op = match Ast.unwrap_mcode op with
-      | Ast.Plus | Ast.Minus -> mulit
+      Ast.Arith op' -> left_prec_of_arith op'
+    | Ast.Logical op' -> left_prec_of_logical op'
+    | Ast.MetaBinary (mv,_,_,_) ->
+	let (res,name_string,line,lcol,rcol) = lookup_metavar mv in
+	(match res with
+	  None ->
+	    if generating then minprio else failwith "unbound MetaBinary"
+	| Some (Ast_c.MetaBinaryOpVal bop) -> left_prec_of_c bop
+	| Some _ -> failwith "bad MetaBinary value") in
+  let rec right_prec_of op =
+    let right_prec_of_arith op =
+      match Ast.unwrap_mcode op with
+	Ast.Plus | Ast.Minus -> mulit
       | Ast.Mul | Ast.Div | Ast.Mod -> cast
       | Ast.Min | Ast.Max -> shift
       | Ast.DecLeft | Ast.DecRight -> addit
       | Ast.And -> equal
       | Ast.Or -> bit_xor
       | Ast.Xor -> bit_and in
-    let right_prec_of_logical op = match Ast.unwrap_mcode op with	  
-      | Ast.Inf -> shift
+    let right_prec_of_logical op =
+      match Ast.unwrap_mcode op with
+	Ast.Inf -> shift
       | Ast.Sup -> shift
       | Ast.InfEq -> shift
       | Ast.SupEq -> shift
@@ -359,10 +395,15 @@ let rec expression e =
       | Ast.AndLog -> bit_or
       | Ast.OrLog -> log_and in
     match Ast.unwrap op with
-      | Ast.Arith op' -> right_prec_of_arith op'
-      | Ast.Logical op' -> right_prec_of_logical op'    
-      | Ast.MetaBinary (mv,_,_,_)->
-        failwith "Here, see to which operator the metavariable has been bound and return thepriority of that operator." in
+      Ast.Arith op' -> right_prec_of_arith op'
+    | Ast.Logical op' -> right_prec_of_logical op'
+    | Ast.MetaBinary (mv,_,_,_) ->
+	let (res,name_string,line,lcol,rcol) = lookup_metavar mv in
+	(match res with
+	  None ->
+	    if generating then minprio else failwith "unbound MetaBinary"
+	| Some (Ast_c.MetaBinaryOpVal bop) -> right_prec_of_c bop
+	| Some _ -> failwith "bad MetaBinary value") in
   let prec_of_c = function
     | Ast_c.Ident (ident) -> primary
     | Ast_c.Constant (c) -> primary
@@ -405,9 +446,10 @@ let rec expression e =
     | Ast_c.ParenExpr (e) -> primary
     | Ast_c.New (_, t) -> unary
     | Ast_c.Delete(t) -> unary
+    | Ast_c.Defined _ -> primary
   in
 
-  let rec loop e prec = 
+  let rec loop e prec =
   match Ast.unwrap e with
     Ast.Ident(id) -> ident id
   | Ast.Constant(const) -> mcode constant const
@@ -420,7 +462,7 @@ let rec expression e =
       dots (function _ -> ()) arg_expression args;
       mcode (print_string_with_hint EndBox) rp
   | Ast.Assignment(left,op,right,_) ->
-      loop left unary; pr_space(); assignOp op unknown unknown;
+      loop left unary; pr_space(); assignOp op;
       pr_space(); loop right assign
   | Ast.Sequence(left,op,right) ->
       loop left top; mcode print_string op;
@@ -433,7 +475,7 @@ let rec expression e =
   | Ast.Infix(exp,op) -> mcode fixOp op; loop exp unary
   | Ast.Unary(exp,op) -> mcode unaryOp op; loop exp unary
   | Ast.Binary(left,op,right) ->
-      loop left (left_prec_of op); pr_space(); binaryOp op unknown unknown; pr_space();
+      loop left (left_prec_of op); pr_space(); binaryOp op; pr_space();
       loop right (right_prec_of op)
   | Ast.Nested(left,op,right) -> failwith "nested only in minus code"
   | Ast.Paren(lp,exp,rp) ->
@@ -465,7 +507,7 @@ let rec expression e =
 
   | Ast.MetaExpr (name,_,_,_typedontcare,_formdontcare,_) ->
       handle_metavar name (function
-        | Ast_c.MetaExprVal ((((e, _), _) as exp),_) ->
+        | Ast_c.MetaExprVal ((((e, _), _) as exp),_,_) ->
 	    if prec_of_c e < prec then
 	      begin
 		print_text "(";
@@ -487,13 +529,18 @@ let rec expression e =
       )
 
   | Ast.AsExpr(expr,asexpr) -> loop expr prec
+  | Ast.AsSExpr(expr,asstm) -> loop expr prec
 
   | Ast.EComma(cm) ->
       mcode (print_string_with_hint (SpaceOrNewline (ref " ")))  cm
 
   | Ast.DisjExpr(exp_list) ->
       if generating
-      then print_disj_list expression exp_list
+      then print_disj_list expression exp_list "|"
+      else raise CantBeInPlus
+  | Ast.ConjExpr(exp_list) ->
+      if generating
+      then print_disj_list expression exp_list "&"
       else raise CantBeInPlus
   | Ast.NestExpr(starter,expr_dots,ender,Some whencode,multi)
     when generating ->
@@ -503,25 +550,19 @@ let rec expression e =
   | Ast.NestExpr(starter,expr_dots,ender,None,multi) when generating ->
       nest_dots starter ender expression (function _ -> ()) expr_dots
   | Ast.NestExpr _ -> raise CantBeInPlus
-  | Ast.Edots(dots,Some whencode)
-  | Ast.Ecircles(dots,Some whencode)
-  | Ast.Estars(dots,Some whencode) ->
+  | Ast.Edots(dots,Some whencode) ->
       if generating
       then
 	(mcode print_string dots;
 	 print_text "   when != ";
 	 expression whencode)
       else raise CantBeInPlus
-  | Ast.Edots(dots,None)
-  | Ast.Ecircles(dots,None)
-  | Ast.Estars(dots,None) ->
+  | Ast.Edots(dots,None) ->
       if generating
       then mcode print_string dots
       else raise CantBeInPlus
 
-  | Ast.OptExp(exp) | Ast.UniqueExp(exp) ->
-      raise CantBeInPlus
-  in
+  | Ast.OptExp(exp) -> raise CantBeInPlus in
   loop e top
 
 and arg_expression e =
@@ -555,7 +596,7 @@ and string_format e =
 	    pretty_print_c.Pretty_print_c.format fmt
 	| _ -> raise (Impossible 157))
 
-and  unaryOp = function
+and unaryOp = function
     Ast.GetRef -> print_string "&"
   | Ast.GetRefLabel -> print_string "&&"
   | Ast.DeRef -> print_string "*"
@@ -564,28 +605,34 @@ and  unaryOp = function
   | Ast.Tilde -> print_string "~"
   | Ast.Not -> print_string "!"
 
-and  assignOp op = match Ast.unwrap op with
-    Ast.SimpleAssign _ -> print_string "="
-  | Ast.OpAssign(aop) ->
-      (function line -> function lcol ->
-	arithOp (Ast.unwrap_mcode aop) line lcol; print_string "=" line lcol)
-  | Ast.MetaAssign(mv,_,_,_) ->
-    failwith "Here, see to which operator the metavariable has been bound and return thepriority of that operator."
-and opAssignOp op line col =
-  (arithOp op line col);
-  print_string "=" line col   
-and  fixOp = function
+and assignOp op =
+  match Ast.unwrap op with
+    Ast.SimpleAssign op -> mcode print_string op
+  | Ast.OpAssign(aop) -> mcode (arithOp true) aop
+  | Ast.MetaAssign(name,_,_,_) ->
+      handle_metavar name (function
+	  Ast_c.MetaAssignOpVal aop ->
+	    pretty_print_c.Pretty_print_c.assignOp aop
+	| _ -> raise (Impossible 159))
+
+and fixOp = function
     Ast.Dec -> print_string "--"
   | Ast.Inc -> print_string "++"
 
-and  binaryOp op = match Ast.unwrap op with
-    Ast.Arith(aop) -> arithOp (Ast.unwrap_mcode aop)
-  | Ast.Logical(lop) -> logicalOp (Ast.unwrap_mcode lop)
-  | Ast.MetaBinary(mv,_,_,_) ->
-    failwith "Here, see to which operator the metavariable has been bound and return thepriority of that operator."
+and binaryOp op =
+  match Ast.unwrap op with
+    Ast.Arith(aop) -> mcode (arithOp false) aop
+  | Ast.Logical(lop) -> mcode logicalOp lop
+  | Ast.MetaBinary(name,_,_,_) ->
+      handle_metavar name (function
+	  Ast_c.MetaBinaryOpVal bop ->
+	    pretty_print_c.Pretty_print_c.binaryOp bop
+	| _ -> raise (Impossible 160))
 
-and  arithOp = function
-   Ast.Plus -> print_string "+"
+and arithOp eq op =
+  let print_string s = if eq then print_string (s^"=") else print_string s in
+  match op with
+    Ast.Plus -> print_string "+"
   | Ast.Minus -> print_string "-"
   | Ast.Mul -> print_string "*"
   | Ast.Div -> print_string "/"
@@ -598,7 +645,7 @@ and  arithOp = function
   | Ast.Or -> print_string "|"
   | Ast.Xor -> print_string "^"
 
-and  logicalOp = function
+and logicalOp = function
     Ast.Inf -> print_string "<"
   | Ast.Sup -> print_string ">"
   | Ast.InfEq -> print_string "<="
@@ -626,11 +673,10 @@ and fullType ft =
 	Ast.Pointer(_,_) ->
 	  typeC ty; print_option_prespace (mcode const_vol) cv
       |	_ -> print_option_space (mcode const_vol) cv; typeC ty)
-      
+
   | Ast.AsType(ty, asty) -> fullType ty
   | Ast.DisjType _ -> failwith "can't be in plus"
-  | Ast.OptType(_) | Ast.UniqueType(_) ->
-      raise CantBeInPlus
+  | Ast.OptType(_) -> raise CantBeInPlus
 
 and print_function_pointer (ty,lp1,star,rp1,lp2,params,rp2) fn =
   fullType ty; mcode print_string lp1; mcode print_string star; fn();
@@ -782,7 +828,7 @@ and ft_space ty =
 	| _ -> false in
       if isptr then () else pr_space()
   | _ -> pr_space()
-	
+
 and declaration d =
   match Ast.unwrap d with
     Ast.MetaDecl(name,_,_) ->
@@ -830,11 +876,15 @@ and declaration d =
       end;
       close_box(); mcode print_string rp1;
       mcode print_string sem
-  | Ast.MacroDecl(name,lp,args,rp,sem) ->
+  | Ast.MacroDecl(stg,name,lp,args,rp,sem) ->
+      print_option (mcode storage) stg;
+      print_option (function _ -> pr_space()) stg;
       ident name; mcode print_string_box lp;
       dots (function _ -> ()) arg_expression args;
       close_box(); mcode print_string rp; mcode print_string sem
-  | Ast.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+  | Ast.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem) ->
+      print_option (mcode storage) stg;
+      print_option (function _ -> pr_space()) stg;
       ident name; mcode print_string_box lp;
       dots (function _ -> ()) arg_expression args;
       close_box(); mcode print_string rp;
@@ -846,8 +896,7 @@ and declaration d =
       fullType ty; pr_space(); typeC id;
       mcode print_string sem
   | Ast.DisjDecl(_) -> raise CantBeInPlus
-  | Ast.OptDecl(decl)  | Ast.UniqueDecl(decl) ->
-      raise CantBeInPlus
+  | Ast.OptDecl(decl) -> raise CantBeInPlus
 
 and annotated_decl d =
   match Ast.unwrap d with
@@ -872,7 +921,7 @@ and initialiser nlcomma i =
   | Ast.AsInit(init,asinit) -> initialiser nlcomma init
   | Ast.InitExpr(exp) -> expression exp
   | Ast.ArInitList(lb,initlist,rb) ->
-      (match Ast.undots initlist with
+      (match Ast.unwrap initlist with
 	[] -> mcode print_string lb; mcode print_string rb
       |	lst ->
 	  mcode print_string lb; start_block();
@@ -905,8 +954,7 @@ and initialiser nlcomma i =
       if generating
       then mcode print_string dots
       else raise CantBeInPlus
-  | Ast.OptIni(ini) | Ast.UniqueIni(ini) ->
-      raise CantBeInPlus
+  | Ast.OptIni(ini) -> raise CantBeInPlus
 
 and initialiser_list nlcomma = function
   (* awkward, because the comma is separate from the initialiser *)
@@ -948,10 +996,10 @@ and parameterTypeDef p =
 
   | Ast.PComma(cm) ->
       mcode (print_string_with_hint (SpaceOrNewline (ref " ")))  cm
-  | Ast.Pdots(dots) | Ast.Pcircles(dots) when generating ->
+  | Ast.Pdots(dots) when generating ->
       mcode print_string dots
-  | Ast.Pdots(_) | Ast.Pcircles(_) -> raise CantBeInPlus
-  | Ast.OptParam(param) | Ast.UniqueParam(param) -> raise CantBeInPlus
+  | Ast.Pdots(_) -> raise CantBeInPlus
+  | Ast.OptParam(param) -> raise CantBeInPlus
 
 and parameter_list l = dots (function _ -> ()) parameterTypeDef l
 in
@@ -1053,6 +1101,7 @@ and rule_elem arity re =
   | Ast.Exp(exp) -> pr_arity arity; expression exp
   | Ast.TopExp(exp) -> pr_arity arity; expression exp
   | Ast.Ty(ty) -> pr_arity arity; fullType ty
+  | Ast.TopId(id) -> pr_arity arity; ident id
   | Ast.TopInit(init) -> initialiser false init
   | Ast.Include(inc,s) ->
       mcode print_string inc; print_text " "; mcode inc_file s
@@ -1082,14 +1131,17 @@ and rule_elem arity re =
       raise (Impossible 155)
 
   | Ast.MetaStmt(name,_,_,_) ->
-      handle_metavar name  (function
-        | Ast_c.MetaStmtVal stm ->
+      handle_metavar name (function
+        | Ast_c.MetaStmtVal(stm,_) ->
             pretty_print_c.Pretty_print_c.statement stm
-        | _ -> raise (Impossible 156)
-                           )
+        | _ -> raise (Impossible 156))
+
   | Ast.MetaStmtList(name,_,_) ->
-      failwith
-	"MetaStmtList not supported (not even in ast_c metavars binding)"
+      handle_metavar name (function
+        | Ast_c.MetaStmtListVal(statxs,_) ->
+            pretty_print_c.Pretty_print_c.statement_seq_list statxs
+        | _ -> raise (Impossible 161))
+  | Ast.AsRe(re,asre) -> rule_elem arity re
 
 and pragmainfo pi =
   match Ast.unwrap pi with
@@ -1117,9 +1169,7 @@ and print_define_param param =
     Ast.DParam(id) -> ident id
   | Ast.DPComma(comma) -> mcode print_string comma
   | Ast.DPdots(dots) -> mcode print_string dots
-  | Ast.DPcircles(circles) -> mcode print_string circles
   | Ast.OptDParam(dp) -> print_text "?"; print_define_param dp
-  | Ast.UniqueDParam(dp) -> print_text "!"; print_define_param dp
 
 and exec_code (e : Ast_cocci.exec_code) =
   match Ast.unwrap e with
@@ -1140,7 +1190,7 @@ let indent_if_needed s f =
 		failwith
 		  (Printf.sprintf "variable %s not known on SP line %d\n"
 		     name_string line)
-	    | Some (Ast_c.MetaStmtVal stm) ->
+	    | Some (Ast_c.MetaStmtVal(stm,_)) ->
 		(match Ast_c.unwrap stm with
 		  Ast_c.Compound _ -> true
 		| _ -> false)
@@ -1160,7 +1210,7 @@ let rec statement arity s =
     Ast.Seq(lbrace,body,rbrace) ->
       rule_elem arity lbrace;
       dots force_newline (statement arity) body;
-      end_block (Ast.undots body);
+      end_block (Ast.unwrap body);
       rule_elem arity rbrace
 
   | Ast.IfThen(header,branch,_) ->
@@ -1198,7 +1248,7 @@ let rec statement arity s =
   | Ast.FunDecl(header,lbrace,body,rbrace,_) ->
       rule_elem arity header; rule_elem arity lbrace;
       dots force_newline (statement arity) body;
-      end_block (Ast.undots body);
+      end_block (Ast.unwrap body);
       rule_elem arity rbrace
 
   | Ast.Define(header,body) ->
@@ -1207,7 +1257,7 @@ let rec statement arity s =
 
   | Ast.AsStmt(stmt,asstmt) -> statement arity stmt
 
-  | Ast.Disj([stmt_dots]) ->
+  | Ast.Disj([stmt_dots]) | Ast.Conj([stmt_dots]) ->
       if generating
       then
 	(pr_arity arity;
@@ -1222,6 +1272,15 @@ let rec statement arity s =
 	   stmt_dots_list;
 	 print_text "\n)")
       else raise CantBeInPlus
+  | Ast.Conj(stmt_dots_list) -> (* ignores newline directive for readability *)
+      if generating
+      then
+	(pr_arity arity; print_text "\n(\n";
+	 print_between (function _ -> print_text "\n&\n")
+	   (dots force_newline (statement arity))
+	   stmt_dots_list;
+	 print_text "\n)")
+      else raise CantBeInPlus
   | Ast.Nest(starter,stmt_dots,ender,whn,multi,_,_) when generating ->
       pr_arity arity;
       nest_dots starter ender (statement arity)
@@ -1231,7 +1290,7 @@ let rec statement arity s =
 	  force_newline())
 	stmt_dots
   | Ast.Nest(_) -> raise CantBeInPlus
-  | Ast.Dots(d,whn,_,_) | Ast.Circles(d,whn,_,_) | Ast.Stars(d,whn,_,_) ->
+  | Ast.Dots(d,whn,_,_) ->
       if generating
       then
 	(pr_arity arity; mcode print_string d;
@@ -1240,8 +1299,7 @@ let rec statement arity s =
 	 force_newline())
       else raise CantBeInPlus
 
-  | Ast.OptStm(s) | Ast.UniqueStm(s) ->
-      raise CantBeInPlus
+  | Ast.OptStm(s) -> raise CantBeInPlus
 
 and whencode notfn alwaysfn = function
     Ast.WhenNot a ->
@@ -1297,12 +1355,12 @@ let rec pp_any = function
 
   | Ast.ConstantTag(x) -> constant x unknown unknown; false
   | Ast.UnaryOpTag(x) -> unaryOp x unknown unknown; false
-  | Ast.AssignOpTag(x) -> assignOp x unknown unknown; false
-  | Ast.SimpleAssignOpTag(x) -> print_string "=" unknown unknown; false
-  | Ast.OpAssignOpTag(x) -> opAssignOp x unknown unknown; false
+  | Ast.AssignOpTag(x) -> assignOp x; false
+  | Ast.SimpleAssignOpTag(x) -> print_string x unknown unknown; false
+  | Ast.OpAssignOpTag(x) -> arithOp true x unknown unknown; false
   | Ast.FixOpTag(x) -> fixOp x unknown unknown; false
-  | Ast.BinaryOpTag(x) -> binaryOp x unknown unknown; false
-  | Ast.ArithOpTag(x) -> arithOp x unknown unknown; false
+  | Ast.BinaryOpTag(x) -> binaryOp x; false
+  | Ast.ArithOpTag(x) -> arithOp false x unknown unknown; false
   | Ast.LogicalOpTag(x) -> logicalOp x unknown unknown; false
 
   | Ast.InitTag(x) -> initialiser false x; false
@@ -1317,7 +1375,7 @@ let rec pp_any = function
   | Ast.CaseLineTag(x) -> case_line "" x; false
   | Ast.StringFragmentTag(x) -> string_fragment x; false
 
-  | Ast.ConstVolTag(x) ->  const_vol x unknown unknown; false
+  | Ast.ConstVolTag(x) -> const_vol x unknown unknown; false
   | Ast.Directive(xs) ->
       (match xs with (Ast.Space s)::_ -> pr_space() | _ -> ());
       let rec loop = function
@@ -1379,7 +1437,7 @@ in
 	match Ast.unwrap s with Ast.FunDecl _ -> true | _ -> false in
       let prnl x = force_newline() in
       let newline_before _ =
-	if before =*= After
+	if before = After
 	then
 	  let hd = List.hd xxs in
 	  match hd with
@@ -1394,13 +1452,13 @@ in
 	  | (Ast.DeclarationTag _::_) | (Ast.Token ("}",_)::_) -> prnl hd
           | _ -> () in
       let newline_after _ =
-	if before =*= Before
+	if before = Before
 	then
 	  match List.rev(List.hd(List.rev xxs)) with
 	    (Ast.StatementTag s::_) ->
 	      (if isfn s then force_newline());
 	      force_newline()
-	  | (Ast.Directive _::_)
+	  | (Ast.Directive _::_) | (Ast.StmtDotsTag _::_)
           | (Ast.Rule_elemTag _::_) | (Ast.InitTag _::_)
 	  | (Ast.DeclarationTag _::_) | (Ast.Token ("{",_)::_) ->
 	       force_newline()
