@@ -110,7 +110,7 @@ let get_function_name rule env =
     bind (k e)
     (match Ast.unwrap e with
       Ast.FunCall(fn,lp,args,rp) ->
-	(match Ast.undots args with
+	(match Ast.unwrap args with
 	  [e] ->
 	    (match Ast.unwrap e with
 	      Ast.MetaExprList(nm,_,_,_) ->
@@ -141,7 +141,7 @@ let get_function_name rule env =
   match names with
     [name] ->
       (match env_lookup (function nm -> nm = name) env with
-	Ast_c.MetaIdVal(s,_) | Ast_c.MetaFuncVal(s)
+	Ast_c.MetaIdVal(s) | Ast_c.MetaFuncVal(s)
       | Ast_c.MetaLocalFuncVal(s) -> s
       |	_ -> error rule "not possible")
   | _ -> error rule "inconsistent rule generation"
@@ -234,7 +234,7 @@ let print_extra_typedefs pr env =
 	Ast_c.MetaIdVal(_) | Ast_c.MetaFuncVal(_)
       | Ast_c.MetaAssignOpVal(_) | Ast_c.MetaBinaryOpVal(_)
       | Ast_c.MetaLocalFuncVal(_) -> ()
-      | Ast_c.MetaExprVal(exp,_) -> Visitor_c.vk_expr bigf exp
+      | Ast_c.MetaExprVal(exp,_,_) -> Visitor_c.vk_expr bigf exp
       | Ast_c.MetaExprListVal(args) -> Visitor_c.vk_argument_list bigf args
       | Ast_c.MetaParamVal(param) -> Visitor_c.vk_param bigf param
       | Ast_c.MetaParamListVal(params) -> Visitor_c.vk_param_list bigf params
@@ -249,9 +249,12 @@ let print_extra_typedefs pr env =
       | Ast_c.MetaFmtVal(fmt) -> Visitor_c.vk_string_format bigf fmt
       | Ast_c.MetaFragListVal(frags) ->
 	  Visitor_c.vk_string_fragments bigf frags
-      | Ast_c.MetaStmtVal(stm) -> Visitor_c.vk_statement bigf stm
+      | Ast_c.MetaStmtVal(stm,_) -> Visitor_c.vk_statement bigf stm
+      | Ast_c.MetaStmtListVal(stms,_) ->
+	  Visitor_c.vk_statement_sequencable_list bigf stms
       | Ast_c.MetaPosVal _ | Ast_c.MetaPosValList _
-      | Ast_c.MetaListlenVal _ -> ())
+      | Ast_c.MetaListlenVal _ -> ()
+      | Ast_c.MetaNoVal -> failwith "referencing a metavar with no value")
     env
 
 let rename argids env =
@@ -280,8 +283,8 @@ let rename argids env =
 	 Ast_c.MetaIdVal(_) | Ast_c.MetaFuncVal(_)
        | Ast_c.MetaAssignOpVal(_) | Ast_c.MetaBinaryOpVal(_)
        | Ast_c.MetaLocalFuncVal(_) -> vl
-       | Ast_c.MetaExprVal(exp,c) ->
-	   Ast_c.MetaExprVal(Visitor_c.vk_expr_s bigf exp,c)
+       | Ast_c.MetaExprVal(exp,c,ty) ->
+	   Ast_c.MetaExprVal(Visitor_c.vk_expr_s bigf exp,c,ty)
        | Ast_c.MetaExprListVal(args) ->
 	   Ast_c.MetaExprListVal(Visitor_c.vk_arguments_s bigf args)
        | Ast_c.MetaParamVal(param) ->
@@ -305,10 +308,14 @@ let rename argids env =
 	   Ast_c.MetaFmtVal(Visitor_c.vk_string_format_s bigf fmt)
        | Ast_c.MetaFragListVal(frags) ->
 	   Ast_c.MetaFragListVal(Visitor_c.vk_string_fragments_s bigf frags)
-       | Ast_c.MetaStmtVal(stm) ->
-	   Ast_c.MetaStmtVal(Visitor_c.vk_statement_s bigf stm)
+       | Ast_c.MetaStmtVal(stm,ty) ->
+	   Ast_c.MetaStmtVal(Visitor_c.vk_statement_s bigf stm,ty)
+       | Ast_c.MetaStmtListVal(stm,ty) ->
+	   Ast_c.MetaStmtListVal
+	     (Visitor_c.vk_statement_sequencable_list_s bigf stm,ty)
        | Ast_c.MetaPosVal _ | Ast_c.MetaPosValList _
-       | Ast_c.MetaListlenVal _ -> vl))
+       | Ast_c.MetaListlenVal _ -> vl
+       | Ast_c.MetaNoVal -> failwith "referencing a metavar with no value"))
     env
 
 let print_one_type pr env = function
@@ -418,6 +425,7 @@ let pp_meta_decl pr env decl =
       no_arity ar; pr "declarer "; pp_name name; pr ";\n"
   | Ast.MetaIteratorDecl(ar, name) ->
       no_arity ar; pr "iterator "; pp_name name; pr ";\n"
+  | Ast.MetaScriptDecl(_, name) -> failwith "script metavariable"
 
 let print_metavariables pr local_metas paramst env header_req function_name =
   (if header_req
