@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./arity.ml"
 (* Arities matter for the minus slice, but not for the plus slice. *)
 
 (* ? only allowed on rule_elems, and on subterms if the context is ? also. *)
@@ -42,14 +21,13 @@ let fail w str =
        ((Ast0.get_info w).Ast0.pos_info.Ast0.line_start)
        str)
 
-let make_opt_unique optfn uniquefn info tgt arity term =
+let make_opt optfn info tgt arity term =
   let term = Ast0.rewrap info term in
   if tgt = arity
   then term
   else (* tgt must be NONE *)
     match arity with
       Ast0.OPT -> Ast0.copywrap info (optfn term)
-    | Ast0.UNIQUE -> Ast0.copywrap info (uniquefn term)
     | Ast0.NONE -> failwith "tgt must be NONE"
 
 let all_same opt_allowed tgt line arities =
@@ -112,70 +90,14 @@ let mcode x = x (* nothing to do ... *)
 (* --------------------------------------------------------------------- *)
 (* Dots *)
 
-let dots fn d =
-  Ast0.rewrap d
-    (match Ast0.unwrap d with
-      Ast0.DOTS(x) -> Ast0.DOTS(List.map fn x)
-    | Ast0.CIRCLES(x) -> Ast0.CIRCLES(List.map fn x)
-    | Ast0.STARS(x) -> Ast0.STARS(List.map fn x))
-
-let only_dots l =
-  not
-    (List.exists
-       (function x ->
-	  match Ast0.unwrap x with
-	   Ast0.Circles(_,_) | Ast0.Stars(_,_) -> true
-	 | _ -> false)
-       l)
-
-let only_circles l =
-  not (List.exists
-	(function x ->
-	  match Ast0.unwrap x with
-	    Ast0.Dots(_,_) | Ast0.Stars(_,_) -> true
-	  | _ -> false)
-	 l)
-
-let only_stars l =
-  not (List.exists
-	(function x ->
-	  match Ast0.unwrap x with
-	    Ast0.Dots(_,_) | Ast0.Circles(_,_) -> true
-	  | _ -> false)
-	 l)
-
-let concat_dots fn d =
-  Ast0.rewrap d
-    (match Ast0.unwrap d with
-      Ast0.DOTS(x) ->
-	let l = List.map fn x in
-	if only_dots l
-	then Ast0.DOTS(l)
-	else fail d "inconsistent dots usage"
-    | Ast0.CIRCLES(x) ->
-	let l = List.map fn x in
-	if only_circles l
-	then Ast0.CIRCLES(l)
-	else fail d "inconsistent dots usage"
-    | Ast0.STARS(x) ->
-	let l = List.map fn x in
-	if only_stars l
-	then Ast0.STARS(l)
-	else fail d "inconsistent dots usage")
-
-let flat_concat_dots fn d =
-  match Ast0.unwrap d with
-    Ast0.DOTS(x) -> List.map fn x
-  | Ast0.CIRCLES(x) -> List.map fn x
-  | Ast0.STARS(x) -> List.map fn x
+let dots fn d = Ast0.rewrap d (List.map fn (Ast0.unwrap d))
 
 (* --------------------------------------------------------------------- *)
 (* Identifier *)
 
 let make_id =
-  make_opt_unique
+  make_opt
     (function x -> Ast0.OptIdent x)
-    (function x -> Ast0.UniqueIdent x)
 
 let rec ident opt_allowed tgt i =
   match Ast0.unwrap i with
@@ -211,16 +133,16 @@ let rec ident opt_allowed tgt i =
 	  then fail i "opt only allowed in the last disjunct"
       |	_ -> ());
       Ast0.rewrap i (Ast0.DisjId(starter,id_list,mids,ender))
-  | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) | Ast0.AsIdent _ ->
+  | Ast0.OptIdent(_) | Ast0.AsIdent _ ->
       failwith "unexpected code"
 
 (* --------------------------------------------------------------------- *)
 (* Expression *)
 
 let make_exp =
-  make_opt_unique
+  make_opt
     (function x -> Ast0.OptExp x)
-    (function x -> Ast0.UniqueExp x)
+
 
 let rec top_expression opt_allowed tgt expr =
   let exp_same = all_same opt_allowed tgt in
@@ -231,8 +153,6 @@ let rec top_expression opt_allowed tgt expr =
 	(match Ast0.unwrap new_id with
 	  Ast0.OptIdent(id) ->
 	    Ast0.OptExp(Ast0.rewrap expr (Ast0.Ident(id)))
-	| Ast0.UniqueIdent(id) ->
-	    Ast0.UniqueExp(Ast0.rewrap expr (Ast0.Ident(id)))
 	| _ -> Ast0.Ident(new_id))
   | Ast0.Constant(const) ->
       let arity = exp_same (mcode2line const) [mcode2arity const] in
@@ -365,30 +285,29 @@ let rec top_expression opt_allowed tgt expr =
 	  then fail expr "opt only allowed in the last disjunct"
       |	_ -> ());
       Ast0.rewrap expr (Ast0.DisjExpr(starter,exps,mids,ender))
-  |  Ast0.NestExpr(starter,exp_dots,ender,whencode,multi) ->
+  | Ast0.ConjExpr(starter,exps,mids,ender) ->
+      (match exps with
+	e::es ->
+	  let e = top_expression opt_allowed tgt e in
+	  let arity =
+	    match Ast0.unwrap e with
+	      Ast0.OptExp _ -> Ast0.OPT
+	    | _ -> Ast0.NONE in
+	  let es = List.map (expression arity) es in
+	  make_exp expr tgt arity (Ast0.ConjExpr(starter,e::es,mids,ender))
+      |	_ -> expr)
+  | Ast0.NestExpr(starter,exp_dots,ender,whencode,multi) ->
       let res =
         Ast0.NestExpr(starter,
                       dots (top_expression true Ast0.NONE) exp_dots,
                       ender,whencode,multi) in
       Ast0.rewrap expr res
-  |  Ast0.Edots(dots,whencode) ->
+  | Ast0.Edots(dots,whencode) ->
       let arity = exp_same (mcode2line dots) [mcode2arity dots] in
       let dots = mcode dots in
       let whencode =
         get_option (fun (a,e,b) -> (a,e,expression Ast0.NONE b)) whencode in
       make_exp expr tgt arity (Ast0.Edots(dots,whencode))
-  | Ast0.Ecircles(dots,whencode) ->
-      let arity = exp_same (mcode2line dots) [mcode2arity dots] in
-      let dots = mcode dots in
-      let whencode =
-        get_option (fun (a,e,b) -> (a,e,expression Ast0.NONE b)) whencode in
-      make_exp expr tgt arity (Ast0.Ecircles(dots,whencode))
-  | Ast0.Estars(dots,whencode) ->
-      let arity = exp_same (mcode2line dots) [mcode2arity dots] in
-      let dots = mcode dots in
-      let whencode =
-        get_option (fun (a,e,b) -> (a,e,expression Ast0.NONE b)) whencode in
-      make_exp expr tgt arity (Ast0.Estars(dots,whencode))
   | Ast0.Constructor(lp,ty,rp,init) ->
       let arity = exp_same (mcode2line lp) [mcode2arity lp;mcode2arity rp] in
       let lp = mcode lp in
@@ -397,15 +316,14 @@ let rec top_expression opt_allowed tgt expr =
       let init = initialiser arity init in
       make_exp expr tgt arity (Ast0.Constructor(lp,ty,rp,init))
   (* why does optexp exist???? *)
-  | Ast0.OptExp(_) | Ast0.UniqueExp(_) | Ast0.AsExpr _ ->
+  | Ast0.OptExp(_) | Ast0.AsExpr _ | Ast0.AsSExpr _ ->
       failwith "unexpected code"
 
 and expression tgt exp = top_expression false tgt exp
 
 and make_fragment =
-  make_opt_unique
+  make_opt
     (function x -> failwith "opt not allowed for string fragment")
-    (function x -> failwith "unique not allowed for string fragment")
 
 and string_fragment tgt e =
   match Ast0.unwrap e with
@@ -431,9 +349,8 @@ and string_fragment tgt e =
       make_fragment e tgt arity (Ast0.MetaFormatList(pct,name,lenname))
 
 and make_format =
-  make_opt_unique
+  make_opt
     (function x -> failwith "opt not allowed for string format")
-    (function x -> failwith "unique not allowed for string format")
 
 and string_format tgt e =
   match Ast0.unwrap e with
@@ -450,9 +367,8 @@ and string_format tgt e =
 (* Types *)
 
 and make_typeC =
-  make_opt_unique
+  make_opt
     (function x -> Ast0.OptType x)
-    (function x -> Ast0.UniqueType x)
 
 and top_typeC tgt opt_allowed typ =
   match Ast0.unwrap typ with
@@ -559,7 +475,7 @@ and top_typeC tgt opt_allowed typ =
       |	_ -> ());
       let res = Ast0.DisjType(starter,types,mids,ender) in
       Ast0.rewrap typ res
-  | Ast0.OptType(_) | Ast0.UniqueType(_) | Ast0.AsType _ ->
+  | Ast0.OptType(_) | Ast0.AsType _ ->
       failwith "unexpected code"
 
 and typeC tgt ty = top_typeC tgt false ty
@@ -570,9 +486,8 @@ and typeC tgt ty = top_typeC tgt false ty
    split out into multiple declarations of a single variable each. *)
 
 and make_decl =
-  make_opt_unique
+  make_opt
     (function x -> Ast0.OptDecl x)
-    (function x -> Ast0.UniqueDecl x)
 
 and declaration tgt decl =
   match Ast0.unwrap decl with
@@ -623,24 +538,28 @@ and declaration tgt decl =
       let params = parameter_list tgt params in
       let va = match va with
         | None -> None
-        | Some (c1, e1) -> Some (mcode c1, mcode e1) in 
+        | Some (c1, e1) -> Some (mcode c1, mcode e1) in
       let rp1 = mcode rp1 in
       let sem = mcode sem in
       make_decl decl tgt arity (Ast0.FunProto(fi,name,lp1,params,va,rp1,sem))
-  | Ast0.MacroDecl(name,lp,args,rp,sem) ->
+  | Ast0.MacroDecl(stg,name,lp,args,rp,sem) ->
       let arity =
 	all_same true tgt (mcode2line lp)
-	  (List.map mcode2arity [lp;rp;sem]) in
+	  ((match stg with None -> [] | Some x -> [mcode2arity x]) @
+	   (List.map mcode2arity [lp;rp;sem])) in
+      let stg = get_option mcode stg in
       let name = ident false arity name in
       let lp = mcode lp in
       let args = dots (expression arity) args in
       let rp = mcode rp in
       let sem = mcode sem in
-      make_decl decl tgt arity (Ast0.MacroDecl(name,lp,args,rp,sem))
-  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+      make_decl decl tgt arity (Ast0.MacroDecl(stg,name,lp,args,rp,sem))
+  | Ast0.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem) ->
       let arity =
 	all_same true tgt (mcode2line lp)
-	  (List.map mcode2arity [lp;rp;eq;sem]) in
+	  ((match stg with None -> [] | Some x -> [mcode2arity x]) @
+	   (List.map mcode2arity [lp;rp;eq;sem])) in
+      let stg = get_option mcode stg in
       let name = ident false arity name in
       let lp = mcode lp in
       let args = dots (expression arity) args in
@@ -648,7 +567,7 @@ and declaration tgt decl =
       let ini = initialiser arity ini in
       let sem = mcode sem in
       make_decl decl tgt arity
-	(Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem))
+	(Ast0.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem))
   | Ast0.TyDecl(ty,sem) ->
       let arity =
 	all_same true tgt (mcode2line sem) [mcode2arity sem] in
@@ -679,16 +598,15 @@ and declaration tgt decl =
       let whencode =
         get_option (fun (a,e,b) -> (a,e,declaration Ast0.NONE b)) whencode in
       make_decl decl tgt arity (Ast0.Ddots(dots,whencode))
-  | Ast0.OptDecl(_) | Ast0.UniqueDecl(_) | Ast0.AsDecl _ ->
+  | Ast0.OptDecl(_) | Ast0.AsDecl _ ->
       failwith "unexpected code"
 
 (* --------------------------------------------------------------------- *)
 (* Initializer *)
 
 and make_init =
-  make_opt_unique
+  make_opt
     (function x -> Ast0.OptIni x)
-    (function x -> Ast0.UniqueIni x)
 
 and initialiser tgt i =
   let init_same = all_same true tgt in
@@ -731,7 +649,7 @@ and initialiser tgt i =
       let whencode =
         get_option (fun (a,e,b) -> (a,e,initialiser Ast0.NONE b)) whencode in
       make_init i tgt arity (Ast0.Idots(dots,whencode))
-  | Ast0.OptIni(_) | Ast0.UniqueIni(_) | Ast0.AsInit _ ->
+  | Ast0.OptIni(_) | Ast0.AsInit _ ->
       failwith "unexpected code"
 
 and designator tgt d =
@@ -763,9 +681,8 @@ and designator tgt d =
 (* Parameter *)
 
 and make_param =
-  make_opt_unique
+  make_opt
     (function x -> Ast0.OptParam x)
-    (function x -> Ast0.UniqueParam x)
 
 and parameterTypeDef tgt param =
   let param_same = all_same true tgt in
@@ -778,8 +695,6 @@ and parameterTypeDef tgt param =
 	(match (Ast0.unwrap ty,Ast0.unwrap id) with
 	  (Ast0.OptType(ty),Ast0.OptIdent(id)) ->
 	    Ast0.OptParam(Ast0.rewrap param (Ast0.Param(ty,Some id)))
-	| (Ast0.UniqueType(ty),Ast0.UniqueIdent(id)) ->
-	    Ast0.UniqueParam(Ast0.rewrap param (Ast0.Param(ty,Some id)))
 	| (Ast0.OptType(ty),_) ->
 	    fail param "arity mismatch in param declaration"
 	| (_,Ast0.OptIdent(id)) ->
@@ -791,8 +706,6 @@ and parameterTypeDef tgt param =
 	(match Ast0.unwrap ty with
 	  Ast0.OptType(ty) ->
 	    Ast0.OptParam(Ast0.rewrap param (Ast0.Param(ty,None)))
-	| Ast0.UniqueType(ty) ->
-	    Ast0.UniqueParam(Ast0.rewrap param (Ast0.Param(ty,None)))
 	| _ -> Ast0.Param(ty,None))
   | Ast0.MetaParam(name,pure) ->
       let arity = param_same (mcode2line name) [mcode2arity name] in
@@ -810,11 +723,7 @@ and parameterTypeDef tgt param =
       let arity = param_same (mcode2line dots) [mcode2arity dots] in
       let dots = mcode dots in
       make_param param tgt arity (Ast0.Pdots(dots))
-  | Ast0.Pcircles(dots) ->
-      let arity = param_same (mcode2line dots) [mcode2arity dots] in
-      let dots = mcode dots in
-      make_param param tgt arity (Ast0.Pcircles(dots))
-  | Ast0.OptParam(_) | Ast0.UniqueParam(_) | Ast0.AsParam _ ->
+  | Ast0.OptParam(_) | Ast0.AsParam _ ->
       failwith "unexpected code"
 
 and parameter_list tgt = dots (parameterTypeDef tgt)
@@ -823,9 +732,8 @@ and parameter_list tgt = dots (parameterTypeDef tgt)
 (* Top-level code *)
 
 and make_rule_elem x =
-  make_opt_unique
+  make_opt
     (function x -> Ast0.OptStm x)
-    (function x -> Ast0.UniqueStm x)
     x
 
 and statement tgt stm =
@@ -837,8 +745,6 @@ and statement tgt stm =
 	(match Ast0.unwrap new_decl with
 	  Ast0.OptDecl(decl) ->
 	    Ast0.OptStm(Ast0.rewrap stm (Ast0.Decl(bef,decl)))
-	| Ast0.UniqueDecl(decl) ->
-	    Ast0.UniqueStm(Ast0.rewrap stm (Ast0.Decl(bef,decl)))
 	| _ -> Ast0.Decl(bef,new_decl))
   | Ast0.Seq(lbrace,body,rbrace) ->
       let arity =
@@ -999,8 +905,6 @@ and statement tgt stm =
 	(match Ast0.unwrap new_exp with
 	  Ast0.OptExp(exp) ->
 	    Ast0.OptStm(Ast0.rewrap stm (Ast0.Exp(exp)))
-	| Ast0.UniqueExp(exp) ->
-	    Ast0.UniqueStm(Ast0.rewrap stm (Ast0.Exp(exp)))
 	| _ -> Ast0.Exp(new_exp))
   | Ast0.TopExp(exp) ->
       let new_exp = top_expression true tgt exp in
@@ -1008,8 +912,6 @@ and statement tgt stm =
 	(match Ast0.unwrap new_exp with
 	  Ast0.OptExp(exp) ->
 	    Ast0.OptStm(Ast0.rewrap stm (Ast0.TopExp(exp)))
-	| Ast0.UniqueExp(exp) ->
-	    Ast0.UniqueStm(Ast0.rewrap stm (Ast0.TopExp(exp)))
 	| _ -> Ast0.TopExp(new_exp))
   | Ast0.Ty(ty) ->
       let new_ty = typeC tgt ty in (* opt makes no sense alone at top level *)
@@ -1017,21 +919,25 @@ and statement tgt stm =
 	(match Ast0.unwrap new_ty with
 	  Ast0.OptType(ty) ->
 	    Ast0.OptStm(Ast0.rewrap stm (Ast0.Ty(ty)))
-	| Ast0.UniqueType(ty) ->
-	    Ast0.UniqueStm(Ast0.rewrap stm (Ast0.Ty(ty)))
 	| _ -> Ast0.Ty(new_ty))
+  | Ast0.TopId(id) ->
+      (* opt makes no sense alone at top level *)
+      let new_id = ident false tgt id in
+      Ast0.rewrap stm
+	(match Ast0.unwrap new_id with
+	  Ast0.OptIdent(id) ->
+	    Ast0.OptStm(Ast0.rewrap stm (Ast0.TopId(id)))
+	| _ -> Ast0.TopId(new_id))
   | Ast0.TopInit(init) ->
       let new_init = initialiser tgt init in
       Ast0.rewrap stm
 	(match Ast0.unwrap new_init with
 	  Ast0.OptIni(init) ->
 	    Ast0.OptStm(Ast0.rewrap stm (Ast0.TopInit(init)))
-	| Ast0.UniqueIni(init) ->
-	    Ast0.UniqueStm(Ast0.rewrap stm (Ast0.TopInit(init)))
 	| _ -> Ast0.TopInit(new_init))
   | Ast0.Disj(starter,rule_elem_dots_list,mids,ender) ->
       let stms =
-	List.map (function x -> concat_dots (statement tgt) x)
+	List.map (function x -> dots (statement tgt) x)
 	  rule_elem_dots_list in
       let (found_opt,unopt) =
 	List.fold_left
@@ -1052,14 +958,7 @@ and statement tgt stm =
 		if List.for_all is_opt l
 		then (true,List.map unopt l)
 		else (false, l) in
-	      let (l,k) =
-		match Ast0.unwrap x with
-		  Ast0.DOTS(l) ->
-		    (l,function l -> Ast0.rewrap x (Ast0.DOTS l))
-		| Ast0.CIRCLES(l) ->
-		    (l,function l -> Ast0.rewrap x (Ast0.CIRCLES l))
-		| Ast0.STARS(l) ->
-		    (l,function l -> Ast0.rewrap x (Ast0.STARS l)) in
+	      let (l,k) = (Ast0.unwrap x, function l -> Ast0.rewrap x l) in
 	      let (found_opt,l) = rebuild l in
 	      (found_opt,(k l)::lines))
 	  (false,[]) stms in
@@ -1068,43 +967,57 @@ and statement tgt stm =
       then
 	make_rule_elem stm tgt Ast0.OPT (Ast0.Disj(starter,unopt,mids,ender))
       else Ast0.rewrap stm (Ast0.Disj(starter,stms,mids,ender))
+  | Ast0.Conj(starter,rule_elem_dots_list,mids,ender) ->
+      let stms =
+	List.map (function x -> dots (statement tgt) x)
+	  rule_elem_dots_list in
+      let (found_opt,unopt) =
+	List.fold_left
+	  (function (found_opt,lines) ->
+	    function x ->
+	      let rebuild l =
+		(* previously just checked the last thing in the list,
+		   but everything should be optional for the whole thing to
+		   be optional *)
+		let is_opt x =
+		  match Ast0.unwrap x with
+		    Ast0.OptStm(x) -> true
+		  | _ -> false in
+		let unopt x =
+		  match Ast0.unwrap x with
+		    Ast0.OptStm(x) -> x
+		  | _ -> x in
+		if List.for_all is_opt l
+		then (true,List.map unopt l)
+		else (false, l) in
+	      let (l,k) = (Ast0.unwrap x, function l -> Ast0.rewrap x l) in
+	      let (fo,l) = rebuild l in
+	      (found_opt && fo,(k l)::lines))
+	  (true,[]) stms in
+      let unopt = List.rev unopt in
+      if found_opt
+      then
+	make_rule_elem stm tgt Ast0.OPT (Ast0.Disj(starter,unopt,mids,ender))
+      else Ast0.rewrap stm (Ast0.Conj(starter,stms,mids,ender))
   | Ast0.Nest(starter,rule_elem_dots,ender,whn,multi) ->
       let new_rule_elem_dots =
-	concat_dots (statement Ast0.NONE) rule_elem_dots in
+	dots (statement Ast0.NONE) rule_elem_dots in
       let whn =
 	List.map
-	  (whencode (concat_dots (statement Ast0.NONE)) (statement Ast0.NONE)
+	  (whencode (dots (statement Ast0.NONE)) (statement Ast0.NONE)
 	     (expression Ast0.NONE))
 	  whn in
       Ast0.rewrap stm
 	(Ast0.Nest(starter,new_rule_elem_dots,ender,whn,multi))
-  | Ast0.Dots(dots,whn)    ->
-      let arity = stm_same (mcode2line dots) [mcode2arity dots] in
-      let dots = mcode dots in
+  | Ast0.Dots(d,whn)    ->
+      let arity = stm_same (mcode2line d) [mcode2arity d] in
+      let d = mcode d in
       let whn =
 	List.map
-	  (whencode (concat_dots (statement Ast0.NONE)) (statement Ast0.NONE)
+	  (whencode (dots (statement Ast0.NONE)) (statement Ast0.NONE)
 	     (expression Ast0.NONE))
 	  whn in
-      make_rule_elem stm tgt arity (Ast0.Dots(dots,whn))
-  | Ast0.Circles(dots,whn) ->
-      let arity = stm_same (mcode2line dots) [mcode2arity dots] in
-      let dots = mcode dots in
-      let whn =
-	List.map
-	  (whencode (concat_dots (statement Ast0.NONE)) (statement Ast0.NONE)
-	     (expression Ast0.NONE))
-	  whn in
-      make_rule_elem stm tgt arity (Ast0.Circles(dots,whn))
-  | Ast0.Stars(dots,whn)   ->
-      let arity = stm_same (mcode2line dots) [mcode2arity dots] in
-      let dots = mcode dots in
-      let whn =
-	List.map
-	  (whencode (concat_dots (statement Ast0.NONE)) (statement Ast0.NONE)
-	     (expression Ast0.NONE))
-	  whn in
-      make_rule_elem stm tgt arity (Ast0.Stars(dots,whn))
+      make_rule_elem stm tgt arity (Ast0.Dots(d,whn))
   | Ast0.FunDecl(bef,fi,name,lp,params,va,rp,lbrace,body,rbrace,aft) ->
       let arity =
 	all_same true tgt (mcode2line lp)
@@ -1146,13 +1059,12 @@ and statement tgt stm =
       let id = ident false arity id in
       let body = pragmainfo arity body in
       make_rule_elem stm tgt arity (Ast0.Pragma(prg,id,body))
-  | Ast0.OptStm(_) | Ast0.UniqueStm(_) | Ast0.AsStmt _ ->
+  | Ast0.OptStm(_) | Ast0.AsStmt _ ->
       failwith "unexpected code"
 
 and make_pragma =
-  make_opt_unique
+  make_opt
     (function x -> failwith "opt not allowed for pragma")
-    (function x -> failwith "unique not allowed for pragma")
 
 and pragmainfo tgt pi =
   match Ast0.unwrap pi with
@@ -1183,9 +1095,8 @@ and define_parameters tgt params =
       Ast0.rewrap params (Ast0.DParams(lp,params,rp))
 
 and make_define_param x =
-  make_opt_unique
+  make_opt
     (function x -> Ast0.OptDParam x)
-    (function x -> Ast0.UniqueDParam x)
     x
 
 and define_param tgt param =
@@ -1196,8 +1107,6 @@ and define_param tgt param =
 	(match Ast0.unwrap new_id with
 	  Ast0.OptIdent(id) ->
 	    Ast0.OptDParam(Ast0.rewrap param (Ast0.DParam(id)))
-	| Ast0.UniqueIdent(decl) ->
-	    Ast0.UniqueDParam(Ast0.rewrap param (Ast0.DParam(id)))
 	| _ -> Ast0.DParam(new_id))
   | Ast0.DPComma(cm) ->
       let arity =
@@ -1209,12 +1118,7 @@ and define_param tgt param =
 	all_same true tgt (mcode2line dots) [mcode2arity dots] in
       let dots = mcode dots in
       make_define_param param tgt arity (Ast0.DPdots(dots))
-  | Ast0.DPcircles(circles) ->
-      let arity =
-	all_same true tgt (mcode2line circles) [mcode2arity circles] in
-      let circles = mcode circles in
-      make_define_param param tgt arity (Ast0.DPcircles(circles))
-  | Ast0.OptDParam(dp) | Ast0.UniqueDParam(dp) ->
+  | Ast0.OptDParam(dp) ->
       failwith "unexpected code"
 
 and fninfo arity = function
@@ -1241,9 +1145,8 @@ and whencode notfn alwaysfn expression = function
   | Ast0.WhenNotFalse (w,e,a) -> Ast0.WhenNotFalse (w,e,expression a)
 
 and make_case_line =
-  make_opt_unique
+  make_opt
     (function x -> Ast0.OptCase x)
-    (function x -> failwith "unique not allowed for case_line")
 
 and case_line tgt c =
   match Ast0.unwrap c with
@@ -1275,9 +1178,8 @@ and case_line tgt c =
   | Ast0.OptCase(_) -> failwith "unexpected OptCase"
 
 and make_exec_code =
-  make_opt_unique
+  make_opt
     (function x -> failwith "opt not allowed for exec code")
-    (function x -> failwith "unique not allowed for exec code")
 
 and exec_code tgt e =
   match Ast0.unwrap e with
@@ -1309,7 +1211,7 @@ let top_level tgt t =
     | Ast0.NONDECL(stmt) ->
 	Ast0.NONDECL(statement tgt stmt)
     | Ast0.CODE(rule_elem_dots) ->
-	Ast0.CODE(concat_dots (statement tgt) rule_elem_dots)
+	Ast0.CODE(dots (statement tgt) rule_elem_dots)
     | Ast0.TOPCODE(rule_elem_dots) ->  fail t "eliminated by top_level"
     | Ast0.ERRORWORDS(exps) ->
 	Ast0.ERRORWORDS(List.map (top_expression false Ast0.NONE) exps)
