@@ -1,37 +1,16 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./ast0_cocci.ml"
 module Ast = Ast_cocci
 module TC = Type_cocci
 
 (* --------------------------------------------------------------------- *)
 (* Modified code *)
 
-type arity = OPT | UNIQUE | NONE
+type arity = OPT | NONE
 
 type token_info =
     { tline_start : int; tline_end : int;
@@ -100,12 +79,7 @@ and pure = Impure | Pure | Context | PureContext (* pure and only context *)
 (* --------------------------------------------------------------------- *)
 (* Dots *)
 
-and 'a base_dots =
-    DOTS of 'a list
-  | CIRCLES of 'a list
-  | STARS of 'a list
-
-and 'a dots = 'a base_dots wrap
+and 'a dots = 'a list wrap
 
 (* --------------------------------------------------------------------- *)
 (* Identifier *)
@@ -119,7 +93,6 @@ and base_ident =
   | DisjId        of string mcode * ident list *
                      string mcode list (* the |s *) * string mcode
   | OptIdent      of ident
-  | UniqueIdent   of ident
 
 and ident = base_ident wrap
 
@@ -163,20 +136,18 @@ and base_expression =
   | MetaExprList   of Ast.meta_name mcode (* only in arg lists *) *
 	              listlen * pure
   | AsExpr         of expression * expression (* as expr, always metavar *)
+  | AsSExpr        of expression * statement (* as expr, always metavar *)
   | EComma         of string mcode (* only in arg lists *)
   | DisjExpr       of string mcode * expression list *
 	              string mcode list (* the |s *) * string mcode
+  | ConjExpr       of string mcode * expression list *
+	              string mcode list (* the &s *) * string mcode
   | NestExpr       of string mcode * expression dots * string mcode *
 	              (string mcode * string mcode * expression) option
 	              (* whencode *) * Ast.multi
   | Edots          of string mcode (* ... *) * (string mcode * string mcode *
                       expression) option (* whencode *)
-  | Ecircles       of string mcode (* ooo *) * (string mcode * string mcode *
-	              expression) option (* whencode *)
-  | Estars         of string mcode (* *** *) * (string mcode * string mcode *
-	              expression) option (* whencode *)
   | OptExp         of expression
-  | UniqueExp      of expression
 
 and expression = base_expression wrap
 
@@ -207,7 +178,7 @@ and string_format = base_string_format wrap
 
 (* --------------------------------------------------------------------- *)
 (* First class operators *)
-and base_assignOp = 
+and base_assignOp =
     SimpleAssign of simpleAssignOp mcode
   | OpAssign of Ast_cocci.arithOp mcode
   | MetaAssign of Ast_cocci.meta_name mcode * assignOpconstraint * pure
@@ -256,7 +227,6 @@ and base_typeC =
   | DisjType        of string mcode * typeC list * (* only after iso *)
                        string mcode list (* the |s *)  * string mcode
   | OptType         of typeC
-  | UniqueType      of typeC
 
 and typeC = base_typeC wrap
 
@@ -283,9 +253,11 @@ and base_declaration =
 	string mcode (* ) *) *
 	string mcode (* ; *)
   | TyDecl of typeC * string mcode (* ; *)
-  | MacroDecl of ident (* name *) * string mcode (* ( *) *
+  | MacroDecl of Ast.storage mcode option *
+	ident (* name *) * string mcode (* ( *) *
         expression dots * string mcode (* ) *) * string mcode (* ; *)
-  | MacroDeclInit of ident (* name *) * string mcode (* ( *) *
+  | MacroDeclInit of Ast.storage mcode option *
+	ident (* name *) * string mcode (* ( *) *
         expression dots * string mcode (* ) *) * string mcode (*=*) *
 	initialiser * string mcode (* ; *)
   | Typedef of string mcode (* typedef *) * typeC * typeC * string mcode (*;*)
@@ -295,7 +267,6 @@ and base_declaration =
   | Ddots      of string mcode (* ... *) * (string mcode * string mcode *
 	          declaration) option (* whencode *)
   | OptDecl    of declaration
-  | UniqueDecl of declaration
 
 and declaration = base_declaration wrap
 
@@ -319,7 +290,6 @@ and base_initialiser =
   | Idots  of string mcode (* ... *) *
               (string mcode * string mcode * initialiser) option (* whencode *)
   | OptIni    of initialiser
-  | UniqueIni of initialiser
 
 and designator =
     DesignatorField of string mcode (* . *) * ident
@@ -343,9 +313,7 @@ and base_parameterTypeDef =
   | AsParam       of parameterTypeDef * expression (* expr, always metavar *)
   | PComma        of string mcode
   | Pdots         of string mcode (* ... *)
-  | Pcircles      of string mcode (* ooo *)
   | OptParam      of parameterTypeDef
-  | UniqueParam   of parameterTypeDef
 
 and parameterTypeDef = base_parameterTypeDef wrap
 
@@ -358,9 +326,7 @@ and base_define_param =
     DParam        of ident
   | DPComma       of string mcode
   | DPdots        of string mcode (* ... *)
-  | DPcircles     of string mcode (* ooo *)
   | OptDParam     of define_param
-  | UniqueDParam  of define_param
 
 and define_param = base_define_param wrap
 
@@ -419,17 +385,16 @@ and base_statement =
   | Exp           of expression  (* only in dotted statement lists *)
   | TopExp        of expression (* for macros body *)
   | Ty            of typeC (* only at top level *)
+  | TopId         of ident (* only at top level *)
   | TopInit       of initialiser (* only at top level *)
   | Disj          of string mcode * statement dots list *
 	             string mcode list (* the |s *)  * string mcode
+  | Conj          of string mcode * statement dots list *
+	             string mcode list (* the &s *)  * string mcode
   | Nest          of string mcode * statement dots * string mcode *
 	             (statement dots,statement) whencode list * Ast.multi
   | Dots          of string mcode (* ... *) *
                      (statement dots,statement) whencode list
-  | Circles       of string mcode (* ooo *) *
-	             (statement dots,statement) whencode list
-  | Stars         of string mcode (* *** *) *
-	             (statement dots,statement) whencode list
   | FunDecl of (info * mcodekind) (* before the function decl *) *
 	fninfo list * ident (* name *) *
 	string mcode (* ( *) * parameter_list *
@@ -444,7 +409,6 @@ and base_statement =
 	define_parameters (*params*) * statement dots
   | Pragma of string mcode (* #pragma *) * ident * pragmainfo
   | OptStm   of statement
-  | UniqueStm of statement
 
 and base_pragmainfo =
     PragmaTuple of string mcode(* ( *) * expression dots * string mcode(* ) *)
@@ -515,18 +479,21 @@ and rule = top_level list
 and parsed_rule =
     CocciRule of
       (rule (*minus*) * Ast.metavar list (*minus metavars*) *
-	(string list (*isos*) * string list (*drop_isos*) * 
+	(string list (*isos*) * string list (*drop_isos*) *
          Ast.dependency (*dependencies*) * string (*rulename*) * Ast.exists)) *
       (rule (*plus*) * Ast.metavar list (*plus metavars*)) * Ast.ruletype
   | ScriptRule of string (* name *) * string * Ast.dependency *
-	(Ast.script_meta_name * Ast.meta_name * Ast.metavar) list *
+	(Ast.script_meta_name * Ast.meta_name * Ast.metavar * Ast.mvinit)
+	  list *
 	Ast.meta_name list (*script vars*) *
 	string
   | InitialScriptRule of string (* name *) * string * Ast.dependency *
-	(Ast.script_meta_name * Ast.meta_name * Ast.metavar) list *
+	(Ast.script_meta_name * Ast.meta_name * Ast.metavar * Ast.mvinit)
+	  list *
 	string
   | FinalScriptRule of string (* name *) * string * Ast.dependency *
-	(Ast.script_meta_name * Ast.meta_name * Ast.metavar) list *
+	(Ast.script_meta_name * Ast.meta_name * Ast.metavar * Ast.mvinit)
+	  list *
 	string (* no script vars possible here *)
 
 (* --------------------------------------------------------------------- *)
@@ -709,6 +676,18 @@ let rec meta_pos_name = function
       | _ -> failwith "bad metavariable")
   | _ -> failwith "bad metavariable"
 
+let rec meta_pos_constraint_names = function
+    ExprTag(e) ->
+      (match unwrap e with
+	MetaExpr(name,constraints,ty,form,pure) ->
+	  (match ty with
+	    Some tylist ->
+	      List.fold_left (fun prev cur -> TC.meta_names cur @ prev)
+		[] tylist
+	  | None -> [])
+      |	_ -> [])
+  | _ -> []
+
 (* --------------------------------------------------------------------- *)
 
 (* unique indices, for mcode and tree nodes *)
@@ -717,14 +696,6 @@ let fresh_index _ = let cur = !index_counter in index_counter := cur + 1; cur
 
 (* --------------------------------------------------------------------- *)
 
-let undots d =
-  match unwrap d with
-  | DOTS    e -> e
-  | CIRCLES e -> e
-  | STARS   e -> e
-
-(* --------------------------------------------------------------------- *)
-
 let rec ast0_type_to_type inmeta ty =
   match unwrap ty with
     ConstVol(cv,ty) -> TC.ConstVol(const_vol cv,ast0_type_to_type inmeta ty)
@@ -799,8 +770,7 @@ let rec ast0_type_to_type inmeta ty =
       Common.pr2_once
 	"disjtype not supported in smpl type inference, assuming unknown";
       TC.Unknown
-  | OptType(ty) | UniqueType(ty) ->
-      ast0_type_to_type inmeta ty
+  | OptType(ty) -> ast0_type_to_type inmeta ty
 
 and baseType = function
     Ast.VoidType -> TC.VoidType
