@@ -1,34 +1,13 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./ast0_cocci.mli"
 (* --------------------------------------------------------------------- *)
 (* Modified code *)
 
-type arity = OPT | UNIQUE | NONE
+type arity = OPT | NONE
 
 type token_info =
     { tline_start : int; tline_end : int;
@@ -89,12 +68,7 @@ and pure = Impure | Pure | Context | PureContext (* pure and only context *)
 (* --------------------------------------------------------------------- *)
 (* Dots *)
 
-and 'a base_dots =
-    DOTS of 'a list
-  | CIRCLES of 'a list
-  | STARS of 'a list
-
-and 'a dots = 'a base_dots wrap
+and 'a dots = 'a list wrap
 
 (* --------------------------------------------------------------------- *)
 (* Identifier *)
@@ -109,7 +83,6 @@ and base_ident =
   | DisjId        of string mcode * ident list *
                      string mcode list (* the |s *) * string mcode
   | OptIdent      of ident
-  | UniqueIdent   of ident
 
 and ident = base_ident wrap
 
@@ -153,20 +126,18 @@ and base_expression =
   | MetaExprList   of Ast_cocci.meta_name mcode (* only in arglists *) *
 	              listlen * pure
   | AsExpr         of expression * expression (* as expr, always metavar *)
+  | AsSExpr        of expression * statement (* as expr, always metavar *)
   | EComma         of string mcode (* only in arglists *)
   | DisjExpr       of string mcode * expression list * string mcode list *
 	              string mcode
+  | ConjExpr       of string mcode * expression list * string mcode list *
+	              string mcode
   | NestExpr       of string mcode * expression dots * string mcode *
-                      (string mcode * string mcode * expression) option 
+                      (string mcode * string mcode * expression) option
                       (* whencode *) * Ast_cocci.multi
   | Edots          of string mcode (* ... *) * (string mcode * string mcode *
                       expression) option (* whencode *)
-  | Ecircles       of string mcode (* ooo *) * (string mcode * string mcode *
-                      expression) option (* whencode *)
-  | Estars         of string mcode (* *** *) * (string mcode * string mcode *
-                      expression) option (* whencode *)
   | OptExp         of expression
-  | UniqueExp      of expression
 
 and expression = base_expression wrap
 
@@ -197,7 +168,7 @@ and string_format = base_string_format wrap
 
 (* --------------------------------------------------------------------- *)
 (* First class operators *)
-and  base_assignOp = 
+and  base_assignOp =
     SimpleAssign of simpleAssignOp mcode
   | OpAssign of Ast_cocci.arithOp mcode
   | MetaAssign of Ast_cocci.meta_name mcode * assignOpconstraint * pure
@@ -247,7 +218,6 @@ and base_typeC =
   | DisjType        of string mcode * typeC list * (* only after iso *)
                        string mcode list (* the |s *)  * string mcode
   | OptType         of typeC
-  | UniqueType      of typeC
 
 and typeC = base_typeC wrap
 
@@ -271,9 +241,11 @@ and base_declaration =
         (string mcode (* , *) * string mcode (* ...... *) ) option *
 	string mcode (* ) *) * string mcode (* ; *)
   | TyDecl of typeC * string mcode (* ; *)
-  | MacroDecl of ident (* name *) * string mcode (* ( *) *
+  | MacroDecl of Ast_cocci.storage mcode option *
+	ident (* name *) * string mcode (* ( *) *
         expression dots * string mcode (* ) *) * string mcode (* ; *)
-  | MacroDeclInit of ident (* name *) * string mcode (* ( *) *
+  | MacroDeclInit of Ast_cocci.storage mcode option *
+	ident (* name *) * string mcode (* ( *) *
         expression dots * string mcode (* ) *) * string mcode (*=*) *
         initialiser * string mcode (* ; *)
   | Typedef of string mcode (* typedef *) * typeC * typeC * string mcode (*;*)
@@ -282,7 +254,6 @@ and base_declaration =
   | Ddots      of string mcode (* ... *) * (string mcode * string mcode *
                   declaration) option (* whencode *)
   | OptDecl    of declaration
-  | UniqueDecl of declaration
 
 and declaration = base_declaration wrap
 
@@ -305,7 +276,6 @@ and base_initialiser =
   | Idots  of string mcode (* ... *) *
               (string mcode * string mcode * initialiser) option (* whencode *)
   | OptIni    of initialiser
-  | UniqueIni of initialiser
 
 and designator =
     DesignatorField of string mcode (* . *) * ident
@@ -329,9 +299,7 @@ and base_parameterTypeDef =
   | AsParam       of parameterTypeDef * expression (* expr, always metavar *)
   | PComma        of string mcode
   | Pdots         of string mcode (* ... *)
-  | Pcircles      of string mcode (* ooo *)
   | OptParam      of parameterTypeDef
-  | UniqueParam   of parameterTypeDef
 
 and parameterTypeDef = base_parameterTypeDef wrap
 
@@ -344,9 +312,7 @@ and base_define_param =
     DParam        of ident
   | DPComma       of string mcode
   | DPdots        of string mcode (* ... *)
-  | DPcircles     of string mcode (* ooo *)
   | OptDParam     of define_param
-  | UniqueDParam  of define_param
 
 and define_param = base_define_param wrap
 
@@ -405,17 +371,16 @@ and base_statement =
   | Exp           of expression  (* only in dotted statement lists *)
   | TopExp        of expression (* for macros body *)
   | Ty            of typeC (* only at top level *)
+  | TopId         of ident (* only at top level *)
   | TopInit       of initialiser (* only at top level *)
   | Disj          of string mcode * statement dots list * string mcode list *
 	             string mcode
+  | Conj          of string mcode * statement dots list * string mcode list *
+	             string mcode
   | Nest          of string mcode * statement dots * string mcode *
 	             (statement dots,statement) whencode list * Ast_cocci.multi
   | Dots          of string mcode (* ... *) *
                      (statement dots,statement) whencode list
-  | Circles       of string mcode (* ooo *) *
-	             (statement dots,statement) whencode list
-  | Stars         of string mcode (* *** *) *
-	             (statement dots,statement) whencode list
   | FunDecl of (info * mcodekind) (* before the function decl *) *
 	fninfo list * ident (* name *) *
 	string mcode (* ( *) * parameter_list *
@@ -430,7 +395,6 @@ and base_statement =
 	define_parameters (*params*) * statement dots
   | Pragma of string mcode (* #pragma *) * ident * pragmainfo
   | OptStm   of statement
-  | UniqueStm of statement
 
 and base_pragmainfo =
     PragmaTuple of string mcode(* ( *) * expression dots * string mcode(* ) *)
@@ -508,16 +472,19 @@ and parsed_rule =
   | ScriptRule of string (* name *) *
       string * Ast_cocci.dependency *
 	(Ast_cocci.script_meta_name *
-	   Ast_cocci.meta_name * Ast_cocci.metavar) list (*inherited vars*) *
+	   Ast_cocci.meta_name * Ast_cocci.metavar * Ast_cocci.mvinit)
+	    list (*inherited vars*) *
 	Ast_cocci.meta_name list (*script vars*) *
 	string
   | InitialScriptRule of string (* name *) * string * Ast_cocci.dependency *
 	(Ast_cocci.script_meta_name *
-	   Ast_cocci.meta_name * Ast_cocci.metavar) list (*virtual vars*) *
+	   Ast_cocci.meta_name * Ast_cocci.metavar * Ast_cocci.mvinit)
+	     list (*virtual vars*) *
 	string
   | FinalScriptRule of string (* name *) * string * Ast_cocci.dependency *
 	(Ast_cocci.script_meta_name *
-	   Ast_cocci.meta_name * Ast_cocci.metavar) list (*virtual vars*) *
+	   Ast_cocci.meta_name * Ast_cocci.metavar * Ast_cocci.mvinit)
+	     list (*virtual vars*) *
 	string
 
 (* --------------------------------------------------------------------- *)
@@ -583,10 +550,6 @@ val case_line : case_line -> anything
 val string_fragment : string_fragment -> anything
 val top : top_level -> anything
 
-(* --------------------------------------------------------------------- *)
-
-val undots : 'a dots -> 'a list
-
 (* --------------------------------------------------------------------- *)
 (* Avoid cluttering the parser.  Calculated in compute_lines.ml. *)
 
@@ -635,6 +598,7 @@ val make_minus_mcode : 'a -> 'a mcode
 val get_rule_name : parsed_rule -> string
 
 val meta_pos_name : anything -> Ast_cocci.meta_name mcode
+val meta_pos_constraint_names : anything -> Ast_cocci.meta_name list
 
 val ast0_type_to_type : bool -> typeC -> Type_cocci.typeC
 val reverse_type : Type_cocci.typeC -> base_typeC
