@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./ast0toast.ml"
 (* Arities matter for the minus slice, but not for the plus slice. *)
 
 (* + only allowed on code in a nest (in_nest = true).  ? only allowed on
@@ -224,9 +203,11 @@ let check_allminus =
 
   let expression r k e =
     match Ast0.unwrap e with
-      Ast0.DisjExpr(starter,expr_list,mids,ender) ->
+      Ast0.DisjExpr(starter,expr_list,mids,ender) (* ignore starter, etc *)
+    | Ast0.ConjExpr(starter,expr_list,mids,ender) ->
 	List.for_all r.VT0.combiner_rec_expression expr_list
     | Ast0.AsExpr(exp,asexp) -> k exp
+    | Ast0.AsSExpr(exp,asstm) -> k exp
     | _ -> k e in
 
   let declaration r k e =
@@ -250,7 +231,8 @@ let check_allminus =
 
   let statement r k e =
     match Ast0.unwrap e with
-      Ast0.Disj(starter,statement_dots_list,mids,ender) ->
+      Ast0.Disj(starter,statement_dots_list,mids,ender)
+    | Ast0.Conj(starter,statement_dots_list,mids,ender) ->
 	List.for_all r.VT0.combiner_rec_statement_dots statement_dots_list
     | Ast0.AsStmt(stmt,asstmt) -> k stmt
     | _ -> k e in
@@ -348,12 +330,7 @@ let no_isos = []
 let tokenwrap (_,info,_,_) s ast = wrap ast info.Ast.line no_isos
 let iso_tokenwrap (_,info,_,_) s ast iso = wrap ast info.Ast.line iso
 
-let dots fn d =
-  rewrap d no_isos
-    (match Ast0.unwrap d with
-      Ast0.DOTS(x) -> Ast.DOTS(List.map fn x)
-    | Ast0.CIRCLES(x) -> Ast.CIRCLES(List.map fn x)
-    | Ast0.STARS(x) -> Ast.STARS(List.map fn x))
+let dots fn d = rewrap d no_isos (List.map fn (Ast0.unwrap d))
 
 (* --------------------------------------------------------------------- *)
 (* Identifier *)
@@ -374,8 +351,7 @@ and ident i =
 	Ast.MetaLocalFunc(mcode name,constraints,unitary,false)
     | Ast0.AsIdent(id,asid) ->
 	Ast.AsIdent(ident id,ident asid)
-    | Ast0.OptIdent(id) -> Ast.OptIdent(ident id)
-    | Ast0.UniqueIdent(id) -> Ast.UniqueIdent(ident id))
+    | Ast0.OptIdent(id) -> Ast.OptIdent(ident id))
 
 (* --------------------------------------------------------------------- *)
 (* Expression *)
@@ -446,9 +422,17 @@ and expression e =
 	Ast.MetaExprList(mcode name,do_lenname lenname,unitary,false)
     | Ast0.AsExpr(expr,asexpr) ->
 	Ast.AsExpr(expression expr,expression asexpr)
+    | Ast0.AsSExpr(expr,asstm) ->
+	let stm =
+	  match Ast.unwrap (statement asstm) with
+	    Ast.Atomic(re) -> re
+	  | _ -> failwith "stmt should be metavar, and thus atomic" in
+	Ast.AsSExpr(expression expr,stm)
     | Ast0.EComma(cm)         -> Ast.EComma(mcode cm)
     | Ast0.DisjExpr(_,exps,_,_)     ->
 	Ast.DisjExpr(List.map expression exps)
+    | Ast0.ConjExpr(_,exps,_,_)     ->
+	Ast.ConjExpr(List.map expression exps)
     | Ast0.NestExpr(starter,exp_dots,ender,whencode,multi) ->
 	let starter = mcode starter in
 	let whencode = get_option (fun (_,_,b) -> expression b) whencode in
@@ -458,16 +442,7 @@ and expression e =
 	let dots = mcode dots in
 	let whencode = get_option (fun (_,_,b) -> expression b) whencode in
 	Ast.Edots(dots,whencode)
-    | Ast0.Ecircles(dots,whencode) ->
-	let dots = mcode dots in
-	let whencode = get_option (fun (_,_,b) -> expression b) whencode in
-	Ast.Ecircles(dots,whencode)
-    | Ast0.Estars(dots,whencode) ->
-	let dots = mcode dots in
-	let whencode = get_option (fun (_,_,b) -> expression b) whencode in
-	Ast.Estars(dots,whencode)
-    | Ast0.OptExp(exp) -> Ast.OptExp(expression exp)
-    | Ast0.UniqueExp(exp) -> Ast.UniqueExp(expression exp)) in
+    | Ast0.OptExp(exp) -> Ast.OptExp(expression exp)) in
   if Ast0.get_test_exp e then Ast.set_test_exp e1 else e1
 
 and assignOp op =
@@ -487,7 +462,7 @@ and binaryOp op =
       Ast0.Arith op' -> Ast.Arith (mcode op')
     | Ast0.Logical op' -> Ast.Logical (mcode op')
     | Ast0.MetaBinary(mv, c, _) ->
-      Ast.MetaBinary(mcode mv, binaryOpconstraint c, unitary, false))    
+      Ast.MetaBinary(mcode mv, binaryOpconstraint c, unitary, false))
 and binaryOpconstraint = function
   | Ast0.BinaryOpNoConstraint -> Ast.BinaryOpNoConstraint
   | Ast0.BinaryOpInSet ops -> Ast.BinaryOpInSet (List.map binaryOp ops)
@@ -564,8 +539,7 @@ and typeC allminus t =
 	Ast.DisjType(List.map (typeC allminus) types)
     | Ast0.AsType(ty,asty) ->
 	Ast.AsType(typeC allminus ty,typeC allminus asty)
-    | Ast0.OptType(ty) -> Ast.OptType(typeC allminus ty)
-    | Ast0.UniqueType(ty) -> Ast.UniqueType(typeC allminus ty))
+    | Ast0.OptType(ty) -> Ast.OptType(typeC allminus ty))
 
 and base_typeC allminus t =
   match Ast0.unwrap t with
@@ -638,14 +612,18 @@ and declaration d =
 	  let rp = mcode rp in
 	  let sem = mcode sem in
 	  Ast.FunProto(fi,name,lp,params,va,rp,sem)
-    | Ast0.MacroDecl(name,lp,args,rp,sem) ->
+    | Ast0.MacroDecl(stg,name,lp,args,rp,sem) ->
+	(* this would seem to need allminus... *)
+	let stg = get_option mcode stg in
 	let name = ident name in
 	let lp = mcode lp in
 	let args = dots expression args in
 	let rp = mcode rp in
 	let sem = mcode sem in
-	Ast.MacroDecl(name,lp,args,rp,sem)
-    | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+	Ast.MacroDecl(stg,name,lp,args,rp,sem)
+    | Ast0.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem) ->
+	(* this would seem to need allminus... *)
+	let stg = get_option mcode stg in
 	let name = ident name in
 	let lp = mcode lp in
 	let args = dots expression args in
@@ -653,7 +631,7 @@ and declaration d =
 	let eq = mcode eq in
 	let ini = initialiser ini in
 	let sem = mcode sem in
-	Ast.MacroDeclInit(name,lp,args,rp,eq,ini,sem)
+	Ast.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem)
     | Ast0.TyDecl(ty,sem) ->
 	let allminus = check_allminus.VT0.combiner_rec_declaration d in
 	Ast.TyDecl(typeC allminus ty,mcode sem)
@@ -666,8 +644,7 @@ and declaration d =
 	| _ -> failwith "bad typedef")
     | Ast0.Ddots(dots,whencode) -> failwith "should not be possible"
     | Ast0.DisjDecl(_,decls,_,_) -> Ast.DisjDecl(List.map declaration decls)
-    | Ast0.OptDecl(decl) -> Ast.OptDecl(declaration decl)
-    | Ast0.UniqueDecl(decl) -> Ast.UniqueDecl(declaration decl))
+    | Ast0.OptDecl(decl) -> Ast.OptDecl(declaration decl))
 
 and annotated_decl bef d =
   rewrap d (do_isos (Ast0.get_iso d))
@@ -698,43 +675,41 @@ and strip_idots initlist =
     match Ast0.get_mcode_mcodekind mc with
       Ast0.MINUS _ -> true
     | _ -> false in
-  match Ast0.unwrap initlist with
-    Ast0.DOTS(l) ->
-      let l =
-	match List.rev l with
-	  [] | [_] -> l
-	| x::y::xs ->
-	    (match (Ast0.unwrap x,Ast0.unwrap y) with
-	      (Ast0.IComma _,Ast0.Idots _) ->
+  let l = Ast0.unwrap initlist in
+  let l =
+    match List.rev l with
+      [] | [_] -> l
+    | x::y::xs ->
+	(match (Ast0.unwrap x,Ast0.unwrap y) with
+	  (Ast0.IComma _,Ast0.Idots _) ->
 		(* drop comma that was added by add_comma *)
-		List.rev (y::xs)
-	    | _ -> l) in
-      let (whencode,init,dotinfo) =
-	let rec loop = function
-	    [] -> ([],[],[])
-	  | x::rest ->
-	      (match Ast0.unwrap x with
-		Ast0.Idots(dots,Some whencode) ->
-		  let (restwhen,restinit,dotinfo) = loop rest in
-		  (whencode :: restwhen, restinit,
-		    (isminus dots)::dotinfo)
-	      |	Ast0.Idots(dots,None) ->
-		  let (restwhen,restinit,dotinfo) = loop rest in
-		  (restwhen, restinit, (isminus dots)::dotinfo)
-	      |	_ ->
-		  let (restwhen,restinit,dotinfo) = loop rest in
-		  (restwhen,x::restinit,dotinfo)) in
-	loop l in
-      let allminus =
-	if List.for_all (function x -> not x) dotinfo
-	then false (* false if no dots *)
-	else
-	  if List.for_all (function x -> x) dotinfo
-	  then true
-	  else failwith "inconsistent annotations on initialiser list dots" in
-      (whencode, init, allminus)
-  | Ast0.CIRCLES(x) | Ast0.STARS(x) -> failwith "not possible for an initlist"
-
+	    List.rev (y::xs)
+	| _ -> l) in
+  let (whencode,init,dotinfo) =
+    let rec loop = function
+	[] -> ([],[],[])
+      | x::rest ->
+	  (match Ast0.unwrap x with
+	    Ast0.Idots(dots,Some whencode) ->
+	      let (restwhen,restinit,dotinfo) = loop rest in
+	      (whencode :: restwhen, restinit,
+		(isminus dots)::dotinfo)
+	  | Ast0.Idots(dots,None) ->
+	      let (restwhen,restinit,dotinfo) = loop rest in
+	      (restwhen, restinit, (isminus dots)::dotinfo)
+	  | _ ->
+	      let (restwhen,restinit,dotinfo) = loop rest in
+	      (restwhen,x::restinit,dotinfo)) in
+    loop l in
+  let allminus =
+    if List.for_all (function x -> not x) dotinfo
+    then false (* false if no dots *)
+    else
+      if List.for_all (function x -> x) dotinfo
+      then true
+      else failwith "inconsistent annotations on initialiser list dots" in
+  (whencode, init, allminus)
+    
 and initialiser i =
   rewrap i no_isos
     (match Ast0.unwrap i with
@@ -761,8 +736,7 @@ and initialiser i =
 	let dots = mcode dots in
 	let whencode = get_option (fun (_,_,b) -> initialiser b) whencode in
 	Ast.Idots(dots,whencode)
-    | Ast0.OptIni(ini) -> Ast.OptIni(initialiser ini)
-    | Ast0.UniqueIni(ini) -> Ast.UniqueIni(initialiser ini))
+    | Ast0.OptIni(ini) -> Ast.OptIni(initialiser ini))
 
 and designator = function
     Ast0.DesignatorField(dot,id) -> Ast.DesignatorField(mcode dot,ident id)
@@ -790,9 +764,7 @@ and parameterTypeDef p =
 	Ast.AsParam(parameterTypeDef p,expression asexpr)
     | Ast0.PComma(cm) -> Ast.PComma(mcode cm)
     | Ast0.Pdots(dots) -> Ast.Pdots(mcode dots)
-    | Ast0.Pcircles(dots) -> Ast.Pcircles(mcode dots)
-    | Ast0.OptParam(param) -> Ast.OptParam(parameterTypeDef param)
-    | Ast0.UniqueParam(param) -> Ast.UniqueParam(parameterTypeDef param))
+    | Ast0.OptParam(param) -> Ast.OptParam(parameterTypeDef param))
 
 and parameter_list l = dots parameterTypeDef l
 
@@ -818,13 +790,13 @@ and statement s =
 	  Ast.Atomic(rewrap_rule_elem s
 		       (Ast.Decl(annotated_decl (Some bef) decl)))
       | Ast0.Seq(lbrace,body,rbrace) ->
-	  let lbrace = mcode lbrace in
-	  let body = dots (statement seqible) body in
-	  let rbrace = mcode rbrace in
-	  Ast.Seq(iso_tokenwrap lbrace s (Ast.SeqStart(lbrace))
-		    (do_isos (Ast0.get_iso s)),
-		  body,
-		  tokenwrap rbrace s (Ast.SeqEnd(rbrace)))
+          let lbrace = mcode lbrace in
+          let body = dots (statement seqible) body in
+          let rbrace = mcode rbrace in
+          Ast.Seq(iso_tokenwrap lbrace s (Ast.SeqStart(lbrace))
+                    (do_isos (Ast0.get_iso s)),
+                  body,
+                  tokenwrap rbrace s (Ast.SeqEnd(rbrace)))
       | Ast0.ExprStatement(exp,sem) ->
 	  Ast.Atomic(rewrap_rule_elem s
 		       (Ast.ExprStatement
@@ -884,7 +856,7 @@ and statement s =
 	  let rp = mcode rp in
 	  let lb = mcode lb in
 	  let decls = dots (statement seqible) decls in
-	  let cases = List.map case_line (Ast0.undots cases) in
+	  let cases = List.map case_line (Ast0.unwrap cases) in
 	  let rb = mcode rb in
 	  Ast.Switch(rewrap_rule_elem s (Ast.SwitchHeader(switch,lp,exp,rp)),
 		     tokenwrap lb s (Ast.SeqStart(lb)),
@@ -926,9 +898,14 @@ and statement s =
       | Ast0.Ty(ty) ->
 	  let allminus = check_allminus.VT0.combiner_rec_statement s in
 	  Ast.Atomic(rewrap_rule_elem s (Ast.Ty(typeC allminus ty)))
+      | Ast0.TopId(id) ->
+	  Ast.Atomic(rewrap_rule_elem s (Ast.TopId(ident id)))
       | Ast0.Disj(_,rule_elem_dots_list,_,_) ->
 	  Ast.Disj(List.map (function x -> statement_dots seqible x)
 		     rule_elem_dots_list)
+      | Ast0.Conj(_,rule_elem_dots_list,_,_) ->
+	  Ast.Conj(List.map (function x -> statement_dots seqible x)
+		     rule_elem_dots_list)
       | Ast0.Nest(starter,rule_elem_dots,ender,whn,multi) ->
 	  Ast.Nest
 	    (mcode starter,statement_dots Ast.Sequencible rule_elem_dots,
@@ -946,22 +923,6 @@ and statement s =
 		 (statement Ast.NotSequencible))
 	      whn in
 	  Ast.Dots(d,whn,[],[])
-      | Ast0.Circles(d,whn) ->
-	  let d = mcode d in
-	  let whn =
-	    List.map
-	      (whencode (statement_dots Ast.Sequencible)
-		 (statement Ast.NotSequencible))
-	      whn in
-	  Ast.Circles(d,whn,[],[])
-      | Ast0.Stars(d,whn) ->
-	  let d = mcode d in
-	  let whn =
-	    List.map
-	      (whencode (statement_dots Ast.Sequencible)
-		 (statement Ast.NotSequencible))
-	      whn in
-	  Ast.Stars(d,whn,[],[])
       | Ast0.FunDecl((_,bef),fi,name,lp,params,va,rp,lbrace,body,rbrace,
 		     (_,aft)) ->
 	  let fi = List.map fninfo fi in
@@ -972,18 +933,18 @@ and statement s =
             | None -> None
             | Some (comma, ellipsis) -> Some (mcode comma, mcode ellipsis) in
 	  let rp = mcode rp in
-	  let lbrace = mcode lbrace in
-	  let body = dots (statement seqible) body in
-	  let rbrace = mcode rbrace in
-	  let allminus = check_allminus.VT0.combiner_rec_statement s in
-	  Ast.FunDecl(rewrap_rule_elem s
-			(Ast.FunHeader
-			   (convert_allminus_mcodekind allminus bef,
-			    allminus,fi,name,lp,params,newva,rp)),
-		      tokenwrap lbrace s (Ast.SeqStart(lbrace)),
-		      body,
-		      tokenwrap rbrace s (Ast.SeqEnd(rbrace)),
-		      ([],[],[],convert_allminus_mcodekind allminus aft))
+          let lbrace = mcode lbrace in
+          let body = dots (statement seqible) body in
+          let rbrace = mcode rbrace in
+          let allminus = check_allminus.VT0.combiner_rec_statement s in
+          Ast.FunDecl(rewrap_rule_elem s
+                        (Ast.FunHeader
+                           (convert_allminus_mcodekind allminus bef,
+                            allminus,fi,name,lp,params,newva,rp)),
+                      tokenwrap lbrace s (Ast.SeqStart(lbrace)),
+                      body,
+                      tokenwrap rbrace s (Ast.SeqEnd(rbrace)),
+                      ([],[],[],convert_allminus_mcodekind allminus aft))
       |	Ast0.Include(inc,str) ->
 	  Ast.Atomic(rewrap_rule_elem s (Ast.Include(mcode inc,mcode str)))
       |	Ast0.Undef(def,id) ->
@@ -997,8 +958,7 @@ and statement s =
       |	Ast0.Pragma(prg,id,body) ->
 	  Ast.Atomic(rewrap_rule_elem s
 		       (Ast.Pragma(mcode prg,ident id,pragmainfo body)))
-      | Ast0.OptStm(stm) -> Ast.OptStm(statement seqible stm)
-      | Ast0.UniqueStm(stm) -> Ast.UniqueStm(statement seqible stm))
+      | Ast0.OptStm(stm) -> Ast.OptStm(statement seqible stm))
 
   and pragmainfo pi =
     rewrap pi no_isos
@@ -1026,9 +986,7 @@ and statement s =
 	Ast0.DParam(id) -> Ast.DParam(ident id)
       | Ast0.DPComma(comma) -> Ast.DPComma(mcode comma)
       | Ast0.DPdots(d) -> Ast.DPdots(mcode d)
-      | Ast0.DPcircles(c) -> Ast.DPcircles(mcode c)
-      | Ast0.OptDParam(dp) -> Ast.OptDParam(define_param dp)
-      | Ast0.UniqueDParam(dp) -> Ast.UniqueDParam(define_param dp))
+      | Ast0.OptDParam(dp) -> Ast.OptDParam(define_param dp))
 
   and whencode notfn alwaysfn = function
       Ast0.WhenNot (_,_,a) -> Ast.WhenNot (notfn a)
@@ -1060,11 +1018,7 @@ and statement s =
 
   and statement_dots seqible d =
     let isos = do_isos (Ast0.get_iso d) in
-    rewrap d no_isos
-      (match Ast0.unwrap d with
-	Ast0.DOTS(x) -> Ast.DOTS(process_list seqible isos x)
-      | Ast0.CIRCLES(x) -> Ast.CIRCLES(process_list seqible isos x)
-      | Ast0.STARS(x) -> Ast.STARS(process_list seqible isos x))
+    rewrap d no_isos (process_list seqible isos (Ast0.unwrap d))
 
   (* the following is no longer used.
    the goal was to let one put a statement at the very beginning of a function
@@ -1093,7 +1047,8 @@ and statement s =
 	      (match decls with
 		[] -> ([],x::other)
 	      | _ -> (x :: decls,other))
-	  | Ast0.Disj(starter,stmt_dots_list,mids,ender) ->
+	  | Ast0.Disj(starter,stmt_dots_list,mids,ender)
+	  | Ast0.Conj(starter,stmt_dots_list,mids,ender) ->
 	      let disjs = List.map collect_dot_decls stmt_dots_list in
 	      let all_decls = List.for_all (function (_,s) -> s=[]) disjs in
 	      if all_decls
@@ -1103,21 +1058,14 @@ and statement s =
 	      else ([],l)
 	  | _ -> ([],l))
 
-    and collect_dot_decls d =
-      match Ast0.unwrap d with
-	Ast0.DOTS(x) -> collect_decls x
-      | Ast0.CIRCLES(x) -> collect_decls x
-      | Ast0.STARS(x) -> collect_decls x in
+    and collect_dot_decls d = collect_decls (Ast0.unwrap d) in
 
-    let process l d fn =
+    let process l d =
       let (decls,other) = collect_decls l in
-      (rewrap d no_isos (fn (List.map (statement seqible) decls)),
+      (rewrap d no_isos (List.map (statement seqible) decls),
        rewrap d no_isos
-	 (fn (process_list seqible (do_isos (Ast0.get_iso d)) other))) in
-    match Ast0.unwrap d with
-      Ast0.DOTS(x) -> process x d (function x -> Ast.DOTS x)
-    | Ast0.CIRCLES(x) -> process x d (function x -> Ast.CIRCLES x)
-    | Ast0.STARS(x) -> process x d (function x -> Ast.STARS x) *) in
+	 (process_list seqible (do_isos (Ast0.get_iso d)) other)) in
+     process x (Ast0.unwrap d) *) in
 
   statement Ast.Sequencible s
 
