@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./check_meta.ml"
 (* For minus fragment, checks that all of the identifier metavariables that
 are used are not declared as fresh, and check that all declared variables
 are used.  For plus fragment, just check that the variables declared as
@@ -71,11 +50,7 @@ let get_opt fn = Common.do_option fn
 (* --------------------------------------------------------------------- *)
 (* Dots *)
 
-let dots fn d =
-  match Ast0.unwrap d with
-    Ast0.DOTS(x) -> List.iter fn x
-  | Ast0.CIRCLES(x) -> List.iter fn x
-  | Ast0.STARS(x) -> List.iter fn x
+let dots fn d = List.iter fn (Ast0.unwrap d)
 
 (* --------------------------------------------------------------------- *)
 (* Identifier *)
@@ -119,7 +94,7 @@ let rec ident context old_metas table minus i =
   | Ast0.AsIdent(id,asid) -> failwith "not generated yet"
   | Ast0.DisjId(_,id_list,_,_) ->
       List.iter (ident context old_metas table minus) id_list
-  | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) ->
+  | Ast0.OptIdent(_) ->
       failwith "unexpected code"
 
 and seed table minus = function
@@ -216,15 +191,16 @@ let rec expression context old_metas table minus e =
   | Ast0.MetaExprList(name,_,_) ->
       check_table table minus name
   | Ast0.AsExpr(exp,asexp) -> failwith "not generated yet"
-  | Ast0.DisjExpr(_,exps,_,_) ->
+  | Ast0.AsSExpr(exp,asstm) -> failwith "not generated yet"
+  | Ast0.DisjExpr(_,exps,_,_)
+  | Ast0.ConjExpr(_,exps,_,_) ->
       List.iter (expression context old_metas table minus) exps
   | Ast0.NestExpr(_,exp_dots,_,w,_) ->
       dots (expression ID old_metas table minus) exp_dots;
       get_opt (function (_,_,x) -> expression ID old_metas table minus x) w
-  | Ast0.Edots(_,Some (_,_,x)) | Ast0.Ecircles(_,Some (_,_,x))
-  | Ast0.Estars(_,Some (_,_,x)) ->
+  | Ast0.Edots(_,Some (_,_,x)) ->
       expression ID old_metas table minus x
-  | Ast0.OptExp(x) | Ast0.UniqueExp(x) ->
+  | Ast0.OptExp(x) ->
       expression ID old_metas table minus x
   | _ -> () (* no metavariable subterms *)
 
@@ -273,7 +249,7 @@ and typeC old_metas table minus t =
   | Ast0.StructUnionDef(ty,lb,decls,rb) ->
       typeC old_metas table minus ty;
       dots (declaration GLOBAL old_metas table minus) decls
-  | Ast0.OptType(ty) | Ast0.UniqueType(ty) ->
+  | Ast0.OptType(ty) ->
       failwith "unexpected code"
   | _ -> () (* no metavariable subterms *)
 
@@ -311,10 +287,10 @@ and declaration context old_metas table minus d =
       ident FN old_metas table minus name;
       List.iter (fninfo old_metas table minus) fi;
       parameter_list old_metas table minus params
-  | Ast0.MacroDecl(name,lp,args,rp,sem) ->
+  | Ast0.MacroDecl(stg,name,lp,args,rp,sem) ->
       ident GLOBAL old_metas table minus name;
       dots (expression ID old_metas table minus) args
-  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+  | Ast0.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem) ->
       ident GLOBAL old_metas table minus name;
       dots (expression ID old_metas table minus) args;
       (match Ast0.unwrap ini with
@@ -328,7 +304,7 @@ and declaration context old_metas table minus d =
       List.iter (declaration ID old_metas table minus) decls
   | Ast0.Ddots(_,Some (_,_,x)) -> declaration ID old_metas table minus x
   | Ast0.Ddots(_,None) -> ()
-  | Ast0.OptDecl(_) | Ast0.UniqueDecl(_) ->
+  | Ast0.OptDecl(_) ->
       failwith "unexpected code"
 
 (* --------------------------------------------------------------------- *)
@@ -354,7 +330,7 @@ and initialiser old_metas table minus ini =
       ident FIELD old_metas table minus name;
       initialiser old_metas table minus ini
   | Ast0.Idots(_,Some (_,_,x)) -> initialiser old_metas table minus x
-  | Ast0.OptIni(_) | Ast0.UniqueIni(_) ->
+  | Ast0.OptIni(_) ->
       failwith "unexpected code"
   | _ -> () (* no metavariable subterms *)
 
@@ -459,8 +435,10 @@ and statement old_metas table minus s =
   | Ast0.Exp(exp) -> expression ID old_metas table minus exp
   | Ast0.TopExp(exp) -> expression ID old_metas table minus exp
   | Ast0.Ty(ty) -> typeC old_metas table minus ty
+  | Ast0.TopId(id) -> ident ID old_metas table minus id
   | Ast0.TopInit(init) -> initialiser old_metas table minus init
-  | Ast0.Disj(_,rule_elem_dots_list,_,_) ->
+  | Ast0.Disj(_,rule_elem_dots_list,_,_)
+  | Ast0.Conj(_,rule_elem_dots_list,_,_) ->
       List.iter (dots (statement old_metas table minus)) rule_elem_dots_list
   | Ast0.Nest(_,rule_elem_dots,_,w,_) ->
       dots (statement old_metas table minus) rule_elem_dots;
@@ -468,7 +446,7 @@ and statement old_metas table minus s =
 		   (statement old_metas table minus)
 		   (expression ID old_metas table minus))
 	w
-  | Ast0.Dots(_,x) | Ast0.Circles(_,x) | Ast0.Stars(_,x) ->
+  | Ast0.Dots(_,x) ->
       List.iter
 	(whencode (dots (statement old_metas table minus))
 	   (statement old_metas table minus)
@@ -502,10 +480,9 @@ and pragmainfo old_metas table minus pi =
 and define_param old_metas table minus p =
   match Ast0.unwrap p with
     Ast0.DParam(id) -> ident GLOBAL old_metas table minus id
-  | Ast0.DPComma(_) | Ast0.DPdots(_) | Ast0.DPcircles(_) ->
+  | Ast0.DPComma(_) | Ast0.DPdots(_) ->
       () (* no metavariable subterms *)
   | Ast0.OptDParam(dp)    -> define_param old_metas table minus dp
-  | Ast0.UniqueDParam(dp) -> define_param old_metas table minus dp
 
 and define_parameters old_metas table minus x =
   match Ast0.unwrap x with
@@ -559,20 +536,30 @@ let rule old_metas table minus rules =
 
 (* --------------------------------------------------------------------- *)
 
-let positions table rules =
+let positions rname table rules =
+  let do_tynames var =
+    let tynames = Ast0.meta_pos_constraint_names var in
+    List.iter
+      (function name ->
+	(* only needed if the name is a local variable, not an inherited one *)
+	if fst name = rname
+	then (find_loop table name) := true)
+      tynames in
   let rec rmcode x = (* needed for type inference, nonpolymorphic *)
     List.iter
       (function var ->
 	let name = Ast0.meta_pos_name var in
 	(find_loop table (Ast0.unwrap_mcode name)) := true;
-	rmcode name)
+	rmcode name;
+	do_tynames var)
       (Ast0.get_pos x) in
   let rec mcode x =
     List.iter
       (function var ->
 	let name = Ast0.meta_pos_name var in
 	(find_loop table (Ast0.unwrap_mcode name)) := true;
-	rmcode name)
+	rmcode name;
+	do_tynames var)
       (Ast0.get_pos x) in
   let option_default = () in
   let bind x y = () in
@@ -605,7 +592,8 @@ let dup_positions rules =
 
   let expression r k e =
     match Ast0.unwrap e with
-      Ast0.DisjExpr(_,explist,_,_) ->
+      Ast0.DisjExpr(_,explist,_,_)
+    | Ast0.ConjExpr(_,explist,_,_) ->
 	List.fold_left Common.union_set option_default
 	  (List.map r.VT0.combiner_rec_expression explist)
     | _ -> k e in
@@ -626,7 +614,8 @@ let dup_positions rules =
 
   let statement r k e =
     match Ast0.unwrap e with
-      Ast0.Disj(_,stmts,_,_) ->
+      Ast0.Disj(_,stmts,_,_)
+    | Ast0.Conj(_,stmts,_,_) ->
 	List.fold_left Common.union_set option_default
 	  (List.map r.VT0.combiner_rec_statement_dots stmts)
     | _ -> k e in
@@ -699,7 +688,7 @@ let check_meta rname old_metas inherited_metavars metavars minus plus =
 
   add_to_fresh_table fresh;
   rule old_metas [iother_table;other_table;err_table] true minus;
-  positions [iother_table;other_table] minus;
+  positions rname [iother_table;other_table] minus;
   dup_positions minus;
   check_all_marked rname "metavariable" other_table "in the - or context code";
   rule old_metas [iother_table;fresh_table;err_table] false plus;
