@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./data.ml"
 module Ast0 = Ast0_cocci
 module Ast = Ast_cocci
 
@@ -49,9 +28,9 @@ type clt =
 (* Things that need to be seen by the lexer and parser. *)
 
 and line_type =
-    MINUS | OPTMINUS | UNIQUEMINUS
+    MINUS | OPTMINUS
   | PLUS | PLUSPLUS
-  | CONTEXT | UNIQUE | OPT
+  | CONTEXT | OPT
 
 type iconstraints = Ast.idconstraint
 type econstraints = Ast0.constraints
@@ -79,7 +58,7 @@ let all_metadecls =
 
 let uninitialized_add_meta = fun _ -> failwith "uninitialized add_meta"
 
-let clear_meta: (unit -> unit) ref = 
+let clear_meta: (unit -> unit) ref =
   ref uninitialized_add_meta
 
 let add_meta_meta: (Ast.meta_name -> Ast0.pure -> unit) ref =
