@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./data.mli"
 (* types that clutter the .mly file *)
 (* for iso metavariables, true if they can only match nonmodified, unitary
    metavariables *)
@@ -44,9 +23,9 @@ type clt =
 (* ---------------------------------------------------------------------- *)
 
 and line_type =
-    MINUS | OPTMINUS | UNIQUEMINUS
+    MINUS | OPTMINUS
   | PLUS | PLUSPLUS
-  | CONTEXT | UNIQUE | OPT
+  | CONTEXT | OPT
 
 type iconstraints = Ast_cocci.idconstraint
 type econstraints = Ast0_cocci.constraints
@@ -152,11 +131,11 @@ val add_fmt_meta: (Ast_cocci.meta_name -> iconstraints -> unit) ref
 
 val add_fmtlist_meta: (Ast_cocci.meta_name -> Ast_cocci.list_len -> unit) ref
 
-val add_assignOp_meta: 
+val add_assignOp_meta:
     (Ast_cocci.meta_name ->
       Ast0_cocci.assignOpconstraint -> Ast0_cocci.pure -> unit) ref
 
-val add_binaryOp_meta: 
+val add_binaryOp_meta:
     (Ast_cocci.meta_name ->
       Ast0_cocci.binaryOpconstraint -> Ast0_cocci.pure -> unit) ref
 
