@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./free_vars.ml"
 (* For each rule return the list of variables that are used after it.
 Also augment various parts of each rule with unitary, inherited, and freshness
 informations *)
@@ -98,6 +77,7 @@ let collect_refs include_constraints =
 	    then
 	      match idconstraint with
 		Ast.IdNegIdSet (_,metas) -> metas
+	      | Ast.IdPosIdSet (_,metas) -> metas
 	      | _ -> []
 	    else [] in
 	  bind (List.rev metas) [metaid name]
@@ -165,6 +145,36 @@ let collect_refs include_constraints =
 	  [metaid name]
       | _ -> option_default) in
 
+  let rec collect_assign_names aop =
+    match Ast.unwrap aop with
+      Ast.MetaAssign(name,Ast.AssignOpNoConstraint,_,_) ->
+	[metaid name]
+    | Ast.MetaAssign(name,Ast.AssignOpInSet l,_,_) ->
+	if include_constraints
+	then
+	  List.fold_left (fun prev a -> bind (collect_assign_names a) prev)
+	    [metaid name] l
+	else [metaid name]
+    | _ -> option_default in
+
+  let astfvassignop recursor k bop =
+    bind (k bop) (collect_assign_names bop) in
+
+  let rec collect_binary_names bop =
+    match Ast.unwrap bop with
+      Ast.MetaBinary(name,Ast.BinaryOpNoConstraint,_,_) ->
+	[metaid name]
+    | Ast.MetaBinary(name,Ast.BinaryOpInSet l,_,_) ->
+	if include_constraints
+	then
+	  List.fold_left (fun prev a -> bind (collect_binary_names a) prev)
+	    [metaid name] l
+	else [metaid name]
+    | _ -> option_default in
+
+  let astfvbinaryop recursor k bop =
+    bind (k bop) (collect_binary_names bop) in
+
   let astfvdecls recursor k d =
     bind (k d)
       (match Ast.unwrap d with
@@ -236,7 +246,7 @@ let collect_refs include_constraints =
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
     mcode mcode mcode mcode
     donothing donothing donothing donothing donothing
-    astfvident astfvexpr astfvfrag astfvfmt donothing donothing
+    astfvident astfvexpr astfvfrag astfvfmt astfvassignop astfvbinaryop
     astfvfullType astfvtypeC astfvinit astfvparam astfvdecls donothing
     astfvrule_elem astfvstatement donothing donothing donothing_a
 
@@ -357,6 +367,18 @@ let collect_saved =
 	Ast.MetaFormat(name,_,TC.Saved,_) -> [metaid name]
       | _ -> option_default) in
 
+  let astfvassign recursor k aop =
+    bind (k aop)
+      (match Ast.unwrap aop with
+	Ast.MetaAssign(name,_,TC.Saved,_) -> [metaid name]
+      | _ -> option_default) in
+
+  let astfvbinary recursor k bop =
+    bind (k bop)
+      (match Ast.unwrap bop with
+	Ast.MetaBinary(name,_,TC.Saved,_) -> [metaid name]
+      | _ -> option_default) in
+
   let astfvtypeC recursor k ty =
     bind (k ty)
       (match Ast.unwrap ty with
@@ -423,8 +445,8 @@ let collect_saved =
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
     mcode mcode
     donothing donothing donothing donothing donothing
-    astfvident astfvexpr astfvfrag astfvfmt donothing donothing donothing
-    astfvtypeC astfvinit astfvparam astfvdecls donothing astfvrule_elem 
+    astfvident astfvexpr astfvfrag astfvfmt astfvassign astfvbinary donothing
+    astfvtypeC astfvinit astfvparam astfvdecls donothing astfvrule_elem
     donothing donothing donothing donothing
 
 (* ---------------------------------------------------------------- *)
@@ -517,17 +539,14 @@ let collect_in_plus_term =
 	       fi) in
 	let nm_metas = collect_all_refs.V.combiner_ident nm in
 	let param_metas =
-	  match Ast.unwrap params with
-	    Ast.DOTS(params) | Ast.CIRCLES(params) ->
-	      List.concat
-		(List.map
-		   (function p ->
-		     match Ast.unwrap p with
-		       Ast.VoidParam(t) | Ast.Param(t,_) ->
-			 collect_all_refs.V.combiner_fullType t
-		     | _ -> [])
-		   params)
-	  | _ -> failwith "not allowed for params" in
+	  List.concat
+	    (List.map
+	       (function p ->
+	          match Ast.unwrap p with
+	            Ast.VoidParam(t) | Ast.Param(t,_) ->
+	              collect_all_refs.V.combiner_fullType t
+		  | _ -> [])
+	       (Ast.unwrap params)) in
 	bind fi_metas
 	  (bind nm_metas
 	     (bind param_metas
@@ -687,6 +706,22 @@ let classify_variables metavar_decls minirules used_after =
 	Ast.rewrap ft (Ast.MetaFormat(name,constraints,unitary,inherited))
     | _ -> ft in
 
+  let assignop r k ft =
+    let ft = k ft in
+    match Ast.unwrap ft with
+      Ast.MetaAssign(name,constraints,_,_) ->
+	let (unitary,inherited) = classify name in
+	Ast.rewrap ft (Ast.MetaAssign(name,constraints,unitary,inherited))
+    | _ -> ft in
+
+  let binaryop r k ft =
+    let ft = k ft in
+    match Ast.unwrap ft with
+      Ast.MetaBinary(name,constraints,_,_) ->
+	let (unitary,inherited) = classify name in
+	Ast.rewrap ft (Ast.MetaBinary(name,constraints,unitary,inherited))
+    | _ -> ft in
+
   let typeC r k e =
     let e = k e in
     match Ast.unwrap e with
@@ -767,7 +802,8 @@ let classify_variables metavar_decls minirules used_after =
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
       mcode mcode
       donothing donothing donothing donothing donothing
-      ident expression string_fragment string_format donothing donothing donothing typeC
+      ident expression string_fragment string_format assignop binaryop
+      donothing typeC
       init param decl donothing rule_elem
       donothing donothing donothing donothing in
 
@@ -835,6 +871,7 @@ let astfvs metavars bound =
     | Ast.Exp _ -> "Exp"
     | Ast.TopExp _ -> "TopExp"
     | Ast.Ty _ -> "Ty"
+    | Ast.TopId _ -> "TopId"
     | Ast.TopInit _ -> "TopInit"
     | Ast.Include _ -> "Include"
     | Ast.Undef _ -> "Undef"
@@ -842,6 +879,7 @@ let astfvs metavars bound =
     | Ast.Pragma _ -> "Pragma"
     | Ast.Case _ -> "Case"
     | Ast.Default _ -> "Default"
+    | Ast.AsRe _ -> "AsRe"
     | Ast.DisjRuleElem _ -> "DisjRuleElem" in
 
   (* cases for the elements of anything *)
@@ -948,7 +986,7 @@ let astfvs metavars bound =
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
     mcode mcode
     donothing donothing astfvstatement_dots donothing donothing
-    donothing donothing donothing 
+    donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing
     astfvrule_elem astfvstatement astfvcase_line astfvtoplevel donothing
@@ -1061,7 +1099,7 @@ let collect_top_level_used_after metavar_rule_list =
 	  let free_vars =
             match r with
               Ast.ScriptRule (_,_,_,mv,_,_) ->
-                drop_virt(List.map (function (_,(r,v),_) -> (r,v)) mv)
+                drop_virt(List.map (function (_,(r,v),_,_) -> (r,v)) mv)
             | Ast.InitialScriptRule (_,_,_,mv,_)
 	    | Ast.FinalScriptRule (_,_,_,mv,_) ->
 		(* only virtual identifiers *)
