@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./get_constants2.ml"
 module Ast = Ast_cocci
 module V = Visitor_ast
 module TC = Type_cocci
@@ -68,48 +47,6 @@ let rec dep2c = function
   | False -> "false"
   | True -> "true"
 
-let sat f =
-  let subset l1 l2 = List.for_all (fun e1 -> List.mem e1 l2) l1 in
-  let opt_union_set (longer : (string list * string list) list)
-                    (shorter : (string list * string list) list) =
-    (* (A v B) & (A v B v C) = A v B *)
-    (* tries to be efficient by not updating prv, so optimize is still
-       needed *)
-    List.fold_left
-      (function prev ->
-	function ((p,n) as cur) ->
-	  if List.exists (function (p1,n1) -> subset p1 p && subset n1 n) prev
-	  then prev
-	  else cur :: prev)
-      longer shorter in
-  let double_union (p,n) (p1,n1) =
-    ((Common.union_set p p1), (Common.union_set n n1)) in
-  let rec cnf = function
-      Elem x -> [([x],[])]
-    | Not (Elem x) -> [([],[x])]
-    | Not (And l) -> cnf (Or (List.map (function x -> Not x) l))
-    | Not (Or l) -> cnf (And (List.map (function x -> Not x) l))
-    | Not (Not x) -> cnf x
-    | Not True -> [([],[])] (* false *)
-    | Not False -> []  (* true *)
-    | And l ->
-	List.fold_left opt_union_set [] (List.map cnf l)
-    | Or l ->
-	let l = List.map cnf l in
-	(match l with
-	  fst::rest ->
-	    List.fold_left
-	      (function prev ->
-		function cur ->
-		  List.fold_left opt_union_set
-		    []
-		    (List.map (fun x -> List.map (double_union x) prev) cur))
-	      fst rest
-	| [] -> [([],[])]) (* false *)
-    | True -> []
-    | False -> [([],[])] in
-  Dpll.dpll (cnf f)
-
 (* glimpse often fails on large queries.  We can safely remove arguments of
 && as long as we don't remove all of them (note that there is no negation).
 This tries just removing one of them and then orders the results by
@@ -205,7 +142,7 @@ let interpret_grep strict x =
       failwith false_on_top_err
   | _ -> Some (loop [] x)
 
-let interpret_cocci_grep strict x =
+let interpret_cocci_git_grep strict x =
   (* convert to cnf *)
   let subset l1 l2 = List.for_all (fun e1 -> List.mem e1 l2) l1 in
   let opt_union_set longer shorter =
@@ -271,7 +208,10 @@ let interpret_cocci_grep strict x =
 	(function clause ->
 	  Printf.printf "%s\n" (String.concat " " clause))
 	res;*)
-      Some (res1,res2)
+      let res3 =
+	List.map (function x -> "\\( -e "^(String.concat " -e " x)^" \\)")
+	  res in
+      Some (res1,res2,res3)
 
 let combine2c x =
   match interpret_glimpse false x with
@@ -413,6 +353,14 @@ let do_get_constants constants keywords env neg_pos =
 	  (match Ast.unwrap_mcode name with
 	    "NULL" -> keywords "NULL"
 	  | nm -> constants nm)
+    | Ast.MetaId(name,Ast.IdPosIdSet(strs,mids),_,_)
+    | Ast.MetaFunc(name,Ast.IdPosIdSet(strs,mids),_,_)
+    | Ast.MetaLocalFunc(name,Ast.IdPosIdSet(strs,mids),_,_) ->
+	let cur =
+	  build_or
+	    (disj_union_all (List.map constants strs))
+	    (disj_union_all (List.map inherited mids)) in
+	bind (k i) (bind (minherited name) cur)
     | Ast.MetaId(name,_,_,_) | Ast.MetaFunc(name,_,_,_)
     | Ast.MetaLocalFunc(name,_,_,_) ->
 	bind (k i) (minherited name)
@@ -491,7 +439,7 @@ let do_get_constants constants keywords env neg_pos =
     | Ast.DisjExpr(exps) ->
 	disj_union_all (List.map r.V.combiner_expression exps)
     | Ast.OptExp(exp) -> option_default
-    | Ast.Edots(_,_) | Ast.Ecircles(_,_) | Ast.Estars(_,_) -> option_default
+    | Ast.Edots(_,_) -> option_default
     | _ -> k e in
 
   (* cases for metavariabes *)
@@ -612,8 +560,7 @@ let do_get_constants constants keywords env neg_pos =
     | Ast.Nest(starter,stmt_dots,ender,whn,true,_,_) ->
 	r.V.combiner_statement_dots stmt_dots
     | Ast.OptStm(s) -> option_default
-    | Ast.Dots(d,whn,_,_) | Ast.Circles(d,whn,_,_) | Ast.Stars(d,whn,_,_) ->
-	option_default
+    | Ast.Dots(d,whn,_,_) -> option_default
     | _ -> k s in
 
   V.combiner bind option_default
@@ -626,94 +573,6 @@ let do_get_constants constants keywords env neg_pos =
 
 (* ------------------------------------------------------------------------ *)
 
-let filter_combine combine to_drop =
-  let rec and_loop = function
-      Elem x when List.mem x to_drop -> True
-    | Or l ->  List.fold_left build_or  False (List.map or_loop l)
-    | x -> x
-  and or_loop = function
-      Elem x when List.mem x to_drop -> True
-    | And l -> List.fold_left build_and True  (List.map and_loop l)
-    | x -> x in
- or_loop combine
-
-(* ------------------------------------------------------------------------ *)
-
-let get_all_constants minus_only =
-  let donothing r k e = k e in
-  let bind = Common.union_set in
-  let option_default = [] in
-  let mcode r (x,_,mcodekind,_) =
-    match mcodekind with
-      Ast.MINUS(_,_,_,_) -> [x]
-    | _ when minus_only -> []
-    | _ -> [x] in
-  let other r _ = [] in
-
-  V.combiner bind option_default
-    other mcode other other other other other other other other other other
-    other other
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing
-
-(* ------------------------------------------------------------------------ *)
-
-let get_plus_constants =
-  let donothing r k e = k e in
-  let bind = Common.union_set in
-  let option_default = [] in
-
-  let recurse l =
-    List.fold_left
-      (List.fold_left
-	 (function prev ->
-	   function cur ->
-	     bind ((get_all_constants false).V.combiner_anything cur) prev))
-      [] l in
-  let process_mcodekind = function
-      Ast.MINUS(_,_,_,Ast.REPLACEMENT(anythings,_)) -> recurse anythings
-    | Ast.CONTEXT(_,Ast.BEFORE(a,_)) -> recurse a
-    | Ast.CONTEXT(_,Ast.AFTER(a,_)) -> recurse a
-    | Ast.CONTEXT(_,Ast.BEFOREAFTER(a1,a2,_)) ->
-	Common.union_set (recurse a1) (recurse a2)
-    | _ -> [] in
-
-  let mcode r mc = process_mcodekind (Ast.get_mcodekind mc) in
-  let end_info (_,_,_,mc) = process_mcodekind mc in
-
-  let annotated_decl decl =
-    match Ast.unwrap decl with
-      Ast.DElem(bef,_,_) -> bef
-    | _ -> failwith "not possible" in
-
-  let rule_elem r k e =
-    match Ast.unwrap e with
-      Ast.FunHeader(bef,_,_,_,_,_,_,_) -> bind (process_mcodekind bef) (k e)
-    | Ast.Decl decl ->
-	bind (process_mcodekind (annotated_decl decl)) (k e)
-    | Ast.ForHeader(fr,lp,Ast.ForDecl(decl),e2,sem2,e3,rp) ->
-	bind (process_mcodekind (annotated_decl decl)) (k e)
-    | _ -> k e in
-
-  let statement r k e =
-    match Ast.unwrap e with
-      Ast.IfThen(_,_,ei) | Ast.IfThenElse(_,_,_,_,ei)
-    | Ast.While(_,_,ei)  | Ast.For(_,_,ei)
-    | Ast.Iterator(_,_,ei) -> bind (k e) (end_info ei)
-    | _ -> k e in
-
-  V.combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode mcode mcode mcode
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing rule_elem statement donothing
-    donothing donothing
-
-(* ------------------------------------------------------------------------ *)
-
 (* true means the rule should be analyzed, false means it should be ignored *)
 let rec dependencies env = function
     Ast.Dep s -> (try List.assoc s env with Not_found -> False)
@@ -792,48 +651,40 @@ let all_context =
 
 (* ------------------------------------------------------------------------ *)
 
-let rule_fn nm tls exact_dependencies in_plus env neg_pos =
+(* The whole "in_plus" idea is flawed.  If something is added in one rule and
+matched in a later one, we want to include files that originally contain
+the thing, so no point to keep track of what is added by earlier rules.
+The situation is something like a -> b v (b & c).  We don't actually need
+both b and c, but if we don't have b, then the only way that we can get it is
+fro the first rule matching, in which case the formula is already true. *)
+let rule_fn nm tls exact_dependencies env neg_pos =
   (* tls seems like it is supposed to relate to multiple minirules.  If we
      were to actually allow that, then the following could be inefficient,
      because it could run sat on the same rule name (x) more than once. *)
-  let relevant_in_plus =
-    List.fold_left Common.union_set []
-      (List.map snd
-	 (List.filter
-	    (function (x,_) -> sat (build_and (Elem x) exact_dependencies))
-	    in_plus)) in
   List.fold_left
-    (function (rest_info,in_plus) ->
+    (function rest_info ->
       function (cur,neg_pos) ->
 	let minuses =
 	  let getter = do_get_constants keep drop env neg_pos in
 	  getter.V.combiner_top_level cur in
-	let all_minuses =
-	  if !Flag.sgrep_mode2
-	  then [] (* nothing removed for sgrep *)
-	  else (get_all_constants true).V.combiner_top_level cur in
-	let plusses = get_plus_constants.V.combiner_top_level cur in
 	(* the following is for eg -foo(2) +foo(x) then in another rule
 	   -foo(10); don't want to consider that foo is guaranteed to be
 	   created by the rule.  not sure this works completely: what if foo is
 	   in both - and +, but in an or, so the cases aren't related?
 	   not sure this whole thing is a good idea.  how do we know that
 	   something that is only in plus is really freshly created? *)
-	let plusses = Common.minus_set plusses all_minuses in
 	let was_bot = minuses = True in
-	let new_minuses = filter_combine minuses relevant_in_plus in
-	let new_plusses = (nm,plusses) :: in_plus in
 	(* perhaps it should be build_and here?  we don't really have multiple
 	   minirules anymore anyway. *)
-	match new_minuses with
+	match minuses with
 	  True ->
 	    let getter = do_get_constants drop keep env neg_pos in
 	    let retry = getter.V.combiner_top_level cur in
 	    (match retry with
-	      True when not was_bot -> (rest_info, new_plusses)
-	    | x -> (build_or x rest_info, new_plusses))
-	| x -> (build_or x rest_info, new_plusses))
-    (False,in_plus) (List.combine tls neg_pos)
+	      True when not was_bot -> rest_info
+	    | x -> build_or x rest_info)
+	| x -> build_or x rest_info)
+    False (List.combine tls neg_pos)
 
 let debug_deps nm deps res =
   if !Flag_parsing_cocci.debug_parse_cocci
@@ -847,15 +698,15 @@ let debug_deps nm deps res =
     end
 
 let run rules neg_pos_vars =
-  let (info,_,_,_) =
+  let (info,_,_) =
     List.fold_left
-      (function (rest_info,in_plus,env,locals(*dom of env*)) ->
+      (function (rest_info,env,locals(*dom of env*)) ->
         function
 	    (Ast.ScriptRule (nm,_,deps,mv,_,_),_) ->
 	      let extra_deps =
 		List.fold_left
 		  (function prev ->
-		    function (_,(rule,_),_) ->
+		    function (_,(rule,_),_,_) ->
 		      if rule = "virtual"
 		      then prev
 		      else Ast.AndDep (Ast.Dep rule,prev))
@@ -864,45 +715,54 @@ let run rules neg_pos_vars =
 	      debug_deps nm extra_deps dependencies;
 	      (match dependencies with
 		False ->
-		  (rest_info, in_plus, (nm,True)::env, nm::locals)
+		  (rest_info, (nm,True)::env, nm::locals)
 	      | dependencies ->
-		  (build_or dependencies rest_info, in_plus, env, locals))
+		  (build_or dependencies rest_info, env, locals))
           | (Ast.InitialScriptRule (_,_,deps,_,_),_)
 	  | (Ast.FinalScriptRule (_,_,deps,_,_),_) ->
 	      (* initialize and finalize dependencies are irrelevant to
 		 get_constants *)
 	      (* only possible metavariables are virtual *)
-	      (rest_info, in_plus, env, locals)
+	      (rest_info, env, locals)
           | (Ast.CocciRule (nm,(dep,_,_),cur,_,_),neg_pos_vars) ->
 	      let dependencies = dependencies env dep in
 	      let exact_dependencies = exact_dependencies dep in
-	      let (cur_info,cur_plus) =
-		rule_fn nm cur exact_dependencies in_plus ((nm,True)::env)
+	      let cur_info =
+		rule_fn nm cur exact_dependencies ((nm,True)::env)
 		  neg_pos_vars in
 	      debug_deps nm dep dependencies;
 	      (match dependencies with
-		False -> (rest_info,cur_plus,env,locals)
+		False -> (rest_info,env,locals)
 	      | dependencies ->
 		  let re_cur_info = build_and dependencies cur_info in
 		  if List.for_all all_context.V.combiner_top_level cur
-		  then (rest_info,cur_plus,(nm,re_cur_info)::env,nm::locals)
+		  then (rest_info,(nm,re_cur_info)::env,nm::locals)
 		  else
 		    (* no constants if dependent on another rule; then we need
 		       to find the constants of that rule *)
 		    (* why does env not use re_cur_info? *)
 		    (build_or re_cur_info rest_info,
-		     cur_plus,(nm,cur_info)::env,locals)))
-      (False,[],[],[])
+		     (nm,cur_info)::env,locals)))
+      (False,[],[])
       (List.combine (rules : Ast.rule list) neg_pos_vars) in
   info
-    
+
+(* The return value is a tuple of four components.
+1. A list of all words, regardless of & and |, for use with grep (or only)
+2. A list of single strings using the glimpse ; and ,  operators
+3. A triple of 1 and of a CNF representation, both as regexps, and of the
+CNF as a list of git grep strings.  coccigrep uses 1 for basic scanning and
+then the CNF regexp for more refined scanning.  git grep uses the second
+CNF representation.
+4. An arbitrary formula, usable by the support for idutils *)
+
 let get_constants rules neg_pos_vars =
   if !Flag.worth_trying_opt
-  then 
+  then
     begin
     let res = run rules neg_pos_vars in
     let grep = interpret_grep true res in (* useful because in string form *)
-    let coccigrep = interpret_cocci_grep true res in
+    let coccigrep = interpret_cocci_git_grep true res in
     match !Flag.scanner with
       Flag.NoScanner ->
 	(grep,None,coccigrep,None)
@@ -910,7 +770,6 @@ let get_constants rules neg_pos_vars =
 	(grep,interpret_glimpse true res,coccigrep,None)
     | Flag.IdUtils ->
 	(grep,None,coccigrep,Some res)
-    | Flag.CocciGrep -> (grep,None,coccigrep,None)
+    | Flag.CocciGrep | Flag.GitGrep -> (grep,None,coccigrep,None)
     end
   else (None,None,None,None)
-
