@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./parse_aux.ml"
 (* exports everything, used only by parser_cocci_menhir.mly *)
 module Ast0 = Ast0_cocci
 module Ast = Ast_cocci
@@ -92,10 +71,6 @@ let clt2mcode_ext str isSymbol = function
       (str,Ast0.OPT,
        make_info line lline llineend offset col strbef straft isSymbol ws,
        Ast0.MINUS(ref(Ast.NOREPLACEMENT,Ast0.default_token_info)),ref pos,-1)
-  | (Data.UNIQUEMINUS,line,lline,llineend,offset,col,strbef,straft,pos,ws) ->
-      (str,Ast0.UNIQUE,
-       make_info line lline llineend offset col strbef straft isSymbol ws,
-       Ast0.MINUS(ref(Ast.NOREPLACEMENT,Ast0.default_token_info)),ref pos,-1)
   | (Data.PLUS,line,lline,llineend,offset,col,strbef,straft,pos,ws)        ->
       (str,Ast0.NONE,
        make_info line lline llineend offset col strbef straft isSymbol ws,
@@ -116,12 +91,6 @@ let clt2mcode_ext str isSymbol = function
        Ast0.CONTEXT(ref(Ast.NOTHING,
 			Ast0.default_token_info,Ast0.default_token_info)),
        ref pos,-1)
-  | (Data.UNIQUE,line,lline,llineend,offset,col,strbef,straft,pos, ws)     ->
-      (str,Ast0.UNIQUE,
-       make_info line lline llineend offset col strbef straft isSymbol ws,
-       Ast0.CONTEXT(ref(Ast.NOTHING,
-			Ast0.default_token_info,Ast0.default_token_info)),
-       ref pos,-1)
 
 let clt2mcode name clt = clt2mcode_ext name false clt
 let id2name   (name, clt) = name
@@ -132,21 +101,16 @@ let sym2mcode (name, clt) = clt2mcode_ext name true clt
 let mkdots str (dot,whencode) =
   match str with
     "..." -> Ast0.wrap(Ast0.Dots(clt2mcode str dot, whencode))
-  | "ooo" -> Ast0.wrap(Ast0.Circles(clt2mcode str dot, whencode))
-  | "***" -> Ast0.wrap(Ast0.Stars(clt2mcode str dot, whencode))
   | _ -> failwith "cannot happen"
 
 let mkedots str (dot,whencode) =
   match str with
     "..." -> Ast0.wrap(Ast0.Edots(clt2mcode str dot, whencode))
-  | "ooo" -> Ast0.wrap(Ast0.Ecircles(clt2mcode str dot, whencode))
-  | "***" -> Ast0.wrap(Ast0.Estars(clt2mcode str dot, whencode))
   | _ -> failwith "cannot happen"
 
 let mkdpdots str dot =
   match str with
     "..." -> Ast0.wrap(Ast0.DPdots(clt2mcode str dot))
-  | "ooo" -> Ast0.wrap(Ast0.DPcircles(clt2mcode str dot))
   | _ -> failwith "cannot happen"
 
 let mkidots str (dot,whencode) =
@@ -168,7 +132,6 @@ let mkddots_one str (dot,whencode) =
 let mkpdots str dot =
   match str with
     "..." -> Ast0.wrap(Ast0.Pdots(clt2mcode str dot))
-  | "ooo" -> Ast0.wrap(Ast0.Pcircles(clt2mcode str dot))
   | _ -> failwith "cannot happen"
 
 let arith_op ast_op left op right =
@@ -182,17 +145,7 @@ let logic_op ast_op left op right =
 let make_cv cv ty =
   match cv with None -> ty | Some x -> Ast0.wrap (Ast0.ConstVol(x,ty))
 
-let top_dots l =
-  let circle x =
-    match Ast0.unwrap x with Ast0.Circles(_) -> true | _ -> false in
-  let star x =
-    match Ast0.unwrap x with Ast0.Stars(_) -> true | _ -> false in
-  if List.exists circle l
-  then Ast0.wrap(Ast0.CIRCLES(l))
-  else
-    if List.exists star l
-    then Ast0.wrap(Ast0.STARS(l))
-    else Ast0.wrap(Ast0.DOTS(l))
+let top_dots l = Ast0.wrap l
 
 (* here the offset is that of the first in the sequence of *s, not that of
 each * individually *)
@@ -241,6 +194,14 @@ let lookup rule name =
       raise
 	(Semantic_cocci.Semantic("bad rule "^rule^" or bad variable "^name))
 
+let meta_lookup rule name v =
+  match lookup rule name with
+    Ast.MetaScriptDecl(cell,_) ->
+      (match !cell with
+	Some x -> x
+      | None -> cell := Some v; v)
+  | res -> res
+
 let check_meta_tyopt type_irrelevant v =
   let fail name =
     raise
@@ -248,11 +209,11 @@ let check_meta_tyopt type_irrelevant v =
 	 ("incompatible inheritance declaration "^name)) in
   match v with
     Ast.MetaMetaDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaMetaDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaIdDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaIdDecl(_,_) | Ast.MetaFreshIdDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaFreshIdDecl((rule,name),seed) ->
@@ -260,84 +221,84 @@ let check_meta_tyopt type_irrelevant v =
 	(Semantic_cocci.Semantic
 	   "can't inherit the freshness of an identifier")
   | Ast.MetaTypeDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaTypeDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaInitDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaInitDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaInitListDecl(Ast.NONE,(rule,name),len_name) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaInitListDecl(_,_,_) -> ()
       | _ -> fail name)
   | Ast.MetaListlenDecl((rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaListlenDecl(_) -> ()
       | _ -> fail name)
   | Ast.MetaParamDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaParamDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaParamListDecl(Ast.NONE,(rule,name),len_name) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaParamListDecl(_,_,_) -> ()
       | _ -> fail name)
   | Ast.MetaConstDecl(Ast.NONE,(rule,name),ty) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaConstDecl(_,_,ty1) when type_irrelevant || ty = ty1 -> ()
       | _ -> fail name)
   | Ast.MetaErrDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaErrDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaExpDecl(Ast.NONE,(rule,name),ty) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaExpDecl(_,_,ty1) when type_irrelevant || ty = ty1 -> ()
       | _ -> fail name)
   | Ast.MetaIdExpDecl(Ast.NONE,(rule,name),ty) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaIdExpDecl(_,_,ty1) when type_irrelevant || ty = ty1 -> ()
       | _ -> fail name)
   | Ast.MetaLocalIdExpDecl(Ast.NONE,(rule,name),ty) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaLocalIdExpDecl(_,_,ty1) when type_irrelevant || ty = ty1 -> ()
       | _ -> fail name)
   | Ast.MetaExpListDecl(Ast.NONE,(rule,name),len_name) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaExpListDecl(_,_,_) -> ()
       | Ast.MetaParamListDecl(_,_,_) when not (!Flag.make_hrule = None) -> ()
       | _ -> fail name)
   | Ast.MetaDeclDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaDeclDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaFieldDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaFieldDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaFieldListDecl(Ast.NONE,(rule,name),len_name) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaFieldListDecl(_,_,_) -> ()
       | _ -> fail name)
   | Ast.MetaStmDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaStmDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaStmListDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaStmListDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaFuncDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaFuncDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaLocalFuncDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaLocalFuncDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaPosDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaPosDecl(_,_) ->
 	  if not (List.mem rule !Data.inheritable_positions) &&
 	    not !Data.ignore_patch_or_match
@@ -347,26 +308,26 @@ let check_meta_tyopt type_irrelevant v =
 		 ("position cannot be inherited over modifications: "^name))
       | _ -> fail name)
   | Ast.MetaFmtDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaFmtDecl(_,_) -> ()
       | _ -> fail name)
   | Ast.MetaFragListDecl(Ast.NONE,(rule,name),len) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaFragListDecl(_,_,_) -> ()
       | _ -> fail name)
   | Ast.MetaAnalysisDecl(analyzer,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaAnalysisDecl(analyzer1,_) ->
 	  if analyzer = analyzer1
 	  then ()
 	  else fail name
       | _ -> fail name)
   | Ast.MetaDeclarerDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaDeclarerDecl(Ast.NONE,(rule,name)) -> ()
       | _ -> fail name)
   | Ast.MetaIteratorDecl(Ast.NONE,(rule,name)) ->
-      (match lookup rule name with
+      (match meta_lookup rule name v with
 	Ast.MetaIteratorDecl(Ast.NONE,(rule,name)) -> ()
       | _ -> fail name)
   | _ ->
@@ -539,8 +500,7 @@ let switch s lp e rp lb d c rb =
       d in
   Ast0.wrap(Ast0.Switch(clt2mcode "switch" s,clt2mcode "(" lp,e,
 			clt2mcode ")" rp,clt2mcode "{" lb,
-			Ast0.wrap(Ast0.DOTS(d)),
-			Ast0.wrap(Ast0.DOTS(c)),clt2mcode "}" rb))
+			Ast0.wrap d,Ast0.wrap c,clt2mcode "}" rb))
 
 let ret_exp r e pv =
   Ast0.wrap(Ast0.ReturnExpr(clt2mcode "return" r,e,clt2mcode ";" pv))
@@ -627,24 +587,21 @@ let struct_initializer initlist =
     match Ast0.unwrap i with
       Ast0.InitGccExt _ -> true
     | Ast0.InitGccName _ -> true
-    | Ast0.OptIni i | Ast0.UniqueIni i -> loop i
+    | Ast0.OptIni i -> loop i
     | Ast0.MetaInit _ | Ast0.MetaInitList _ -> false (* ambiguous... *)
     | _ -> false in
-  let l = Ast0.undots initlist in
+  let l = Ast0.unwrap initlist in
   (l = []) || (List.exists loop l)
 
 let drop_dot_commas initlist =
-  match Ast0.unwrap initlist with
-    Ast0.DOTS(l) ->
-      let rec loop after_comma = function
-	  [] -> []
-	| x::xs ->
-	    (match Ast0.unwrap x with
-	      Ast0.Idots(dots,whencode) -> x :: (loop true xs)
-	    | Ast0.IComma(comma) when after_comma -> (*drop*) loop false xs
-	    | _ -> x :: (loop false xs)) in
-      Ast0.rewrap initlist (Ast0.DOTS(loop false l))
-  | _ -> failwith "not supported"
+  let rec loop after_comma = function
+      [] -> []
+    | x::xs ->
+	(match Ast0.unwrap x with
+	  Ast0.Idots(dots,whencode) -> x :: (loop true xs)
+	| Ast0.IComma(comma) when after_comma -> (*drop*) loop false xs
+	| _ -> x :: (loop false xs)) in
+  Ast0.rewrap initlist (loop false (Ast0.unwrap initlist))
 
 (* ----------------------------------------------------------------------- *)
 (* strings *)
@@ -833,7 +790,7 @@ let parse_string str ((mc,b,c,d,e,f,g,h,i,_) as clt) =
 		from first*)
 	     let clt = (mc,b,c,d,e,f,g,h,i,"") in
 	     let (line,middle) = drop_minus_plus str clt in
-	     let middle = Ast0.wrap (Ast0.DOTS middle) in
+	     let middle = Ast0.wrap middle in
 	     let last = clt2mcode "\"" (update_line clt (line-1)) in
 	     contains_string_constant := true;
 	     Ast0.wrap(Ast0.StringConstant(first,middle,last))
