@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./single_statement.ml"
 (* detect statements that are between dots in the minus code, because they
 may need a special treatment if they are if branches *)
 
@@ -38,12 +17,12 @@ module VT0 = Visitor_ast0_types
 (* Helpers *)
 
 let left_dots f l =
-  match Ast0.undots l with
+  match Ast0.unwrap l with
     [] -> false
   | x::xs -> f x
 
 let right_dots f l =
-  match List.rev (Ast0.undots l) with
+  match List.rev (Ast0.unwrap l) with
     [] -> false
   | x::xs -> f x
 
@@ -122,7 +101,6 @@ let rec left_ident i =
   | Ast0.MetaLocalFunc(name,_,_) -> modif_before_mcode name
   | Ast0.DisjId(_,id_list,_,_) -> List.exists left_ident id_list
   | Ast0.OptIdent(id) -> left_ident id
-  | Ast0.UniqueIdent(id) -> left_ident id
   | Ast0.AsIdent _ -> failwith "not possible"
 
 let rec right_ident i =
@@ -134,7 +112,6 @@ let rec right_ident i =
   | Ast0.MetaLocalFunc(name,_,_) -> modif_after_mcode name
   | Ast0.DisjId(_,id_list,_,_) -> List.exists right_ident id_list
   | Ast0.OptIdent(id) -> right_ident id
-  | Ast0.UniqueIdent(id) -> right_ident id
   | Ast0.AsIdent _ -> failwith "not possible"
 
 (* --------------------------------------------------------------------- *)
@@ -169,12 +146,12 @@ let rec left_expression e =
   | Ast0.MetaExprList(name,_,_) -> modif_before_mcode name
   | Ast0.EComma(cm) -> modif_before_mcode cm
   | Ast0.DisjExpr(_,exp_list,_,_) -> List.exists left_expression exp_list
+  | Ast0.ConjExpr(_,exp_list,_,_) -> List.exists left_expression exp_list
   | Ast0.NestExpr(starter,expr_dots,ender,_,multi) ->
       left_dots left_expression expr_dots
-  | Ast0.Edots(dots,_) | Ast0.Ecircles(dots,_) | Ast0.Estars(dots,_) -> false
+  | Ast0.Edots(dots,_) -> false
   | Ast0.OptExp(exp) -> left_expression exp
-  | Ast0.UniqueExp(exp) -> left_expression exp
-  | Ast0.AsExpr _ -> failwith "not possible"
+  | Ast0.AsExpr _ | Ast0.AsSExpr _ -> failwith "not possible"
 
 (* --------------------------------------------------------------------- *)
 (* Types *)
@@ -198,7 +175,6 @@ and left_typeC t =
   | Ast0.MetaType(name,_) -> modif_before_mcode name
   | Ast0.DisjType(lp,types,mids,rp) -> List.exists left_typeC types
   | Ast0.OptType(ty) -> left_typeC ty
-  | Ast0.UniqueType(ty) -> left_typeC ty
   | Ast0.AsType _ -> failwith "not possible"
 
 (* --------------------------------------------------------------------- *)
@@ -219,14 +195,16 @@ and left_declaration d =
   | Ast0.FunProto(fninfo,name,lp1,params,va,rp1,sem) ->
       (* should not be nested in anything anyway *)
       false
-  | Ast0.MacroDecl(name,lp,args,rp,sem) -> left_ident name
-  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) -> left_ident name
+  | Ast0.MacroDecl(Some stg,name,lp,args,rp,sem) -> modif_before_mcode stg
+  | Ast0.MacroDecl(None,name,lp,args,rp,sem) -> left_ident name
+  | Ast0.MacroDeclInit(Some stg,name,lp,args,rp,eq,ini,sem) ->
+      modif_before_mcode stg
+  | Ast0.MacroDeclInit(None,name,lp,args,rp,eq,ini,sem) -> left_ident name
   | Ast0.TyDecl(ty,sem) -> left_typeC ty
   | Ast0.Typedef(stg,ty,id,sem) -> modif_before_mcode stg
   | Ast0.DisjDecl(_,decls,_,_) -> List.exists left_declaration decls
   | Ast0.Ddots(dots,_) -> false
   | Ast0.OptDecl(decl) -> left_declaration decl
-  | Ast0.UniqueDecl(decl) -> left_declaration decl
   | Ast0.AsDecl _ -> failwith "not possible"
 
 and right_declaration d =
@@ -238,14 +216,13 @@ and right_declaration d =
   | Ast0.Init(_,ty,id,eq,ini,sem) -> modif_after_mcode sem
   | Ast0.UnInit(_,ty,id,sem) -> modif_after_mcode sem
   | Ast0.FunProto(fninfo,name,lp1,params,va,rp1,sem) -> modif_after_mcode sem
-  | Ast0.MacroDecl(name,lp,args,rp,sem) -> modif_after_mcode sem
-  | Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) -> modif_after_mcode sem
+  | Ast0.MacroDecl(_,name,lp,args,rp,sem) -> modif_after_mcode sem
+  | Ast0.MacroDeclInit(_,name,lp,args,rp,eq,ini,sem) -> modif_after_mcode sem
   | Ast0.TyDecl(ty,sem) -> modif_after_mcode sem
   | Ast0.Typedef(stg,ty,id,sem) -> modif_after_mcode sem
   | Ast0.DisjDecl(_,decls,_,_) -> List.exists right_declaration decls
   | Ast0.Ddots(dots,_) -> false
   | Ast0.OptDecl(decl) -> right_declaration decl
-  | Ast0.UniqueDecl(decl) -> right_declaration decl
   | Ast0.AsDecl _ -> failwith "not possible"
 
 (* --------------------------------------------------------------------- *)
@@ -286,14 +263,14 @@ and left_statement s =
   | Ast0.Exp(exp) -> false (* can only be replaced by an expression *)
   | Ast0.TopExp(exp) -> false (* as above *)
   | Ast0.Ty(ty) -> false (* can only be replaced by a type *)
+  | Ast0.TopId(id) -> false (* can only be replaced by an ident *)
   | Ast0.TopInit(init) -> false (* can only be replaced by an init *)
-  | Ast0.Dots(d,whn) | Ast0.Circles(d,whn) | Ast0.Stars(d,whn) -> false
+  | Ast0.Dots(d,whn) -> false
   | Ast0.Include(inc,s) -> modif_before_mcode inc
   | Ast0.Undef(def,id) -> modif_before_mcode def
   | Ast0.Define(def,id,params,body) -> modif_before_mcode def
   | Ast0.Pragma(prg,id,body) -> modif_before_mcode prg
   | Ast0.OptStm(re) -> left_statement re
-  | Ast0.UniqueStm(re) -> left_statement re
   | Ast0.AsStmt _ -> failwith "not possible"
 
 and right_statement s =
@@ -330,14 +307,14 @@ and right_statement s =
   | Ast0.Exp(exp) -> false (* can only be replaced by an expression *)
   | Ast0.TopExp(exp) -> false (* as above *)
   | Ast0.Ty(ty) -> false (* can only be replaced by a type *)
+  | Ast0.TopId(id) -> false (* can only be replaced by a type *)
   | Ast0.TopInit(init) -> false (* can only be replaced by an init *)
-  | Ast0.Dots(d,whn) | Ast0.Circles(d,whn) | Ast0.Stars(d,whn) -> false
+  | Ast0.Dots(d,whn) -> false
   | Ast0.Include(inc,s) -> modif_after_mcode s
   | Ast0.Undef(def,id) -> right_ident id
   | Ast0.Define(def,id,params,body) -> right_dots right_statement body
   | Ast0.Pragma(prg,id,body) -> right_pragma body -- not defined, b/c not used
   | Ast0.OptStm(re) -> right_statement re
-  | Ast0.UniqueStm(re) -> right_statement re
   | Ast0.AsStmt _ -> failwith "not possible"
 *)
 
@@ -384,7 +361,7 @@ and contains_only_minus =
 
   let dots r k e =
     match Ast0.unwrap e with
-      Ast0.DOTS([]) | Ast0.CIRCLES([]) | Ast0.STARS([]) -> true
+      [] -> true
     | _ -> k e in
 
   let identifier r k e =
@@ -397,7 +374,8 @@ and contains_only_minus =
   let expression r k e =
     mcodekind (Ast0.get_mcodekind e) &&
     match Ast0.unwrap e with
-      Ast0.DisjExpr(starter,expr_list,mids,ender) ->
+      Ast0.DisjExpr(starter,expr_list,mids,ender)
+    | Ast0.ConjExpr(starter,expr_list,mids,ender) ->
 	List.for_all r.VT0.combiner_rec_expression expr_list
     | _ -> k e in
 
@@ -514,8 +492,7 @@ let add_braces orig_s =
 
 let is_dots x =
   match Ast0.unwrap x with
-    Ast0.Dots(_,_) | Ast0.Circles(_,_) | Ast0.Stars(_,_)
-  | Ast0.Nest(_,_,_,_,_) -> true
+    Ast0.Dots(_,_) | Ast0.Nest(_,_,_,_,_) -> true
   | _ -> false
 
 let all_minus s =
@@ -540,10 +517,8 @@ let rec do_branch s =
 	  List.map
 	    (function s ->
 	      match Ast0.unwrap s with
-		Ast0.DOTS([s]) ->
-		  Ast0.rewrap s (Ast0.DOTS([do_branch s]))
-	      |	Ast0.DOTS(_) -> s
-	      |	_ -> failwith "not supported")
+		[s] -> Ast0.rewrap s [do_branch s]
+	      |	_ -> s)
 	    statement_dots_list in
 	Ast0.rewrap s (Ast0.Disj(starter,stmts,mids,ender))
     | _ -> s
@@ -610,8 +585,7 @@ let rec statement dots_before dots_after s =
 	(Ast0.rewrap s
 	   (Ast0.Switch(switch,lp,exp,rp,lb,decls,
 			Ast0.rewrap cases
-			  (Ast0.DOTS
-			     (List.map case_line (Ast0.undots cases))),
+			  (List.map case_line (Ast0.unwrap cases)),
 			rb)))
   | Ast0.Break(br,sem) -> do_one s
   | Ast0.Continue(cont,sem) -> do_one s
@@ -628,6 +602,15 @@ let rec statement dots_before dots_after s =
 		   List.map (statement_dots dots_before dots_after)
 		     statement_dots_list,
 		   mids,ender))
+  | Ast0.Conj(starter,statement_dots_list,mids,ender) ->
+      (match statement_dots_list with
+	s1::ss ->
+	  Ast0.rewrap s
+	    (Ast0.Conj(starter,
+		       (statement_dots dots_before dots_after s1) ::
+		       (List.map (statement_dots false false) ss),
+		       mids,ender))
+      |	_ -> s)
   | Ast0.Nest(starter,stmt_dots,ender,whencode,multi) ->
       (match Ast0.get_mcode_mcodekind starter with
 	Ast0.MINUS _ -> (* everything removed, like -... *) s
@@ -639,8 +622,9 @@ let rec statement dots_before dots_after s =
   | Ast0.Exp(exp) -> s
   | Ast0.TopExp(exp) -> s
   | Ast0.Ty(ty) -> s
+  | Ast0.TopId(id) -> s
   | Ast0.TopInit(init) -> s
-  | Ast0.Dots(d,whn) | Ast0.Circles(d,whn) | Ast0.Stars(d,whn) -> s
+  | Ast0.Dots(d,whn) -> s
   | Ast0.Include(inc,string) -> s (* doesn't affect the need for braces *)
   | Ast0.Undef(def,id) -> s (* same as include *)
   | Ast0.Define(def,id,params,body) -> s (* same as include *)
@@ -648,9 +632,6 @@ let rec statement dots_before dots_after s =
   | Ast0.OptStm(re) ->
       Ast0.rewrap s
 	(Ast0.OptStm(statement dots_before dots_after re))
-  | Ast0.UniqueStm(re) ->
-      Ast0.rewrap s
-	(Ast0.UniqueStm(statement dots_before dots_after re))
   | Ast0.AsStmt _ -> failwith "not possible"
 
 and case_line c =
@@ -677,14 +658,7 @@ and do_statement_dots dots_before dots_after = function
       do_statement_dots false dots_after rest
 
 and statement_dots dots_before dots_after d =
-  Ast0.rewrap d
-    (match Ast0.unwrap d with
-      Ast0.DOTS(l) ->
-	Ast0.DOTS(do_statement_dots dots_before dots_after l)
-    | Ast0.CIRCLES(l) ->
-	Ast0.CIRCLES(do_statement_dots dots_before dots_after l)
-    | Ast0.STARS(l) ->
-	Ast0.STARS(do_statement_dots dots_before dots_after l))
+  Ast0.rewrap d (do_statement_dots dots_before dots_after (Ast0.unwrap d))
 
 let top_level t =
   Ast0.rewrap t
