@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./unify_ast.ml"
 (* --------------------------------------------------------------------- *)
 (* Given two patterns, A and B, determine whether B can match any matched
 subterms of A.  For simplicity, this doesn't maintain an environment; it
@@ -84,6 +63,7 @@ let disjunct_bindings b1 b2 =
   match b1 with MAYBE -> b1 | NO -> b2
 
 let disjunct_all_bindings = List.fold_left disjunct_bindings NO
+let conjunct_all_bindings = List.fold_left conjunct_bindings MAYBE
 
 (* --------------------------------------------------------------------- *)
 
@@ -100,15 +80,11 @@ let unify_lists fn dfn la lb =
   loop (la,lb)
 
 let unify_dots fn dfn d1 d2 =
-  match (Ast.unwrap d1,Ast.unwrap d2) with
-    (Ast.DOTS(l1),Ast.DOTS(l2))
-  | (Ast.CIRCLES(l1),Ast.CIRCLES(l2))
-  | (Ast.STARS(l1),Ast.STARS(l2)) -> unify_lists fn dfn l1 l2
-  | _ -> return false
+  unify_lists fn dfn (Ast.unwrap d1) (Ast.unwrap d2)
 
 let edots e =
   match Ast.unwrap e with
-    Ast.Edots(_,_) | Ast.Ecircles(_,_) | Ast.Estars(_,_) -> true
+    Ast.Edots(_,_) -> true
   | _ -> false
 
 let ddots e =
@@ -118,17 +94,17 @@ let ddots e =
 
 let pdots p =
   match Ast.unwrap p with
-    Ast.Pdots(_) | Ast.Pcircles(_) -> true
+    Ast.Pdots(_) -> true
   | _ -> false
 
 let dpdots e =
   match Ast.unwrap e with
-    Ast.DPdots(_) | Ast.DPcircles(_) -> true
+    Ast.DPdots(_) -> true
   | _ -> false
 
 let sdots s =
   match Ast.unwrap s with
-    Ast.Dots(_,_,_,_) | Ast.Circles(_,_,_,_) | Ast.Stars(_,_,_,_) -> true
+    Ast.Dots(_,_,_,_) -> true
   | _ -> false
 
 let idots e =
@@ -173,9 +149,7 @@ let rec unify_ident i1 i2 =
       disjunct_all_bindings (List.map (function x -> unify_ident i1 x) i2)
 
   | (Ast.OptIdent(_),_)
-  | (Ast.UniqueIdent(_),_)
-  | (_,Ast.OptIdent(_))
-  | (_,Ast.UniqueIdent(_)) -> failwith "unsupported ident"
+  | (_,Ast.OptIdent(_)) -> failwith "unsupported ident"
 
 (* --------------------------------------------------------------------- *)
 (* Expression *)
@@ -242,24 +216,28 @@ and unify_expression e1 e2 =
       disjunct_all_bindings
 	(List.map (function x -> unify_expression x e1) [exp2;asexp2])
 
+  (* no idea what to do with the statement *)
+  | (Ast.AsSExpr(exp1,asstm1),_) -> unify_expression exp1 e2
+  | (_,Ast.AsSExpr(exp2,asstm2)) -> unify_expression exp2 e2
+
   | (Ast.EComma(cm1),Ast.EComma(cm2)) -> return true
 
   | (Ast.DisjExpr(e1),_) ->
       disjunct_all_bindings (List.map (function x -> unify_expression x e2) e1)
   | (_,Ast.DisjExpr(e2)) ->
       disjunct_all_bindings (List.map (function x -> unify_expression e1 x) e2)
+  | (Ast.ConjExpr(e1),_) ->
+      conjunct_all_bindings (List.map (function x -> unify_expression x e2) e1)
+  | (_,Ast.ConjExpr(e2)) ->
+      conjunct_all_bindings (List.map (function x -> unify_expression e1 x) e2)
   | (Ast.NestExpr(_,e1,_,_,_),Ast.NestExpr(_,e2,_,_,_)) ->
       unify_dots unify_expression edots e1 e2
 
   (* dots can match against anything.  return true to be safe. *)
-  | (Ast.Edots(_,_),_) | (_,Ast.Edots(_,_))
-  | (Ast.Ecircles(_,_),_) | (_,Ast.Ecircles(_,_))
-  | (Ast.Estars(_,_),_) | (_,Ast.Estars(_,_)) -> return true
+  | (Ast.Edots(_,_),_) | (_,Ast.Edots(_,_)) -> return true
 
   | (Ast.OptExp(_),_)
-  | (Ast.UniqueExp(_),_)
-  | (_,Ast.OptExp(_))
-  | (_,Ast.UniqueExp(_)) -> failwith "unsupported expression"
+  | (_,Ast.OptExp(_)) -> failwith "unsupported expression"
   | _ -> return false
 
 (* --------------------------------------------------------------------- *)
@@ -304,9 +282,7 @@ and unify_fullType ft1 ft2 =
       disjunct_all_bindings (List.map (function x -> unify_fullType ft1 x) ft2)
 
   | (Ast.OptType(_),_)
-  | (Ast.UniqueType(_),_)
-  | (_,Ast.OptType(_))
-  | (_,Ast.UniqueType(_)) -> failwith "unsupported type"
+  | (_,Ast.OptType(_)) -> failwith "unsupported type"
 
 and unify_typeC t1 t2 =
   match (Ast.unwrap t1,Ast.unwrap t2) with
@@ -393,15 +369,21 @@ and unify_declaration d1 d2 =
 	   (conjunct_bindings (unify_ident nm1 nm2)
 	      (unify_dots unify_parameterTypeDef pdots params1 params2))
        else return false
-  | (Ast.MacroDecl(n1,lp1,args1,rp1,sem1),
-     Ast.MacroDecl(n2,lp2,args2,rp2,sem2)) ->
-       conjunct_bindings (unify_ident n1 n2)
-	 (unify_dots unify_expression edots args1 args2)
-  | (Ast.MacroDeclInit(n1,lp1,args1,rp1,eq1,ini1,sem1),
-     Ast.MacroDeclInit(n2,lp2,args2,rp2,eq2,ini2,sem2)) ->
-       conjunct_bindings (unify_ident n1 n2)
-	 (conjunct_bindings (unify_dots unify_expression edots args1 args2)
-	    (unify_initialiser ini1 ini2))
+  | (Ast.MacroDecl(s1,n1,lp1,args1,rp1,sem1),
+     Ast.MacroDecl(s2,n2,lp2,args2,rp2,sem2)) ->
+       if bool_unify_option unify_mcode s1 s2
+       then
+	 conjunct_bindings (unify_ident n1 n2)
+	   (unify_dots unify_expression edots args1 args2)
+       else return false
+  | (Ast.MacroDeclInit(s1,n1,lp1,args1,rp1,eq1,ini1,sem1),
+     Ast.MacroDeclInit(s2,n2,lp2,args2,rp2,eq2,ini2,sem2)) ->
+       if bool_unify_option unify_mcode s1 s2
+       then
+	 conjunct_bindings (unify_ident n1 n2)
+	   (conjunct_bindings (unify_dots unify_expression edots args1 args2)
+	      (unify_initialiser ini1 ini2))
+       else return false
   | (Ast.TyDecl(ft1,s1),Ast.TyDecl(ft2,s2)) -> unify_fullType ft1 ft2
   | (Ast.Typedef(stg1,ft1,id1,s1),Ast.Typedef(stg2,ft2,id2,s2)) ->
       conjunct_bindings (unify_fullType ft1 ft2) (unify_typeC id1 id2)
@@ -413,9 +395,7 @@ and unify_declaration d1 d2 =
 	(List.map (function x -> unify_declaration d1 x) d2)
 
   | (Ast.OptDecl(_),_)
-  | (Ast.UniqueDecl(_),_)
-  | (_,Ast.OptDecl(_))
-  | (_,Ast.UniqueDecl(_)) -> failwith "unsupported decl"
+  | (_,Ast.OptDecl(_)) -> failwith "unsupported decl"
   | _ -> return false
 
 and unify_annotated_decl d1 d2 =
@@ -452,9 +432,7 @@ and unify_initialiser i1 i2 =
       conjunct_bindings (unify_ident namea nameb) (unify_initialiser inia inib)
 
   | (Ast.OptIni(_),_)
-  | (Ast.UniqueIni(_),_)
-  | (_,Ast.OptIni(_))
-  | (_,Ast.UniqueIni(_)) -> failwith "unsupported decl"
+  | (_,Ast.OptIni(_)) -> failwith "unsupported decl"
   | _ -> return false
 
 and unify_designator d1 d2 =
@@ -487,17 +465,14 @@ and unify_parameterTypeDef p1 p2 =
   | (Ast.PComma(_),Ast.PComma(_)) -> return true
 
   (* dots can match against anything.  return true to be safe. *)
-  | (Ast.Pdots(_),_) | (_,Ast.Pdots(_))
-  | (Ast.Pcircles(_),_) | (_,Ast.Pcircles(_)) -> return true
+  | (Ast.Pdots(_),_) | (_,Ast.Pdots(_)) -> return true
 
   (* not sure what to do with the asexp.... *)
   | (Ast.AsParam(param1,asexp1),_) -> unify_parameterTypeDef param1 p2
   | (_,Ast.AsParam(param2,asexp2)) -> unify_parameterTypeDef p1 param2
 
   | (Ast.OptParam(_),_)
-  | (Ast.UniqueParam(_),_)
-  | (_,Ast.OptParam(_))
-  | (_,Ast.UniqueParam(_)) -> failwith "unsupported parameter"
+  | (_,Ast.OptParam(_)) -> failwith "unsupported parameter"
   | _ -> return false
 
 (* --------------------------------------------------------------------- *)
@@ -517,13 +492,10 @@ and unify_define_param p1 p2 =
   | (Ast.DPComma(_),Ast.DPComma(_)) -> return true
 
   (* dots can match against anything.  return true to be safe. *)
-  | (Ast.DPdots(_),_) | (_,Ast.DPdots(_))
-  | (Ast.DPcircles(_),_) | (_,Ast.DPcircles(_)) -> return true
+  | (Ast.DPdots(_),_) | (_,Ast.DPdots(_)) -> return true
 
   | (Ast.OptDParam(_),_)
-  | (Ast.UniqueDParam(_),_)
-  | (_,Ast.OptDParam(_))
-  | (_,Ast.UniqueDParam(_)) -> failwith "unsupported parameter"
+  | (_,Ast.OptDParam(_)) -> failwith "unsupported parameter"
   | _ -> return false
 
 (* --------------------------------------------------------------------- *)
@@ -660,7 +632,7 @@ and subexp f =
   let recursor = V.combiner bind option_default
       mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
       mcode mcode
-      donothing donothing donothing donothing donothing donothing expr 
+      donothing donothing donothing donothing donothing donothing expr
       donothing donothing donothing donothing donothing donothing
       donothing donothing donothing donothing donothing donothing donothing
       donothing donothing in
@@ -704,17 +676,29 @@ let rec unify_statement s1 s2 =
       conjunct_bindings (unify_rule_elem h1 h2) (unify_statement s1 s2)
   | (Ast.Atomic(re1),Ast.Atomic(re2)) -> unify_rule_elem re1 re2
   | (Ast.Disj(s1),_) ->
-      let s2 = Ast.rewrap s2 (Ast.DOTS[s2]) in
+      let s2 = Ast.rewrap s2 [s2] in
       disjunct_all_bindings
 	(List.map
 	   (function x -> unify_dots unify_statement sdots x s2)
 	   s1)
   | (_,Ast.Disj(s2)) ->
-      let s1 = Ast.rewrap s1 (Ast.DOTS[s1]) in
+      let s1 = Ast.rewrap s1 [s1] in
       disjunct_all_bindings
 	(List.map
 	   (function x -> unify_dots unify_statement sdots s1 x)
 	   s2)
+  | (Ast.Conj(s1),_) ->
+      let s2 = Ast.rewrap s2 [s2] in
+      conjunct_all_bindings
+	(List.map
+	   (function x -> unify_dots unify_statement sdots x s2)
+	   s1)
+  | (_,Ast.Conj(s2)) ->
+      let s1 = Ast.rewrap s1 [s1] in
+      conjunct_all_bindings
+	(List.map
+	   (function x -> unify_dots unify_statement sdots s1 x)
+	   s2)
   | (Ast.Nest(_,s1,_,_,_,_,_),Ast.Nest(_,s2,_,_,_,_,_)) ->
       unify_dots unify_statement sdots s1 s2
   | (Ast.FunDecl(h1,lb1,s1,rb1,_),Ast.FunDecl(h2,lb2,s2,rb2,_)) ->
@@ -726,13 +710,9 @@ let rec unify_statement s1 s2 =
       conjunct_bindings (unify_rule_elem h1 h2)
 	(unify_dots unify_statement sdots s1 s2)
   (* dots can match against anything.  return true to be safe. *)
-  | (Ast.Dots(_,_,_,_),_) | (_,Ast.Dots(_,_,_,_))
-  | (Ast.Circles(_,_,_,_),_) | (_,Ast.Circles(_,_,_,_))
-  | (Ast.Stars(_,_,_,_),_) | (_,Ast.Stars(_,_,_,_)) -> return true
+  | (Ast.Dots(_,_,_,_),_) | (_,Ast.Dots(_,_,_,_)) -> return true
   | (Ast.OptStm(_),_)
-  | (Ast.UniqueStm(_),_)
-  | (_,Ast.OptStm(_))
-  | (_,Ast.UniqueStm(_)) -> failwith "unsupported statement"
+  | (_,Ast.OptStm(_)) -> failwith "unsupported statement"
   | _ -> return false
 
 let unify_statement_dots = unify_dots unify_statement sdots
