@@ -1,35 +1,15 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./unitary_ast0.ml"
 (* find unitary metavariables *)
 module Ast0 = Ast0_cocci
 module Ast = Ast_cocci
 module V0 = Visitor_ast0
 module VT0 = Visitor_ast0_types
+module TC = Type_cocci
 
 let set_minus s minus = List.filter (function n -> not (List.mem n minus)) s
 
@@ -96,12 +76,50 @@ let get_free checker t =
 	detect_unitary_frees(List.map r.VT0.combiner_rec_ident id_list)
     | _ -> k i in
 
+  let rec type_collect res = function
+      TC.ConstVol(_,ty) | TC.Pointer(ty) | TC.FunctionPointer(ty)
+    | TC.Array(ty) -> type_collect res ty
+    | TC.EnumName(TC.MV(tyname,_,_)) ->
+	bind [tyname] res
+    | TC.StructUnionName(_,TC.MV(tyname,_,_)) ->
+	bind [tyname] res
+    | TC.MetaType(tyname,_,_) ->
+	bind [tyname] res
+    | TC.Decimal(e1,e2) ->
+	let e2mv = function TC.MV(mv,_,_) -> [mv] | _ -> [] in
+	bind (e2mv e1) (e2mv e2)
+    | TC.SignedT(_,Some ty) -> type_collect res ty
+    | ty -> res in
+
+  let constraints_collect r res = function
+      Ast0.NotExpCstrt(el) ->
+	List.fold_left bind res
+	  (List.map r.VT0.combiner_rec_expression el)
+    | Ast0.SubExpCstrt(names) -> bind names res
+    | _ -> res in
+
   let expression r k e =
     match Ast0.unwrap e with
-      Ast0.MetaErr(name,_,_) | Ast0.MetaExpr(name,_,_,_,_)
+      Ast0.MetaErr(name,constraints,_) ->
+	let constraints =
+	  constraints_collect r option_default constraints in
+	bind (checker name) constraints
+    | Ast0.MetaExpr(name,constraints,type_list,_,_) ->
+	let types =
+	  match type_list with
+	    Some type_list ->
+	      List.fold_left type_collect option_default type_list
+	  | None -> option_default in
+	let constraints =
+	  constraints_collect r types constraints in
+	bind (checker name) constraints
     | Ast0.MetaExprList(name,_,_) -> checker name
     | Ast0.DisjExpr(starter,expr_list,mids,ender) ->
 	detect_unitary_frees(List.map r.VT0.combiner_rec_expression expr_list)
+    | Ast0.ConjExpr(starter,expr_list,mids,ender) ->
+	List.fold_left
+	  (fun prev cur -> bind (r.VT0.combiner_rec_expression  cur) prev)
+	  option_default expr_list
     | _ -> k e in
 
   let typeC r k t =
@@ -136,6 +154,10 @@ let get_free checker t =
     | Ast0.Disj(starter,stmt_list,mids,ender) ->
 	detect_unitary_frees
 	  (List.map r.VT0.combiner_rec_statement_dots stmt_list)
+    | Ast0.Conj(starter,stmt_list,mids,ender) ->
+	List.fold_left
+	  (fun prev cur -> bind (r.VT0.combiner_rec_statement_dots  cur) prev)
+	  option_default stmt_list
     | Ast0.Nest(starter,stmt_dots,ender,whn,multi) ->
 	bind (r.VT0.combiner_rec_statement_dots stmt_dots)
 	  (detect_unitary_frees
@@ -145,7 +167,7 @@ let get_free checker t =
 		   r.VT0.combiner_rec_statement
 		    r.VT0.combiner_rec_expression)
 		whn))
-    | Ast0.Dots(d,whn) | Ast0.Circles(d,whn) | Ast0.Stars(d,whn) ->
+    | Ast0.Dots(d,whn) ->
 	detect_unitary_frees
 	  (List.map
 	     (whencode
