@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./visitor_ast.ml"
 module Ast0 = Ast0_cocci
 module Ast = Ast_cocci
 
@@ -63,12 +42,14 @@ type ('mc,'a) cmcode = 'a combiner -> 'mc Ast_cocci.mcode -> 'a
 type ('cd,'a) ccode = 'a combiner -> ('cd -> 'a) -> 'cd -> 'a
 
 let combiner bind option_default
-    meta_mcodefn string_mcodefn const_mcodefn simpleassign_mcodefn opassign_mcodefn fix_mcodefn
+    meta_mcodefn string_mcodefn const_mcodefn simpleassign_mcodefn
+    opassign_mcodefn fix_mcodefn
     unary_mcodefn arithop_mcodefn logicalop_mcodefn
     cv_mcodefn sign_mcodefn struct_mcodefn storage_mcodefn
     inc_file_mcodefn
     expdotsfn paramdotsfn stmtdotsfn anndecldotsfn initdotsfn
-    identfn exprfn fragfn fmtfn assignOpfn binaryOpfn ftfn tyfn initfn paramfn declfn
+    identfn exprfn fragfn fmtfn assignOpfn binaryOpfn ftfn tyfn initfn
+    paramfn declfn
     annotated_declfn rulefn stmtfn casefn topfn anyfn =
   let multibind l =
     let rec loop = function
@@ -81,10 +62,7 @@ let combiner bind option_default
     | None -> option_default in
 
   let dotsfn param default all_functions arg =
-    let k d =
-      match Ast.unwrap d with
-	Ast.DOTS(l) | Ast.CIRCLES(l) | Ast.STARS(l) ->
-          multibind (List.map default l) in
+    let k d = multibind (List.map default (Ast.unwrap d)) in
     param all_functions k arg in
 
   let rec meta_mcode x = meta_mcodefn all_functions x
@@ -125,12 +103,11 @@ let combiner bind option_default
       | Ast.MetaLocalFunc(name,_,_,_) -> meta_mcode name
       | Ast.AsIdent(id,asid) ->
 (* use let-ins to ensure arg evaluation happens left-to-right *)
-	  let lid = ident id in 
+	  let lid = ident id in
 	  let lasid = ident asid in
 	  bind lid lasid
       | Ast.DisjId(id_list) -> multibind (List.map ident id_list)
-      | Ast.OptIdent(id) -> ident id
-      | Ast.UniqueIdent(id) -> ident id in
+      | Ast.OptIdent(id) -> ident id in
     identfn all_functions k i
 
   and expression e =
@@ -166,15 +143,15 @@ let combiner bind option_default
 	  let lcolon = string_mcode colon in
 	  let lexp3 = expression exp3 in
 	  multibind [lexp1; lwhy; lexp2; lcolon; lexp3]
-      | Ast.Postfix(exp,op) -> 
+      | Ast.Postfix(exp,op) ->
 	  let lexp = expression exp in
 	  let lop = fix_mcode op in
 	  bind lexp lop
       | Ast.Infix(exp,op) ->
-	  let lop = fix_mcode op in 
+	  let lop = fix_mcode op in
 	  let lexp = expression exp in
 	  bind lop lexp
-      | Ast.Unary(exp,op) -> 
+      | Ast.Unary(exp,op) ->
 	  let lop = unary_mcode op in
 	  let lexp = expression exp in
 	  bind lop lexp
@@ -235,25 +212,28 @@ let combiner bind option_default
       | Ast.MetaErr(name,_,_,_)
       | Ast.MetaExpr(name,_,_,_,_,_)
       | Ast.MetaExprList(name,_,_,_) -> meta_mcode name
-      |	Ast.AsExpr(exp,asexp) -> 
+      |	Ast.AsExpr(exp,asexp) ->
 	  let lexp = expression exp in
 	  let lasexp = expression asexp in
 	  bind lexp lasexp
+      | Ast.AsSExpr(exp,asstm) -> 
+	  let lexp = expression exp in
+	  let lasstm = rule_elem asstm in
+	  bind lexp lasstm
       | Ast.EComma(cm) -> string_mcode cm
-      | Ast.DisjExpr(exp_list) -> multibind (List.map expression exp_list)
+      | Ast.DisjExpr(exp_list) | Ast.ConjExpr(exp_list) ->
+	  multibind (List.map expression exp_list)
       | Ast.NestExpr(starter,expr_dots,ender,whncode,multi) ->
 	  let lstarter = string_mcode starter in
 	  let lexpr_dots = expression_dots expr_dots in
 	  let lender = string_mcode ender in
 	  let lwhncode = get_option expression whncode in
 	  multibind [lstarter; lexpr_dots; lender; lwhncode]
-      | Ast.Edots(dots,whncode) | Ast.Ecircles(dots,whncode)
-      | Ast.Estars(dots,whncode) ->
+      | Ast.Edots(dots,whncode) ->
 	  let ldots = string_mcode dots in
 	  let lwhncode = get_option expression whncode in
 	  bind ldots lwhncode
-      | Ast.OptExp(exp) | Ast.UniqueExp(exp) ->
-	  expression exp in
+      | Ast.OptExp(exp) -> expression exp in
     exprfn all_functions k e
 
   and assignOp op =
@@ -297,7 +277,7 @@ let combiner bind option_default
   and fullType ft =
     let k ft =
       match Ast.unwrap ft with
-	Ast.Type(_,cv,ty) -> 
+	Ast.Type(_,cv,ty) ->
 	  let lcv = get_option cv_mcode cv in
 	  let lty = typeC ty in
 	  bind lcv lty
@@ -306,11 +286,10 @@ let combiner bind option_default
 	  let lasty = fullType asty in
 	  bind lty lasty
       | Ast.DisjType(types) -> multibind (List.map fullType types)
-      | Ast.OptType(ty) -> fullType ty
-      | Ast.UniqueType(ty) -> fullType ty in
+      | Ast.OptType(ty) -> fullType ty in
     ftfn all_functions k ft
 
-  and function_pointer 
+  and function_pointer
 	(ty, lp1, star, (id : Ast.ident option), rp1, lp2, params, rp2) =
     (* have to put the treatment of the identifier into the right position *)
     let lty = fullType ty in
@@ -335,7 +314,7 @@ let combiner bind option_default
     let k ty =
       match Ast.unwrap ty with
 	Ast.BaseType(ty,strings) -> multibind (List.map string_mcode strings)
-      | Ast.SignedT(sgn,ty) -> 
+      | Ast.SignedT(sgn,ty) ->
 	  let lsgn = sign_mcode sgn in
 	  let lty = get_option typeC ty in
 	  bind lsgn lty
@@ -421,17 +400,21 @@ let combiner bind option_default
 	  let lparams = parameter_dots params in
           let (lcomma,lellipsis) = match va with
             | None -> ([],[])
-            | Some (comma,ellipsis) -> ([string_mcode comma],[string_mcode ellipsis]) in
+            | Some (comma,ellipsis) ->
+		([string_mcode comma],[string_mcode ellipsis]) in
 	  let lrp1 = string_mcode rp1 in
-	  multibind (lfi @ [lname; llp1; lparams] @ lcomma @ lellipsis @ [lrp1])
-      | Ast.MacroDecl(name,lp,args,rp,sem) ->
+	  multibind
+	    (lfi @ [lname; llp1; lparams] @ lcomma @ lellipsis @ [lrp1])
+      | Ast.MacroDecl(stg,name,lp,args,rp,sem) ->
+	  let lstg = get_option storage_mcode stg in
 	  let lname = ident name in
 	  let llp = string_mcode lp in
 	  let largs = expression_dots args in
 	  let lrp = string_mcode rp in
 	  let lsem = string_mcode sem in
-	  multibind [lname; llp; largs; lrp; lsem]
-      | Ast.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+	  multibind [lstg; lname; llp; largs; lrp; lsem]
+      | Ast.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem) ->
+	  let lstg = get_option storage_mcode stg in
 	  let lname = ident name in
 	  let llp = string_mcode lp in
 	  let largs = expression_dots args in
@@ -439,7 +422,7 @@ let combiner bind option_default
 	  let leq = string_mcode eq in
 	  let lini = initialiser ini in
 	  let lsem = string_mcode sem in
-	  multibind [lname; llp; largs; lrp; leq; lini; lsem]
+	  multibind [lstg; lname; llp; largs; lrp; leq; lini; lsem]
       | Ast.TyDecl(ty,sem) ->
 	  let lty = fullType ty in
 	  let lsem = string_mcode sem in
@@ -451,8 +434,7 @@ let combiner bind option_default
 	  let lsem = string_mcode sem in
 	  multibind [lstg; lty; lid; lsem]
       | Ast.DisjDecl(decls) -> multibind (List.map declaration decls)
-      | Ast.OptDecl(decl) -> declaration decl
-      | Ast.UniqueDecl(decl) -> declaration decl in
+      | Ast.OptDecl(decl) -> declaration decl in
     declfn all_functions k d
 
   and annotated_decl d =
@@ -494,15 +476,14 @@ let combiner bind option_default
       | Ast.InitGccExt(designators,eq,ini) ->
 	  let ldesignators = List.map designator designators in
 	  let leq = string_mcode eq in
-	  let lini = initialiser ini in 
+	  let lini = initialiser ini in
 	  multibind (ldesignators @ [leq; lini])
       | Ast.IComma(cm) -> string_mcode cm
       | Ast.Idots(dots,whncode) ->
 	  let ldots = string_mcode dots in
 	  let lwhncode = get_option initialiser whncode in
 	  bind ldots lwhncode
-      | Ast.OptIni(i) -> initialiser i
-      | Ast.UniqueIni(i) -> initialiser i in
+      | Ast.OptIni(i) -> initialiser i in
     initfn all_functions k i
 
   and designator = function
@@ -537,9 +518,7 @@ let combiner bind option_default
 	  bind lp lasexp
       | Ast.PComma(cm) -> string_mcode cm
       | Ast.Pdots(dots) -> string_mcode dots
-      | Ast.Pcircles(dots) -> string_mcode dots
-      | Ast.OptParam(param) -> parameterTypeDef param
-      | Ast.UniqueParam(param) -> parameterTypeDef param in
+      | Ast.OptParam(param) -> parameterTypeDef param in
     paramfn all_functions k p
 
   and rule_elem re =
@@ -604,7 +583,7 @@ let combiner bind option_default
 	  let lexp = expression exp in
 	  let lrp = string_mcode rp in
 	  multibind [lswitch; llp; lexp; lrp]
-      | Ast.Break(br,sem) -> 
+      | Ast.Break(br,sem) ->
 	  let lbr = string_mcode br in
 	  let lsem = string_mcode sem in
 	  bind lbr lsem
@@ -612,7 +591,7 @@ let combiner bind option_default
 	  let lcont = string_mcode cont in
 	  let lsem = string_mcode sem in
 	  bind lcont lsem
-      |	Ast.Label(l,dd) -> 
+      |	Ast.Label(l,dd) ->
 	  let ll = ident l in
 	  let ldd = string_mcode dd in
 	  bind ll ldd
@@ -621,7 +600,7 @@ let combiner bind option_default
 	  let ll = ident l in
 	  let lsem = string_mcode sem in
 	  multibind [lgoto; ll; lsem]
-      | Ast.Return(ret,sem) -> 
+      | Ast.Return(ret,sem) ->
 	  let lret = string_mcode ret in
 	  let lsem = string_mcode sem in
 	  bind lret lsem
@@ -642,8 +621,9 @@ let combiner bind option_default
       | Ast.Exp(exp) -> expression exp
       | Ast.TopExp(exp) -> expression exp
       | Ast.Ty(ty) -> fullType ty
+      | Ast.TopId(ty) -> ident ty
       | Ast.TopInit(init) -> initialiser init
-      |	Ast.Include(inc,name) -> 
+      |	Ast.Include(inc,name) ->
 	  let linc = string_mcode inc in
 	  let lname = inc_file_mcode name in
 	  bind linc lname
@@ -661,7 +641,7 @@ let combiner bind option_default
 	  let lid = ident id in
 	  let lbody = pragmainfo body in
 	  multibind [lprg; lid; lbody]
-      |	Ast.Default(def,colon) -> 
+      |	Ast.Default(def,colon) ->
 	  let ldef = string_mcode def in
 	  let lcolon = string_mcode colon in
 	  bind ldef lcolon
@@ -670,7 +650,12 @@ let combiner bind option_default
 	  let lexp = expression exp in
 	  let lcolon = string_mcode colon in
 	  multibind [lcase; lexp; lcolon]
-      |	Ast.DisjRuleElem(res) -> multibind (List.map rule_elem res) in
+      | Ast.AsRe(re,asre) ->
+	  let re = rule_elem re in
+	  let asre = rule_elem asre in
+	  bind re asre
+      |	Ast.DisjRuleElem(res) ->
+	  multibind (List.map rule_elem res) in
     rulefn all_functions k re
 
   (* not parameterisable, for now *)
@@ -708,12 +693,7 @@ let combiner bind option_default
 	  multibind [llp; lparams; lrp] in
     k p
 
-  and define_param_dots d =
-    let k d =
-      match Ast.unwrap d with
-	Ast.DOTS(l) | Ast.CIRCLES(l) | Ast.STARS(l) ->
-	  multibind (List.map define_param l) in
-    k d
+  and define_param_dots d = multibind (List.map define_param (Ast.unwrap d))
 
   and define_param p =
     let k p =
@@ -721,9 +701,7 @@ let combiner bind option_default
 	Ast.DParam(id) -> ident id
       | Ast.DPComma(comma) -> string_mcode comma
       | Ast.DPdots(d) -> string_mcode d
-      | Ast.DPcircles(c) -> string_mcode c
-      | Ast.OptDParam(dp) -> define_param dp
-      | Ast.UniqueDParam(dp) -> define_param dp in
+      | Ast.OptDParam(dp) -> define_param dp in
     k p
 
   (* discard the result, because the statement is assumed to be already
@@ -759,7 +737,7 @@ let combiner bind option_default
 	  bind lheader lbody
       | Ast.Do(header,body,tail) ->
 	  let lheader = rule_elem header in
-	  let lbody = statement body in 
+	  let lbody = statement body in
 	  let ltail = rule_elem tail in
 	  multibind [lheader; lbody; ltail]
       | Ast.For(header,body,_) ->
@@ -778,13 +756,13 @@ let combiner bind option_default
 	  let lrb = rule_elem rb in
 	  multibind [lheader; llb; ldecls; lcases; lrb]
       | Ast.Atomic(re) ->rule_elem re
-      | Ast.Disj(stmt_dots_list) ->
+      | Ast.Disj(stmt_dots_list) | Ast.Conj(stmt_dots_list) ->
 	  multibind (List.map statement_dots stmt_dots_list)
       | Ast.Nest(starter,stmt_dots,ender,whn,_,_,_) ->
 	  let lstarter = string_mcode starter in
 	  let lstmt_dots = statement_dots stmt_dots in
 	  let lender = string_mcode ender in
-	  let lwhn = multibind 
+	  let lwhn = multibind
 	    (List.map (whencode statement_dots statement) whn) in
 	  multibind [lstarter; lstmt_dots; lender; lwhn]
       | Ast.FunDecl(header,lbrace,body,rbrace,_) ->
@@ -801,13 +779,12 @@ let combiner bind option_default
 	  let lstm = statement stm in
 	  let lasstm = statement asstm in
 	  bind lstm lasstm
-      | Ast.Dots(d,whn,_,_) | Ast.Circles(d,whn,_,_) | Ast.Stars(d,whn,_,_) ->
+      | Ast.Dots(d,whn,_,_) ->
 	  let ld = string_mcode d in
-	  let lwhn = multibind 
+	  let lwhn = multibind
 	    (List.map (whencode statement_dots statement) whn) in
 	  bind ld lwhn
-      | Ast.OptStm(stmt) | Ast.UniqueStm(stmt) ->
-	  statement stmt in
+      | Ast.OptStm(stmt) -> statement stmt in
     stmtfn all_functions k s
 
   and fninfo = function
@@ -836,7 +813,7 @@ let combiner bind option_default
   and exec_code e =
     (* not configurable *)
     match Ast.unwrap e with
-      Ast.ExecEval(colon,id) -> 
+      Ast.ExecEval(colon,id) ->
 	let lcolon = string_mcode colon in
 	let lid = expression id in
 	bind lcolon lid
@@ -953,23 +930,19 @@ type 'cd rcode = rebuilder -> ('cd inout) -> 'cd inout
 
 
 let rebuilder
-    meta_mcode string_mcode const_mcode simpleassign_mcode opassign_mcode fix_mcode unary_mcode
-    arithop_mcode logicalop_mcode cv_mcode sign_mcode struct_mcode storage_mcode
-    inc_file_mcode
+    meta_mcode string_mcode const_mcode simpleassign_mcode opassign_mcode
+    fix_mcode unary_mcode
+    arithop_mcode logicalop_mcode cv_mcode sign_mcode struct_mcode
+    storage_mcode inc_file_mcode
     expdotsfn paramdotsfn stmtdotsfn anndecldotsfn initdotsfn
-    identfn exprfn fragfn fmtfn assignOpfn binaryOpfn ftfn tyfn initfn paramfn declfn 
-    annotated_declfn rulefn stmtfn casefn topfn anyfn =
+    identfn exprfn fragfn fmtfn assignOpfn binaryOpfn ftfn tyfn initfn
+    paramfn declfn annotated_declfn rulefn stmtfn casefn topfn anyfn =
   let get_option f = function
       Some x -> Some (f x)
     | None -> None in
 
   let dotsfn param default all_functions arg =
-    let k d =
-      Ast.rewrap d
-	(match Ast.unwrap d with
-	  Ast.DOTS(l) -> Ast.DOTS(List.map default l)
-	| Ast.CIRCLES(l) -> Ast.CIRCLES(List.map default l)
-	| Ast.STARS(l) -> Ast.STARS(List.map default l)) in
+    let k d = Ast.rewrap d (List.map default (Ast.unwrap d)) in
     param all_functions k arg in
 
   let iddotsfn all_functions k arg = k arg in
@@ -999,8 +972,7 @@ let rebuilder
 	    Ast.MetaLocalFunc(meta_mcode name,constraints,keep,inherited)
 	| Ast.AsIdent(id,asid) -> Ast.AsIdent(ident id,ident asid)
 	| Ast.DisjId(id_list) -> Ast.DisjId(List.map ident id_list)
-	| Ast.OptIdent(id) -> Ast.OptIdent(ident id)
-	| Ast.UniqueIdent(id) -> Ast.UniqueIdent(ident id)) in
+	| Ast.OptIdent(id) -> Ast.OptIdent(ident id)) in
     identfn all_functions k i
 
   and expression e =
@@ -1016,7 +988,7 @@ let rebuilder
 	    Ast.StringConstant(llq, lstr, lrq)
 	| Ast.FunCall(fn,lp,args,rp) ->
 	    let lfn = expression fn in
-	    let llp = string_mcode lp in 
+	    let llp = string_mcode lp in
 	    let largs = expression_dots args in
 	    let lrp = string_mcode rp in
 	    Ast.FunCall(lfn, llp, largs, lrp)
@@ -1026,7 +998,7 @@ let rebuilder
 	    let lright = expression right in
 	    Ast.Assignment(lleft, lop, lright, simple)
 	| Ast.Sequence(left,op,right) ->
-	    let lleft = expression left in 
+	    let lleft = expression left in
 	    let lop = string_mcode op in
 	    let lright = expression right in
 	    Ast.Sequence(lleft, lop, lright)
@@ -1037,15 +1009,15 @@ let rebuilder
 	    let lcolon = string_mcode colon in
 	    let lexp3 = expression exp3 in
 	    Ast.CondExpr(lexp1, lwhy, lexp2, lcolon, lexp3)
-	| Ast.Postfix(exp,op) -> 
+	| Ast.Postfix(exp,op) ->
 	    let lexp = expression exp in
 	    let lop = fix_mcode op in
 	    Ast.Postfix(lexp, lop)
-	| Ast.Infix(exp,op) -> 
+	| Ast.Infix(exp,op) ->
 	    let lexp = expression exp in
 	    let lop = fix_mcode op in
 	    Ast.Infix(lexp, lop)
-	| Ast.Unary(exp,op) -> 
+	| Ast.Unary(exp,op) ->
 	    let lexp = expression exp in
 	    let lop = unary_mcode op in
 	    Ast.Unary(lexp, lop)
@@ -1109,12 +1081,17 @@ let rebuilder
 	    Ast.MetaExpr(meta_mcode name,constraints,keep,ty,form,inherited)
 	| Ast.MetaExprList(name,lenname_inh,keep,inherited) ->
 	    Ast.MetaExprList(meta_mcode name,lenname_inh,keep,inherited)
-	| Ast.AsExpr(exp,asexp) -> 
+	| Ast.AsExpr(exp,asexp) ->
 	    let lexp = expression exp in
 	    let lasexp = expression asexp in
 	    Ast.AsExpr(lexp, lasexp)
+	| Ast.AsSExpr(exp,asstm) -> 
+	    let lexp = expression exp in
+	    let lasstm = rule_elem asstm in
+	    Ast.AsSExpr(lexp, lasstm)
 	| Ast.EComma(cm) -> Ast.EComma(string_mcode cm)
 	| Ast.DisjExpr(exp_list) -> Ast.DisjExpr(List.map expression exp_list)
+	| Ast.ConjExpr(exp_list) -> Ast.ConjExpr(List.map expression exp_list)
 	| Ast.NestExpr(starter,expr_dots,ender,whncode,multi) ->
 	    let lstarter = string_mcode starter in
 	    let lexpr_dots = expression_dots expr_dots in
@@ -1125,16 +1102,7 @@ let rebuilder
 	    let ldots = string_mcode dots in
 	    let lwhncode = get_option expression whncode in
 	    Ast.Edots(ldots, lwhncode)
-	| Ast.Ecircles(dots,whncode) ->
-	    let ldots = string_mcode dots in
-	    let lwhncode = get_option expression whncode in
-	    Ast.Ecircles(ldots, lwhncode)
-	| Ast.Estars(dots,whncode) ->
-	    let ldots = string_mcode dots in
-	    let lwhncode = get_option expression whncode in
-	    Ast.Estars(ldots, lwhncode)
-	| Ast.OptExp(exp) -> Ast.OptExp(expression exp)
-	| Ast.UniqueExp(exp) -> Ast.UniqueExp(expression exp)) in
+	| Ast.OptExp(exp) -> Ast.OptExp(expression exp)) in
     exprfn all_functions k e
 
   and string_fragment e =
@@ -1177,7 +1145,7 @@ let rebuilder
       Ast.rewrap op
         (match Ast.unwrap op with
           Ast.Arith o -> Ast.Arith (arithop_mcode o)
-        | Ast.Logical o -> Ast.Logical (logicalop_mcode o) 
+        | Ast.Logical o -> Ast.Logical (logicalop_mcode o)
         | Ast.MetaBinary (mv,x,y,z) -> Ast.MetaBinary ((meta_mcode mv),x,y,z)
         ) in
     binaryOpfn all_functions k op
@@ -1190,13 +1158,12 @@ let rebuilder
 	    let lcv = get_option cv_mcode cv in
 	    let lty = typeC ty in
 	    Ast.Type (allminus, lcv, lty)
-	| Ast.AsType(ty,asty) -> 
+	| Ast.AsType(ty,asty) ->
 	    let lty = fullType ty in
 	    let lasty = fullType asty in
 	    Ast.AsType(lty, lasty)
 	| Ast.DisjType(types) -> Ast.DisjType(List.map fullType types)
-	| Ast.OptType(ty) -> Ast.OptType(fullType ty)
-	| Ast.UniqueType(ty) -> Ast.UniqueType(fullType ty)) in
+	| Ast.OptType(ty) -> Ast.OptType(fullType ty)) in
     ftfn all_functions k ft
 
   and typeC ty =
@@ -1296,18 +1263,21 @@ let rebuilder
 	    let lparams = parameter_dots params in
             let lva = match va with
               | None -> None
-              | Some (comma,ellipsis) -> Some (string_mcode comma,string_mcode ellipsis) in 
+              | Some (comma,ellipsis) ->
+		  Some (string_mcode comma,string_mcode ellipsis) in
 	    let lrp = string_mcode rp in
 	    let lsem = string_mcode sem in
 	    Ast.FunProto(lfi,lname,llp,lparams,lva,lrp,lsem)
-	| Ast.MacroDecl(name,lp,args,rp,sem) ->
+	| Ast.MacroDecl(stg,name,lp,args,rp,sem) ->
+	    let lstg = get_option storage_mcode stg in
 	    let lname = ident name in
 	    let llp = string_mcode lp in
 	    let largs = expression_dots args in
 	    let lrp = string_mcode rp in
 	    let lsem = string_mcode sem in
-	    Ast.MacroDecl(lname, llp, largs, lrp, lsem)
-	| Ast.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+	    Ast.MacroDecl(lstg, lname, llp, largs, lrp, lsem)
+	| Ast.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem) ->
+	    let lstg = get_option storage_mcode stg in
 	    let lname = ident name in
 	    let llp = string_mcode lp in
 	    let largs = expression_dots args in
@@ -1315,8 +1285,8 @@ let rebuilder
 	    let leq = string_mcode eq in
 	    let lini = initialiser ini in
 	    let lsem = string_mcode sem in
-	    Ast.MacroDeclInit(lname, llp, largs, lrp, leq, lini, lsem)
-	| Ast.TyDecl(ty,sem) -> 
+	    Ast.MacroDeclInit(lstg, lname, llp, largs, lrp, leq, lini, lsem)
+	| Ast.TyDecl(ty,sem) ->
 	    let lty = fullType ty in
 	    let lsem = string_mcode sem in
 	    Ast.TyDecl(lty, lsem)
@@ -1327,8 +1297,7 @@ let rebuilder
 	    let lsem = string_mcode sem in
 	    Ast.Typedef(lstg, lty, lid, lsem)
 	| Ast.DisjDecl(decls) -> Ast.DisjDecl(List.map declaration decls)
-	| Ast.OptDecl(decl) -> Ast.OptDecl(declaration decl)
-	| Ast.UniqueDecl(decl) -> Ast.UniqueDecl(declaration decl)) in
+	| Ast.OptDecl(decl) -> Ast.OptDecl(declaration decl)) in
     declfn all_functions k d
 
   and annotated_decl d =
@@ -1382,8 +1351,7 @@ let rebuilder
 	    let ldots = string_mcode dots in
 	    let lwhncode = get_option initialiser whncode in
 	    Ast.Idots(ldots, lwhncode)
-	| Ast.OptIni(i) -> Ast.OptIni(initialiser i)
-	| Ast.UniqueIni(i) -> Ast.UniqueIni(initialiser i)) in
+	| Ast.OptIni(i) -> Ast.OptIni(initialiser i)) in
     initfn all_functions k i
 
   and designator = function
@@ -1420,9 +1388,7 @@ let rebuilder
 	    Ast.AsParam(lp, lasexp)
 	| Ast.PComma(cm) -> Ast.PComma(string_mcode cm)
 	| Ast.Pdots(dots) -> Ast.Pdots(string_mcode dots)
-	| Ast.Pcircles(dots) -> Ast.Pcircles(string_mcode dots)
-	| Ast.OptParam(param) -> Ast.OptParam(parameterTypeDef param)
-	| Ast.UniqueParam(param) -> Ast.UniqueParam(parameterTypeDef param)) in
+	| Ast.OptParam(param) -> Ast.OptParam(parameterTypeDef param)) in
     paramfn all_functions k p
 
   and rule_elem re =
@@ -1496,7 +1462,7 @@ let rebuilder
 	    let lcont = string_mcode cont in
 	    let lsem = string_mcode sem in
 	    Ast.Continue(lcont, lsem)
-	| Ast.Label(l,dd) -> 
+	| Ast.Label(l,dd) ->
 	    let ll = ident l in
 	    let ldd = string_mcode dd in
 	    Ast.Label(ll, ldd)
@@ -1529,6 +1495,7 @@ let rebuilder
 	| Ast.Exp(exp) -> Ast.Exp(expression exp)
 	| Ast.TopExp(exp) -> Ast.TopExp(expression exp)
 	| Ast.Ty(ty) -> Ast.Ty(fullType ty)
+	| Ast.TopId(id) -> Ast.TopId(ident id)
 	| Ast.TopInit(init) -> Ast.TopInit(initialiser init)
 	| Ast.Include(inc,name) ->
 	    let linc = string_mcode inc in
@@ -1557,6 +1524,10 @@ let rebuilder
 	    let lexp = expression exp in
 	    let lcolon = string_mcode colon in
 	    Ast.Case(lcase, lexp, lcolon)
+	| Ast.AsRe(re,asre) ->
+	    let re = rule_elem re in
+	    let asre = rule_elem asre in
+	    Ast.AsRe(re,asre)
 	| Ast.DisjRuleElem(res) -> Ast.DisjRuleElem(List.map rule_elem res)) in
     rulefn all_functions k re
 
@@ -1598,13 +1569,7 @@ let rebuilder
     k p
 
   and define_param_dots d =
-    let k d =
-      Ast.rewrap d
-	(match Ast.unwrap d with
-	  Ast.DOTS(l) -> Ast.DOTS(List.map define_param l)
-	| Ast.CIRCLES(l) -> Ast.CIRCLES(List.map define_param l)
-	| Ast.STARS(l) -> Ast.STARS(List.map define_param l)) in
-    k d
+    Ast.rewrap d (List.map define_param (Ast.unwrap d))
 
   and define_param p =
     let k p =
@@ -1613,9 +1578,7 @@ let rebuilder
 	  Ast.DParam(id) -> Ast.DParam(ident id)
 	| Ast.DPComma(comma) -> Ast.DPComma(string_mcode comma)
 	| Ast.DPdots(d) -> Ast.DPdots(string_mcode d)
-	| Ast.DPcircles(c) -> Ast.DPcircles(string_mcode c)
-	| Ast.OptDParam(dp) -> Ast.OptDParam(define_param dp)
-	| Ast.UniqueDParam(dp) -> Ast.UniqueDParam(define_param dp)) in
+	| Ast.OptDParam(dp) -> Ast.OptDParam(define_param dp)) in
     k p
 
   and process_bef_aft s =
@@ -1653,7 +1616,7 @@ let rebuilder
 	    Ast.While(lheader, lbody, aft)
 	| Ast.Do(header,body,tail) ->
 	    let lheader = rule_elem header in
-	    let lbody = statement body in 
+	    let lbody = statement body in
 	    let ltail = rule_elem tail in
 	    Ast.Do(lheader, lbody, ltail)
 	| Ast.For(header,body,aft) ->
@@ -1674,6 +1637,8 @@ let rebuilder
 	| Ast.Atomic(re) -> Ast.Atomic(rule_elem re)
 	| Ast.Disj(stmt_dots_list) ->
 	    Ast.Disj (List.map statement_dots stmt_dots_list)
+	| Ast.Conj(stmt_dots_list) ->
+	    Ast.Conj (List.map statement_dots stmt_dots_list)
 	| Ast.Nest(starter,stmt_dots,ender,whn,multi,bef,aft) ->
 	    let lstarter = string_mcode starter in
 	    let lstmt_dots = statement_dots stmt_dots in
@@ -1690,7 +1655,7 @@ let rebuilder
 	    let lheader = rule_elem header in
 	    let lbody = statement_dots body in
 	    Ast.Define(lheader, lbody)
-	| Ast.AsStmt(stm,asstm) -> 
+	| Ast.AsStmt(stm,asstm) ->
 	    let lstm = statement stm in
 	    let lasstm = statement asstm in
 	    Ast.AsStmt(lstm, lasstm)
@@ -1698,16 +1663,7 @@ let rebuilder
 	    let ld = string_mcode d in
 	    let lwhn = List.map (whencode statement_dots statement) whn in
 	    Ast.Dots(ld, lwhn, bef, aft)
-	| Ast.Circles(d,whn,bef,aft) ->
-	    let ld = string_mcode d in
-	    let lwhn = List.map (whencode statement_dots statement) whn in
-	    Ast.Circles(ld, lwhn, bef, aft)
-	| Ast.Stars(d,whn,bef,aft) ->
-	    let ld = string_mcode d in
-	    let lwhn = List.map (whencode statement_dots statement) whn in
-	    Ast.Stars(ld, lwhn, bef, aft)
-	| Ast.OptStm(stmt) -> Ast.OptStm(statement stmt)
-	| Ast.UniqueStm(stmt) -> Ast.UniqueStm(statement stmt)) in
+	| Ast.OptStm(stmt) -> Ast.OptStm(statement stmt)) in
     let s = stmtfn all_functions k s in
     (* better to do this after, in case there is an equality test on the whole
        statement, eg in free_vars.  equality test would require that this
@@ -1828,4 +1784,3 @@ let rebuilder
       rebuilder_define_parameters = define_parameters;
       rebuilder_anything = anything} in
   all_functions
-
