@@ -1,30 +1,9 @@
 (*
- * Copyright 2012-2015, Inria
- * Julia Lawall, Gilles Muller
- * Copyright 2010-2011, INRIA, University of Copenhagen
- * Julia Lawall, Rene Rydhof Hansen, Gilles Muller, Nicolas Palix
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
- * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
- * This file is part of Coccinelle.
- *
- * Coccinelle is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, according to version 2 of the License.
- *
- * Coccinelle is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
- *
- * The authors reserve the right to distribute this or future versions of
- * Coccinelle under other licenses.
+ * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * See copyright.txt in the Coccinelle source code for more information.
+ * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-
-# 0 "./visitor_ast0.ml"
 (* --------------------------------------------------------------------- *)
 (* Generic traversal: rebuilder *)
 
@@ -56,7 +35,7 @@ let visitor mode bind option_default
   let get_option f = function
       Some x -> let (n,e) = f x in (n,Some e)
     | None -> (option_default,None) in
-  let do_disj starter lst mids ender processor rebuilder =
+  let do_disj starter lst mids ender processor rebuilder = (* disj and conj *)
     let (starter_n,starter) = string_mcode starter in
     (* slightly ugly but ensures correct evaluation order. *)
     let (first_n, first) = processor (List.hd lst) in
@@ -72,15 +51,7 @@ let visitor mode bind option_default
     (multibind [starter_n; multibind bind_value;ender_n],
      rebuilder starter lst mids ender) in
   let dotsfn param default all_functions arg =
-    let k d =
-      rewrap d
-	(match Ast0.unwrap d with
-	  Ast0.DOTS(l) ->
-	    let (n,l) = map_split_bind default l in (n,Ast0.DOTS(l))
-	| Ast0.CIRCLES(l) ->
-	    let (n,l) = map_split_bind default l in (n,Ast0.CIRCLES(l))
-	| Ast0.STARS(l) ->
-	    let (n,l) = map_split_bind default l in (n,Ast0.STARS(l))) in
+    let k d = rewrap d (map_split_bind default (Ast0.unwrap d)) in
     param all_functions k arg in
   let iddotsfn all_functions k arg = k arg in
   let strdotsfn all_functions k arg = k arg in
@@ -117,8 +88,6 @@ let visitor mode bind option_default
 		Ast0.DisjId(starter,id_list,mids,ender))
 	| Ast0.OptIdent(id) ->
 	    let (n,id) = ident id in (n,Ast0.OptIdent(id))
-	| Ast0.UniqueIdent(id) ->
-	    let (n,id) = ident id in (n,Ast0.UniqueIdent(id))
 	| Ast0.AsIdent(id,asid) ->
 	    let (id_n,id) = ident id in
 	    let (asid_n,asid) = ident asid in
@@ -248,6 +217,10 @@ let visitor mode bind option_default
 	    do_disj starter expr_list mids ender expression
 	      (fun starter expr_list mids ender ->
 		Ast0.DisjExpr(starter,expr_list,mids,ender))
+	| Ast0.ConjExpr(starter,expr_list,mids,ender) ->
+	    do_disj starter expr_list mids ender expression
+	      (fun starter expr_list mids ender ->
+		Ast0.ConjExpr(starter,expr_list,mids,ender))
 	| Ast0.NestExpr(starter,expr_dots,ender,whencode,multi) ->
 	    let (starter_n,starter) = string_mcode starter in
 	    let (whencode_n, whencode) = whencode_option expression whencode in
@@ -259,24 +232,17 @@ let visitor mode bind option_default
 	    let (dots_n,dots) = string_mcode dots in
 	    let (whencode_n, whencode) = whencode_option expression whencode in
 	    (bind dots_n whencode_n,Ast0.Edots(dots,whencode))
-	| Ast0.Ecircles(dots,whencode) ->
-	    let (dots_n,dots) = string_mcode dots in
-	    let (whencode_n, whencode) = whencode_option expression whencode in
-	    (bind dots_n whencode_n,Ast0.Ecircles(dots,whencode))
-	| Ast0.Estars(dots,whencode) ->
-	    let (dots_n,dots) = string_mcode dots in
-	    let (whencode_n, whencode) = whencode_option expression whencode in
-	    (bind dots_n whencode_n,Ast0.Estars(dots,whencode))
 	| Ast0.OptExp(exp) ->
 	    let (exp_n,exp) = expression exp in
 	    (exp_n,Ast0.OptExp(exp))
-	| Ast0.UniqueExp(exp) ->
-	    let (exp_n,exp) = expression exp in
-	    (exp_n,Ast0.UniqueExp(exp))
 	| Ast0.AsExpr(exp,asexp) ->
 	    let (exp_n,exp) = expression exp in
 	    let (asexp_n,asexp) = expression asexp in
-	    (bind exp_n asexp_n, Ast0.AsExpr(exp,asexp))) in
+	    (bind exp_n asexp_n, Ast0.AsExpr(exp,asexp))
+	| Ast0.AsSExpr(exp,asstm) ->
+	    let (exp_n,exp) = expression exp in
+	    let (asstm_n,asstm) = statement asstm in
+	    (bind exp_n asstm_n, Ast0.AsSExpr(exp,asstm))) in
     exprfn all_functions k e
 
   and string_fragment e =
@@ -322,7 +288,7 @@ let visitor mode bind option_default
             let (o_n,o) = opAssign_mcode o in
             (o_n, Ast0.OpAssign o)
         | Ast0.MetaAssign (name, c, pure) ->
-          let (name_n,name) = meta_mcode name in 
+          let (name_n,name) = meta_mcode name in
           (name_n,Ast0.MetaAssign(name,c, pure))) in
     assignOpfn all_functions k e
 
@@ -337,9 +303,9 @@ let visitor mode bind option_default
             let (o_n, o) = logicalOp_mcode o in
             (o_n, Ast0.Logical o)
         | Ast0.MetaBinary (name, c, pure) ->
-          let (name_n,name) = meta_mcode name in 
+          let (name_n,name) = meta_mcode name in
           (name_n,Ast0.MetaBinary(name, c, pure))) in
-    binaryOpfn all_functions k e  
+    binaryOpfn all_functions k e
 
   and typeC t =
     let k t =
@@ -366,9 +332,9 @@ let visitor mode bind option_default
 	    let (star_n,star) = string_mcode star in
 	    (bind ty_n star_n, Ast0.Pointer(ty,star))
 	| Ast0.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) ->
-	    let (t,id) = 
+	    let (t,id) =
               function_pointer (ty,lp1,star,None,rp1,lp2,params,rp2) in t
-	| Ast0.Array(ty,lb,size,rb) -> 
+	| Ast0.Array(ty,lb,size,rb) ->
             let (t,id) = array_type (ty,None,lb,size,rb) in t
 	| Ast0.Decimal(dec,lp,length,comma,precision_opt,rp) ->
 	    let (dec_n,dec) = string_mcode dec in
@@ -413,8 +379,6 @@ let visitor mode bind option_default
 		Ast0.DisjType(starter,types,mids,ender))
 	| Ast0.OptType(ty) ->
 	    let (ty_n,ty) = typeC ty in (ty_n, Ast0.OptType(ty))
-	| Ast0.UniqueType(ty) ->
-	    let (ty_n,ty) = typeC ty in (ty_n, Ast0.UniqueType(ty))
 	| Ast0.AsType(ty,asty) ->
 	    let (ty_n,ty) = typeC ty in
 	    let (asty_n,asty) = typeC asty in
@@ -427,7 +391,7 @@ let visitor mode bind option_default
     let (ty_n,ty) = typeC ty in
     let (lp1_n,lp1) = string_mcode lp1 in
     let (star_n,star) = string_mcode star in
-    let (idl,idu) = (match id with 
+    let (idl,idu) = (match id with
       | Some a -> let (b,c) = ident a in ([b],Some c)
       | None -> ([],None)) in
     let (rp1_n,rp1) = string_mcode rp1 in
@@ -441,7 +405,7 @@ let visitor mode bind option_default
   (* returns ((bind value,original value),id) since id may have been updated*)
   and array_type (ty,(id : Ast0.ident option),lb,size,rb) =
     let (ty_n,ty) = typeC ty in
-    let (idl,idu) = (match id with 
+    let (idl,idu) = (match id with
       | Some a -> let (b,c) = ident a in ([b],Some c)
       | None -> ([],None)) in
     let (lb_n,lb) = string_mcode lb in
@@ -453,7 +417,7 @@ let visitor mode bind option_default
   and named_type ty id =
     match Ast0.unwrap ty with
       Ast0.FunctionPointer(rty,lp1,star,rp1,lp2,params,rp2) ->
-	let (tyres, idn) = 
+	let (tyres, idn) =
           function_pointer (rty,lp1,star,Some id,rp1,lp2,params,rp2) in
         let idn = match idn with Some i -> i | None -> failwith "Impossible" in
 	(rewrap ty tyres, idn)
@@ -506,15 +470,17 @@ let visitor mode bind option_default
 	    let (sem_n,sem) = string_mcode sem in
 	    (multibind [fi_n;name_n;lp1_n;params_n;va_n;rp1_n;sem_n],
 	     Ast0.FunProto(fi,name,lp1,params,va,rp1,sem))
-	| Ast0.MacroDecl(name,lp,args,rp,sem) ->
+	| Ast0.MacroDecl(stg,name,lp,args,rp,sem) ->
+	    let (stg_n,stg) = get_option storage_mcode stg in
 	    let (name_n,name) = ident name in
 	    let (lp_n,lp) = string_mcode lp in
 	    let (args_n,args) = expression_dots args in
 	    let (rp_n,rp) = string_mcode rp in
 	    let (sem_n,sem) = string_mcode sem in
-	    (multibind [name_n;lp_n;args_n;rp_n;sem_n],
-	     Ast0.MacroDecl(name,lp,args,rp,sem))
-	| Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem) ->
+	    (multibind [stg_n;name_n;lp_n;args_n;rp_n;sem_n],
+	     Ast0.MacroDecl(stg,name,lp,args,rp,sem))
+	| Ast0.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem) ->
+	    let (stg_n,stg) = get_option storage_mcode stg in
 	    let (name_n,name) = ident name in
 	    let (lp_n,lp) = string_mcode lp in
 	    let (args_n,args) = expression_dots args in
@@ -522,8 +488,8 @@ let visitor mode bind option_default
 	    let (eq_n,eq) = string_mcode eq in
 	    let (ini_n,ini) = initialiser ini in
 	    let (sem_n,sem) = string_mcode sem in
-	    (multibind [name_n;lp_n;args_n;rp_n;eq_n;ini_n;sem_n],
-	     Ast0.MacroDeclInit(name,lp,args,rp,eq,ini,sem))
+	    (multibind [stg_n;name_n;lp_n;args_n;rp_n;eq_n;ini_n;sem_n],
+	     Ast0.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem))
 	| Ast0.TyDecl(ty,sem) ->
 	    let (ty_n,ty) = typeC ty in
 	    let (sem_n,sem) = string_mcode sem in
@@ -541,7 +507,7 @@ let visitor mode bind option_default
 	| Ast0.Ddots(dots,whencode) ->
 	    let (dots_n,dots) = string_mcode dots in
 	    let (whencode_n, whencode) = match whencode with
-              | Some (a,b,c) -> 
+              | Some (a,b,c) ->
                   let (_,a2) = string_mcode a in
                   let (_,b2) = string_mcode b in
                   let (c1,c2) = declaration c in (c1, Some (a2,b2,c2))
@@ -549,8 +515,6 @@ let visitor mode bind option_default
 	    (bind dots_n whencode_n, Ast0.Ddots(dots,whencode))
 	| Ast0.OptDecl(decl) ->
 	    let (n,decl) = declaration decl in (n,Ast0.OptDecl(decl))
-	| Ast0.UniqueDecl(decl) ->
-	    let (n,decl) = declaration decl in (n,Ast0.UniqueDecl(decl))
 	| Ast0.AsDecl(decl,asdecl) ->
 	    let (decl_n,decl) = declaration decl in
 	    let (asdecl_n,asdecl) = declaration asdecl in
@@ -591,7 +555,7 @@ let visitor mode bind option_default
 	| Ast0.Idots(d,whencode) ->
 	    let (d_n,d) = string_mcode d in
 	    let (whencode_n, whencode) = match whencode with
-	      | Some (a,b,c) -> 
+	      | Some (a,b,c) ->
 		  let (_,a2) = string_mcode a in
 		  let (_,b2) = string_mcode b in
 		  let (c1,c2) = initialiser c in (c1, Some (a2,b2,c2))
@@ -599,8 +563,6 @@ let visitor mode bind option_default
 	    (bind d_n whencode_n, Ast0.Idots(d,whencode))
 	| Ast0.OptIni(i) ->
 	    let (n,i) = initialiser i in (n,Ast0.OptIni(i))
-	| Ast0.UniqueIni(i) ->
-	    let (n,i) = initialiser i in (n,Ast0.UniqueIni(i))
 	| Ast0.AsInit(ini,asini) ->
 	    let (ini_n,ini) = initialiser ini in
 	    let (asini_n,asini) = initialiser asini in
@@ -652,13 +614,9 @@ let visitor mode bind option_default
 	    let (n,cm) = string_mcode cm in (n,Ast0.PComma(cm))
 	| Ast0.Pdots(dots) ->
 	    let (n,dots) = string_mcode dots in (n,Ast0.Pdots(dots))
-	| Ast0.Pcircles(dots) ->
-	    let (n,dots) = string_mcode dots in (n,Ast0.Pcircles(dots))
 	| Ast0.OptParam(param) ->
-	    let (n,param) = parameterTypeDef param in (n,Ast0.OptParam(param))
-	| Ast0.UniqueParam(param) ->
 	    let (n,param) = parameterTypeDef param in
-	    (n,Ast0.UniqueParam(param))) in
+	    (n,Ast0.OptParam(param))) in
     paramfn all_functions k p
 
   (* not done for combiner, because the statement is assumed to be already
@@ -819,6 +777,10 @@ let visitor mode bind option_default
 	    do_disj starter statement_dots_list mids ender statement_dots
 	      (fun starter statement_dots_list mids ender ->
 		Ast0.Disj(starter,statement_dots_list,mids,ender))
+	| Ast0.Conj(starter,statement_dots_list,mids,ender) ->
+	    do_disj starter statement_dots_list mids ender statement_dots
+	      (fun starter statement_dots_list mids ender ->
+		Ast0.Conj(starter,statement_dots_list,mids,ender))
 	| Ast0.Nest(starter,stmt_dots,ender,whn,multi) ->
 	    let (starter_n,starter) = string_mcode starter in
 	    let (stmt_dots_n,stmt_dots) = statement_dots stmt_dots in
@@ -836,6 +798,9 @@ let visitor mode bind option_default
 	| Ast0.Ty(ty) ->
 	    let (ty_n,ty) = typeC ty in
 	    (ty_n,Ast0.Ty(ty))
+	| Ast0.TopId(id) ->
+	    let (id_n,id) = ident id in
+	    (id_n,Ast0.TopId(id))
 	| Ast0.TopInit(init) ->
 	    let (init_n,init) = initialiser init in
 	    (init_n,Ast0.TopInit(init))
@@ -844,16 +809,6 @@ let visitor mode bind option_default
 	    let (whn_n,whn) =
 	      map_split_bind (whencode statement_dots statement) whn in
 	    (bind d_n whn_n, Ast0.Dots(d,whn))
-	| Ast0.Circles(d,whn) ->
-	    let (d_n,d) = string_mcode d in
-	    let (whn_n,whn) =
-	      map_split_bind (whencode statement_dots statement) whn in
-	    (bind d_n whn_n, Ast0.Circles(d,whn))
-	| Ast0.Stars(d,whn) ->
-	    let (d_n,d) = string_mcode d in
-	    let (whn_n,whn) =
-	      map_split_bind (whencode statement_dots statement) whn in
-	    (bind d_n whn_n, Ast0.Stars(d,whn))
 	| Ast0.Include(inc,name) ->
 	    let (inc_n,inc) = string_mcode inc in
 	    let (name_n,name) = inc_mcode name in
@@ -876,8 +831,6 @@ let visitor mode bind option_default
 	    (multibind [prg_n;id_n;body_n],Ast0.Pragma(prg,id,body))
 	| Ast0.OptStm(re) ->
 	    let (re_n,re) = statement re in (re_n,Ast0.OptStm(re))
-	| Ast0.UniqueStm(re) ->
-	    let (re_n,re) = statement re in (re_n,Ast0.UniqueStm(re))
 	| Ast0.AsStmt(stm,asstm) ->
 	    let (stm_n,stm) = statement stm in
 	    let (asstm_n,asstm) = statement asstm in
@@ -930,16 +883,7 @@ let visitor mode bind option_default
     k p
 
   and define_param_dots d =
-    let k d =
-      rewrap d
-	(match Ast0.unwrap d with
-	  Ast0.DOTS(l) ->
-	    let (n,l) = map_split_bind define_param l in (n,Ast0.DOTS(l))
-	| Ast0.CIRCLES(l) ->
-	    let (n,l) = map_split_bind define_param l in (n,Ast0.CIRCLES(l))
-	| Ast0.STARS(l) ->
-	    let (n,l) = map_split_bind define_param l in (n,Ast0.STARS(l))) in
-    k d
+    rewrap d (map_split_bind define_param (Ast0.unwrap d))
 
   and define_param p =
     let k p =
@@ -950,12 +894,8 @@ let visitor mode bind option_default
 	    let (n,comma) = string_mcode comma in (n,Ast0.DPComma(comma))
 	| Ast0.DPdots(d) ->
 	    let (n,d) = string_mcode d in (n,Ast0.DPdots(d))
-	| Ast0.DPcircles(c) ->
-	    let (n,c) = string_mcode c in (n,Ast0.DPcircles(c))
 	| Ast0.OptDParam(dp) ->
-	    let (n,dp) = define_param dp in (n,Ast0.OptDParam(dp))
-	| Ast0.UniqueDParam(dp) ->
-	    let (n,dp) = define_param dp in (n,Ast0.UniqueDParam(dp))) in
+	    let (n,dp) = define_param dp in (n,Ast0.OptDParam(dp))) in
     k p
 
   and fninfo = function
@@ -974,7 +914,7 @@ let visitor mode bind option_default
 	let (_,w) = string_mcode w in
 	let (_,e) = string_mcode e in
 	let (n,a) = notfn a in (n,Ast0.WhenNot(w,e,a))
-    | Ast0.WhenAlways (w,e,a) -> 
+    | Ast0.WhenAlways (w,e,a) ->
 	let (_,w) = string_mcode w in
 	let (_,e) = string_mcode e in
 	let (n,a) = alwaysfn a in (n,Ast0.WhenAlways(w,e,a))
@@ -993,7 +933,7 @@ let visitor mode bind option_default
   (* for whencodes that do not have any of the above modifiers
    * returns (the new whencode expression, the updated whencode) *)
   and whencode_option cfn = function
-    | Some (a,b,c) -> 
+    | Some (a,b,c) ->
 	let (_,a2) = string_mcode a in
 	let (_,b2) = string_mcode b in
 	let (c1,c2) = cfn c in (c1, Some (a2,b2,c2))
@@ -1443,12 +1383,13 @@ let rebuilder functions =
     (fun r k e -> ((),functions.VT0.rebuilder_topfn (dz r) (xk k) e)))
 
 let flat_rebuilder
-    meta_mcode string_mcode const_mcode simpleAssign_mcode opAssign_mcode fix_mcode unary_mcode
-    arithOp_mcode logicalOp_mcode cv_mcode sign_mcode struct_mcode storage_mcode
-    inc_mcode
+    meta_mcode string_mcode const_mcode simpleAssign_mcode opAssign_mcode
+    fix_mcode unary_mcode
+    arithOp_mcode logicalOp_mcode cv_mcode sign_mcode struct_mcode
+    storage_mcode inc_mcode
     dotsexprfn dotsinitfn dotsparamfn dotsstmtfn dotsdeclfn dotscasefn
-    identfn exprfn assignOpfn arithOpfn tyfn initfn paramfn declfn stmtfn forinfofn casefn
-    string_fragmentfn topfn =
+    identfn exprfn assignOpfn arithOpfn tyfn initfn paramfn declfn stmtfn
+    forinfofn casefn string_fragmentfn topfn =
   let dz = rebuilder_dz in
   let xk k e = let (_,e) = k e in e in
   rebuilder_dz
@@ -1571,4 +1512,3 @@ let combiner_rebuilder bind option_default functions =
     functions.VT0.combiner_rebuilder_casefn
     functions.VT0.combiner_rebuilder_string_fragmentfn
     functions.VT0.combiner_rebuilder_topfn
-
